/* Crimson2 Mud Server
 * All source written/copyright Ryan Haksi 1995 *
 * This source code is proprietary. Use in whole or in part without
 * explicity permission by the author is strictly prohibited
 *
 * Current email address(es): cryogen@infoserve.net
 * Phone number: (604) 591-5295
 *
 * C4 Script Language written/copyright Cam Lesiuk 1995
 * Email: clesiuk@engr.uvic.ca
 */

/* Inventory related commands */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#include "crimson2.h"
#include "macro.h"
#include "queue.h"
#include "log.h"
#include "str.h"
#include "ini.h"
#include "extra.h"
#include "property.h"
#include "file.h"
#include "thing.h"
#include "index.h"
#include "area.h"
#include "reset.h"
#include "code.h"
#include "exit.h"
#include "world.h"
#include "edit.h"
#include "history.h"
#include "socket.h"
#include "send.h"
#include "base.h"
#include "affect.h"
#include "effect.h"
#include "object.h"
#include "board.h"
#include "char.h"
#include "fight.h"
#include "player.h"
#include "skill.h"
#include "parse.h"
#include "cmd_move.h"
#include "cmd_inv.h"
#include "cmd_wld.h"
#include "sound.h"


/* Each type of object can be worn in up to 3 different locations
   alter the table to taste */

struct WearListType wearList[] = {
  { "NOWEAR",       0, {   0,0,0 } },
  { "HEAD",         0, {   EQU_HEAD,
                                  0,
                                  0 } },
  { "NECK",         0, {   EQU_NECK_1,
                           EQU_NECK_2,
                           0 } },

  { "LARGE-NECK",   0, {   EQU_NECK_1|EQU_NECK_2,
                           0,
                           0 } },

  { "SHIRT",        0, {   EQU_TORSO,
                           0,
                           0 } },

  { "UPPER-BODY",   0, {   EQU_TORSO|EQU_ARM_R|EQU_ARM_L,
                           0,
                           0 } },

  { "ARM",          0, {   EQU_ARM_R,
                           EQU_ARM_L,
                           0 } },

  { "BOTH-ARMS",    0, {   EQU_ARM_R|EQU_ARM_L,
                           0,
                           0 } },

  { "WRIST",        0, {   EQU_WRIST_R,
                           EQU_WRIST_L,
                           0 } },

  { "BOTH-WRISTS",  0, {   EQU_WRIST_R|EQU_WRIST_L,
                           0,
                           0 } },

  { "HELD",         0, {   EQU_HELD_R,
                           EQU_HELD_L,
                           EQU_TAILGRIP } },

  { "TWO-HANDED",   0, {   EQU_HELD_R|EQU_HELD_L,
                           0,
                           0 } },

  { "FINGER",       0, {   EQU_FINGER_R,
                           EQU_FINGER_L,
                           0 } },

  { "WAIST",        0, {   EQU_WAIST,
                           0,
                           0 } },

  { "LEGS",         0, {   EQU_LEGS,
                           0,
                           0 } },

  { "FEET",         0, {   EQU_FEET,
                           0,
                           0 } },

  { "OVERBODY",     0, {   EQU_BODY,
                           0,
                           0 } },

  { "EITHER-HAND",  0, {   EQU_HAND_R,
                           EQU_HAND_L,
                           0 } },

  { "BOTH-HANDS",   0, {   EQU_HAND_R|EQU_HAND_L,
                           0,
                           0 } },

  { "R-HAND",       0, {   EQU_HAND_R,
                           0,
                           0 } },

  { "L-HAND",       0, {   EQU_HAND_L,
                           0,
                           0 } },

  { "MECH-ARM",     0, {   EQU_ARM_R|EQU_HELD_R,
                           EQU_ARM_L|EQU_HELD_L,
                           0 } },

  { "TAIL",         0, {   EQU_TAIL,
                           0,
                           0 } },

  { "",0,{ 0,0,0 } }
};


/* internal equipment position flags */
BYTE *equipList[] = {
  "HEAD",
  "NECK",
  "NECK",
  "TORSO",
  "OVER-BODY",
  "RIGHT-ARM",
  "LEFT-ARM",
  "RIGHT-WRIST",
  "LEFT-WRIST",
  "RIGHT-HAND",
  "LEFT-HAND",
  "HELD-IN-RIGHT-HAND",
  "HELD-IN-LEFT-HAND",
  "FINGER-ON-RIGHT-HAND",
  "FINGER-ON-LEFT-HAND",
  "WAIST",
  "LEGS",
  "FEET",
  "TAIL",
  "GRIPPED-IN-TAIL",
  ""
};

LWORD invAllowDonate;
LWORD invDonateRoom;
INDEX invIndex;

void InvIndexClear(void) {
  IndexClear(&invIndex);
}

BYTE InvIndexIsItem(void *thing) {
  if (IndexFind(&invIndex, thing, NULL))
    return TRUE;
  return FALSE;
}

void InvIndexAddItem(void *thing) {
  IndexAppend(&invIndex, thing);
}

void InvIndexDelete(void *thing) {
  if (InvIndexIsItem(thing))
    IndexDelete(&invIndex, thing, NULL);
}

/*

    I need to limit the number of items in donations. I also need to be able to take out the
    cheapest things to put in better things as requested and limit the number of items
    per item type.
    these are suppost routines to enable this.
    since I cannot maintain a count across calls
    ( I dont have a hook for get/purge from the donations room, hmm maybe should add ? )
    I must do it on the fly. so I will build a listing of items and if there are too many
    get rid of some as needed

    Of course this wouldnt be necessary if we had reference counted things :(
    would save a lot of memory too sigh
    and all ops would be a lot easier to do also.
*/

#define INV_MAX_THING 10
#define INV_MAX_ITEM  20
typedef struct {
  LWORD  oVirtual;
  BYTE   count;
  LWORD  oValue;
  THING* thing[INV_MAX_THING];
}invDonateItem;

invDonateItem invDonateList[INV_MAX_ITEM];

void CmdInvInit( void ) {
  BYTE buf[256];
  THING* dest;
  invAllowDonate = INILWordRead("crimson2.ini", "pAllowDonate", 0);
  invDonateRoom  = INILWordRead("crimson2.ini", "pDonateRoom", -1 );
  if( invDonateRoom == -1 ) {
    invAllowDonate = 0;
  }
  if( invAllowDonate == 0 ) {
    strcpy( buf,"CmdInvInit: Donate disabled.\n" );
  } else {
    dest = WorldOf( invDonateRoom );
    if( !dest ) {
      sprintf( buf,"CmdInvInit:Donation Room#%li Invalid, Donate Disabled!\n", invDonateRoom );
      invAllowDonate = 0;
    } else {
      sprintf( buf,"CmdInvInit:Donations sent to Room# %li\n", invDonateRoom );
    }
  }
  Log(LOG_BOOT, buf );
  memset( &invDonateList,0, sizeof(invDonateList));
  IndexInit(&invIndex,  2048, "invIndex",  0);
}

LWORD InvDonateIsThing( THING* data ) {
  LWORD i;
  /* check if the thing is valid */
  if( !data || data->tType != TTYPE_OBJ ) {
    return FALSE;
  }
  for( i = 0; i < INV_MAX_ITEM; i++ ) {
    /* try to locate in the list */
    if( invDonateList[i].count > 0 &&
        invDonateList[i].oVirtual == Obj(data)->oTemplate->oVirtual ) {
      /* ok found set return code and return success */
      return i;
    }
  }
  /* thing not found in list */
  return -1;
}

BYTE InvDonateInsertItem( const BYTE item, THING* data ) {
  LWORD i;
  /* check if the thing is valid */
  if( !data || data->tType != TTYPE_OBJ ) {
    return FALSE;
  }
  /* check if index is valid */
  if( item >= INV_MAX_ITEM )
    return FALSE;
  for( i = 0; i < invDonateList[i].count; i++ ) {
    if( invDonateList[item].thing[i] ) {
      ThingFrom( invDonateList[item].thing[i] );
      THINGFREE( invDonateList[item].thing[i] );
      invDonateList[item].thing[i] = NULL;
    }
  }
  /* reset the item */
  invDonateList[item].count = 1;
  invDonateList[item].oVirtual = Obj(data)->oTemplate->oVirtual;
  invDonateList[item].oValue   = Obj(data)->oTemplate->oValue;
  invDonateList[item].thing[0] = data;
  /* return success */
  return TRUE;
}

BYTE InvDonateAddThing( const BYTE item, THING* data ) {
  /* check if the thing is valid */
  if( !data || data->tType != TTYPE_OBJ ) {
    return FALSE;
  }
  /* check if index is valid */
  if( item >= INV_MAX_ITEM )
    return FALSE;
  /* check if virtual matches should always but check anyway */
  if( invDonateList[item].oVirtual != Obj(data)->oTemplate->oVirtual ) {
    return FALSE;
  }
  /* check thing count */
  if( invDonateList[item].count >= INV_MAX_THING ) {
    return FALSE;
  }
  /* all good increment count */
  invDonateList[item].thing[invDonateList[item].count] = data;
  invDonateList[item].count++;
  /* all good return success */
  return TRUE;
}

LWORD InvDonateGetCheapest( LWORD ItemValue ) {
  LWORD i;
  LWORD cheap = -1;
  LWORD rc = -1;
  for( i = 0; i < INV_MAX_ITEM; i++ ) {
    if( invDonateList[i].count == 0 ) { /* if we have an empty spot just fill it  */
      return i;
    }
    if( cheap == -1 || cheap > invDonateList[i].oValue ) {
      cheap = invDonateList[i].oValue;
      rc = i;
    }
  }
  /* if this new item is worth more then return success  */
  if( ItemValue > cheap )
    return rc;
  return -1;
}

void InvDonateMarkThing( THING* thing ) {
  THING *search = thing->tContain;
  THING *next;
  for( ;search; search = next ) {
    next = search->tNext;
    if( search->tContain ) {
      InvDonateMarkThing( search );
    }
    BITSET(Obj( search )->oAct, OA_DONATED);
  }
}

BYTE InvDonateCheckAdd( THING* data ) {
  LWORD rc;
  LWORD value;
  /* check if the thing is valid */
  if( !data || data->tType != TTYPE_OBJ ) {
    return FALSE;
  }
  if( ( Obj(data)->oTemplate->oVirtual == OVIRTUAL_CORPSE )&&
      ( BIT(OBJECTGETFIELD(data, OF_CONTAINER_CFLAG),OCF_PLAYERCORPSE)  ) ) {
    value = 999999999;
  } else {
    value = Obj(data)->oTemplate->oValue;
  }
  /* first do we have it in the list */
  rc = InvDonateIsThing( data );
  if( rc == -1 ) {
    /* not in the list */
    rc = InvDonateGetCheapest( value  );
    if( rc != -1 ) {
      return InvDonateInsertItem( rc, data );
    }
    return FALSE;
  }
  /* in the list let addthing handle it from here */
  return InvDonateAddThing( rc, data );
}

CMDPROC(CmdDonate) { /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  BYTE   buffer[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  LWORD  count;
  LWORD  i;
  LWORD  oVirtual;
  THING  *found; /* found item */
  THING  *dest;  /* dest room */
  THING  *next;  /* used when looping in case we trash found */
  THING  *work;

  if( !invAllowDonate ) {
    SendAction("^bIm sorry but the SysAdmin has disabled donate\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  dest = WorldOf( invDonateRoom ) ;
  if( !dest ) {
    invAllowDonate = FALSE;
    SendAction("^bIm sorry but the SysAdmin has disabled donate\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    sprintf( buffer, "Invalid Donations Room# %li! Possible memory corruption!", invDonateRoom );
    Log(LOG_ERROR, buffer );
    /* now checking this on boot so shouldnt ever get this but to be safe */
    return;
  }
  work = Base(thing)->bInside;
  while( work && work->tType != TTYPE_WLD ){
    work = Base(work)->bInside;
  }
  if( !work || work->tType != TTYPE_WLD ) {
    invAllowDonate = FALSE;
    SendAction("^bIm sorry but the SysAdmin has disabled donate\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    sprintf( buffer, "CmdDonate: Invalid room for Player %s ( NULL or not TTYPE_WLD ) donate disabled!", thing->tSDesc->sText );
    Log(LOG_ERROR, buffer );
    /* should NEVER get this! */
    return;
  }
  /*
     check for area/wld nodonate flags exit if found with message
  */
  if( BIT(areaList[Wld(work)->wArea].aResetFlag, RF_NODONATE ) ||
      BIT(Wld(Base(work))->wFlag, WF_NODONATE ) ) {
    SendAction("^bYou cant seem to contact Good Will all communications are out!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  /*
    check for if they are in an unopened area.
    I think thingfrom would take care of this but do it here anyway to be safe
    exit if so with message
  */
  if( BIT(areaList[Wld(work)->wArea].aResetFlag, RF_NOENTER  ) ||
      BIT(areaList[Wld(work)->wArea].aResetFlag, RF_NOTAKE   ) ||
      BIT(areaList[Wld(work)->wArea].aResetFlag, RF_NOGAIN   ) ||
      BIT(areaList[Wld(work)->wArea].aResetFlag, RF_NOMONEY  ) ||
      BIT(areaList[Wld(work)->wArea].aResetFlag, RF_AREAISCLOSED ) ) {
    /* reset flags here  */
    SendAction("^bThis area is not open yet!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  /* ok past things why we cant do this at all onwards */
  /*
     we need to build us a list of items in the donations room first now
     to check against
  */
  InvIndexClear();
  memset( &invDonateList,0, sizeof(invDonateList));
  for( work = dest->tContain; work; work = next ) {
    next = work->tNext;
    if( work->tType == TTYPE_OBJ ) {
      if( !InvDonateCheckAdd( work ) ) {
        ThingFrom( work );
        THINGFREE( work );
      }
    }
  }

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJINV, &srcOffset);
  if (!found) {
    /* nothing found send em a message saying so */
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    SendHint("^;HINT: try donate <Object Name Here>.\n", thing );
    return;
  }
  /*
     since thingfind could conceivably crash on us in the event we delete one of these
     inbetween calls we must build a list first before we can safely delete anything
  */
  do {
    /*
     add to index if ok,
    */
    if( found && found->tType == TTYPE_OBJ ) {
      /* check item for flags why we cant donate it, not deleting so ok to do here */
      if( Obj(found)->oTemplate->oType == OTYPE_EXIT ||
          Obj(found)->oTemplate->oType == OTYPE_VEHICLE ||
          Base(found)->bWeight < 0 ) {
        SendAction("^bYou cant donate that!!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      } else if( Obj(found)->oEquip ) {
        /* is it equipped */
        SendAction("^bTry Unequiping it FIRST!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      } else if( BIT(Obj(found)->oAct, OA_NODROP) ||
                 BIT(Obj(found)->oAct, OA_NODONATE) ) {
        /* object flags */
        SendAction("^bYou cant seem to get a lock on that!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      } else {
        /*
           ok take from room and add to index
        */
        ThingFrom( found );
        InvIndexAddItem( found );
      }
    }
    found = CharThingFind(thing, srcKey, -1, thing, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0)
      srcNum--;
  } while (found && srcNum!=0);

  /*
     ok done adding to the list, now we loop through and handle each one in turn
     it is now safe to delete as needed
  */
  while( invIndex.iNum > 0 ) {
    /* for messages donates [x] blah ect. must maintain a count for this */
    count = 0;
    work = invIndex.iThing[0];
    /* our compare vnum */
    oVirtual = Obj(work)->oTemplate->oVirtual;
    /* just count by vnum first so we can send our messages */
    for( i = 0; i < invIndex.iNum ; i++ ) {
      found = invIndex.iThing[i];
      if( oVirtual == Obj(found)->oTemplate->oVirtual ) {
        count++;
      }
    }
    if( count == 1 ) {
      SendAction("^byou put a ^yLocator^g on ^c$N^b, the air Ionizes and in a -ZAP- it is gone.^g\n", thing, work, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^b$n puts a ^yLocator^g on ^c$N^b, the air Ionizes and in a -ZAP- it is gone.^g\n", thing, work, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    } else if( count > 1 ) {
      sprintf( buffer, "^byou put a ^yLocator^g on ^c[%li] $N^b,"
               " the air Ionizes and in a -ZAP- they are gone.^g\n",count );
      SendAction(buffer, thing, work, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      sprintf( buffer, "^b$n put's a ^yLocator^g on ^c[%li] $N^b,"
               " the air Ionizes and in a -ZAP- they are gone.^g\n",count );
      SendAction(buffer, thing, work, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }
    /*
    now actually move/delete as needed, get count for messages below
    */
    count = 0;
    for( i = 0; i < invIndex.iNum; i++ ) {
      if( oVirtual == Obj(invIndex.iThing[i])->oTemplate->oVirtual ) {
        work = invIndex.iThing[i];
        InvIndexDelete(work);
        if( !InvDonateCheckAdd( work  ) ) {
          THINGFREE( work );
        } else {
          /* mark any objects contained within this one */
          InvDonateMarkThing( work );
          /* mark this one and send it on its way */
          BITSET(Obj( work )->oAct, OA_DONATED);
          ThingTo( work , dest );
          count++;
          /* save work for sending arrive messages */
          work = invIndex.iThing[i];
        }
        i--; /* decrement counter since we removed one */
      }
    }
    /*
    might not be any that arrived
    */
    if( count == 1 ) {
      SendAction("^bThe air Ionizes and in a -ZAP- ^c$N^b Appears from thin air^g\n", dest, work, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    } else if( count > 1 ) {
      sprintf( buffer,
              "^bThe air Ionizes and in a -ZAP- [%li] ^c$N^b Appear from thin air^g\n",
               count );
      SendAction(buffer, dest, work, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }
  }
  InvIndexClear();
}

CMDPROC(CmdGet) {     /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  BYTE   buf[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found;
  THING *compareThing;
  THING *search;
  FLAG   flag;
  THING *world;
  LWORD count;
  LWORD i;
  LWORD j;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  if (*dstKey) { /* search a target */
    search = CharThingFind(thing, dstKey, -1, thing, TF_OBJ, &dstOffset);
    if (!search)
      search = CharThingFind(thing, dstKey, -1, Base(thing)->bInside, TF_OBJ, &dstOffset);
    if (!search) {
      SendAction("^bThere doesnt appear to be anything like that around to look in\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  } else  /* search the room */
    search = Base(thing)->bInside;

  found = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything to get\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* Do the search, build a list of targets */
  /* Cryogen: Note that a maximum limit to the loop would probably be a good idea here */
  InvIndexClear();
  while (found && srcNum!=0) {
    if (found->tType==TTYPE_OBJ && (Base(found)->bWeight<0)) {
      /* cant grab an item with negative weight or NOTAKE item */
      SendAction("^bYou ^ccant^b take ^g$N\n",
        thing, found, SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);

    } else if (!CharCanCarry(thing, found)) {
      /* check if too heavy */
      SendAction("^b$N is too heavy for you to lift\n",
        thing, found, SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);

    } else {
      /* Check whether we are looting a corpse */
      if (*dstKey
       && search->tType==TTYPE_OBJ
       && Obj(search)->oTemplate->oType==OTYPE_CONTAINER)
      {
        flag=OBJECTGETFIELD(search, OF_CONTAINER_CFLAG);
        if ( BIT(flag, OCF_CLOSED) ) {
          SendAction("^bYou try to take something from ^g$N ^b, then realize its closed\n",
            thing, search, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
          SendAction("^b$n tries to take something from ^g$N ^b, then realizes that its closed\n",
            thing, search, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
          return;
        } else if ( BIT(flag, OCF_PLAYERCORPSE) ) {
          /* check whether we are looting our OWN corpse */
          sprintf(buf, FIGHT_CORPSE_KEY, Base(thing)->bKey->sText);
          if (STRICMP(Base(search)->bKey->sText, buf)) {
            if (BIT(flag, OCF_PLAYERLOOTED)) {
              SendAction("^wYou try to loot ^g$N ^wbut someone beat you to it\n",
                thing, search, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
              SendAction("^b$n tries to loot ^g$N ^wbut someone beat $m to it\n",
                thing, search, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
              return;
            } else {
              BITSET(flag, OCF_PLAYERLOOTED);
              OBJECTSETFIELD(search, OF_CONTAINER_CFLAG, flag);
              srcNum = 0; /* only one object */
            }
          }
        }
      }

      /* guess we can take it */
      InvIndexAddItem( found );
    }
    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJ|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }

  /* Okay now that we have built a list of targets, lets take some action on it */
  /* Cryogen: Note that we rely on the list being in like grouped with like order,
   * this should be true because ObjectEquivalent is also used by ThingTo to
   * sort objects when it inserts them into the destination list
   */
  for (i=0; i<invIndex.iNum; i+=count) {
    found = invIndex.iThing[i];

    /* for messages gets [x] blah ect. must maintain a count for this */
    for(count=1; i+count<invIndex.iNum; count++ ) {
      compareThing = invIndex.iThing[i+count];
      if ( compareThing->tType!=TTYPE_OBJ ) break;
      if ( Obj(compareThing)->oTemplate->oType==OTYPE_MONEY ) break;
      if ( !ObjectEquivalent(found, compareThing) ) break;
    }

    if ( ObjectIsJunk(found, thing) ) {
      /* autojunk it its worthless */
      if( count == 1 ) {
        SendAction("^CYou autojunk ^g$N\n", thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^C$n autojunks ^g$N\n", thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      } else {
        sprintf( srcKey,"^bYou autojunk ^g[%li] $N", count );
        SendAction(srcKey,thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        sprintf( srcKey,"^b$n autojunks ^g[%li] $N", count );
        SendAction(srcKey,thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      }
      for (j=0; j<count; j++) ThingFree( invIndex.iThing[i+j] );

    } else if (found->tType==TTYPE_OBJ&&Obj(found)->oTemplate->oType==OTYPE_MONEY) {
      /* guard against no gain zones etc here */
      world = Base(thing)->bInside;
      if (world->tType==TTYPE_WLD
        && BIT(areaList[Wld(world)->wArea].aResetFlag, RF_NOMONEY)) {
        sprintf(buf,
                "^yYou ^rWOULD ^yhave gained %ld credits! (This zone isnt open yet)\n",
                OBJECTGETFIELD(found, OF_MONEY_AMOUNT));
        SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      } else {
        sprintf(buf, "^yYou get %ld credits!\n", OBJECTGETFIELD(found, OF_MONEY_AMOUNT));
        SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
        Character(thing)->cMoney += OBJECTGETFIELD(found, OF_MONEY_AMOUNT);
      }
      /* Count better be one here or we have serious problems */
      for (j=0; j<count; j++) ThingFree( invIndex.iThing[i+j] );

    } else {
      /* Add to their inventory now, then give the message later */
      if( count == 1 ) {
        SendAction("^bYou get ^g$N", thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n gets ^g$N", thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      } else {
        sprintf( srcKey,"^bYou get ^g[%li] $N", count );
        SendAction(srcKey,thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        sprintf( srcKey,"^b$n gets ^g[%li] $N", count );
        SendAction(srcKey,thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      }
      /* complete the action */
      if (*dstKey) {
        SendAction(" ^bfrom $S ^g$N", thing, search, SEND_SRC|SEND_ROOM|SEND_VISIBLE);
      }
      SendAction("\n", thing, search, SEND_SRC|SEND_ROOM|SEND_VISIBLE);
      for (j=0; j<count; j++) ThingTo( invIndex.iThing[i+j], thing );

    }
  }
  /* All done clear the index */
  InvIndexClear();
}

CMDPROC(CmdDrop) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found;
  THING *search;
  LWORD  i;
  LWORD  j;
  THING *compareThing;
  LWORD  count;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  }
  InvIndexClear();
  while (found && srcNum!=0) {
    if (InvUnEquip(thing, found, IUE_BLOCKABLE)) {
      InvIndexAddItem( found );
    }
    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }

  for(i=0; i<invIndex.iNum; i+=count) {
    found = invIndex.iThing[i];

    /* for messages gets [x] blah ect. must maintain a count for this */
    for(count=1; i+count<invIndex.iNum; count++ ) {
      compareThing = invIndex.iThing[i+count];
      if ( compareThing->tType!=TTYPE_OBJ ) break;
      if ( Obj(compareThing)->oTemplate->oType==OTYPE_MONEY ) break;
      if ( !ObjectEquivalent(found, compareThing) ) break;
    }

    if( count == 1 ) {
      strcpy( srcKey,"^bYou drop ^g$N\n");
      strcpy( dstKey,"^b$n drops ^g$N\n");
    } else {
      sprintf( srcKey, "^bYou drop ^g[%li] $N\n", count );
      sprintf( dstKey, "^b$n drops ^g[%li] $N\n", count );
    }
    SendAction(srcKey, thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
    SendAction(dstKey, thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    /* do the drop */
    for (j=0; j<count; j++) ThingTo(invIndex.iThing[i+j], Base(thing)->bInside);
  }

  InvIndexClear();
}


CMDPROC(CmdJunk) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found;
  THING *search;
  LWORD  i;
  LWORD  j;
  THING *compareThing;
  LWORD  count;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  }
  InvIndexClear();
  while (found && srcNum!=0) {
    if (InvUnEquip(thing, found, IUE_BLOCKABLE)) {
      InvIndexAddItem( found );
    }
    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }

  for(i=0; i<invIndex.iNum; i+=count) {
    found = invIndex.iThing[i];

    /* for messages gets [x] blah ect. must maintain a count for this */
    for(count=1; i+count<invIndex.iNum; count++ ) {
      compareThing = invIndex.iThing[i+count];
      if ( compareThing->tType!=TTYPE_OBJ ) break;
      if ( Obj(compareThing)->oTemplate->oType==OTYPE_MONEY ) break;
      if ( !ObjectEquivalent(found, compareThing) ) break;
    }

    if( count == 1 ) {
      strcpy( srcKey,"^bYou junk ^g$N\n");
      strcpy( dstKey,"^b$n junks ^g$N\n");
    } else {
      sprintf( srcKey, "^bYou junk ^g[%li] $N\n",count );
      sprintf( dstKey, "^b$n junks ^g[%li] $N\n",count );
    }
    SendAction(srcKey, thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
    SendAction(dstKey, thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    for (j=0; j<count; j++) ThingFree(invIndex.iThing[i+j]);
  }

  InvIndexClear();
}


CMDPROC(CmdPut) {     /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found = NULL;
  THING *search = NULL;
  THING *dest = NULL;
  THING *ammo = NULL;
  LWORD  ammoType;
  LWORD  ammoUse;
  LWORD  ammoLeft;
  LWORD  count;
  LWORD  i;
  LWORD  j;
  THING *compareThing;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  if (*dstKey) { /* search a target */
    dest = CharThingFind(thing, dstKey, -1, thing, TF_OBJ, &dstOffset);
    if (!dest) /* search the room */
      dest = CharThingFind(thing, dstKey, -1, Base(thing)->bInside, TF_OBJ, &dstOffset);
  } else
    dest = NULL;
  if (!dest) {
    SendAction("^bThere doesnt appear to be anything like that to put things into\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found) { /* also search ourself if no 2nd object specified */
    search = Base(thing)->bInside;
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  }
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  InvIndexClear();
  while (found && srcNum!=0) {
    if (dest) {
      ammo = ObjectGetAmmo(dest, &ammoType, &ammoUse, &ammoLeft);
      if (  (Obj(dest)->oTemplate->oType == OTYPE_CONTAINER)
          ||(Obj(found)->oTemplate->oType == OTYPE_AMMO
             && ammoType
             && OBJECTGETFIELD(found, OF_AMMO_AMMOTYPE) == ammoType)
      ) {
        if (  (Obj(dest)->oTemplate->oType == OTYPE_CONTAINER
               && Base(dest)->bConWeight + Base(found)->bWeight > OBJECTGETFIELD(dest, OF_CONTAINER_MAX))
            ||(ammoType
               && ammo)
        ) {
          if( invIndex.iNum == 0 ) {
            SendAction("^bYou cant fit that in there!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
            return;
          }
          break;
        } else if (Obj(dest)->oTemplate->oType == OTYPE_CONTAINER
                   && BIT(OBJECTGETFIELD(dest, OF_CONTAINER_CFLAG), OCF_CLOSED)
        ) {
          SendAction("^bBut its closed!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
          return;
        } else if (dest == found) {
          SendAction("^bYou cant put something inside itself!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
          return;
        } else {
          InvIndexAddItem( found );
        }
      } else {
        SendAction("^bYou cant put that in there!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
    }
    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (!found) { /* also search room if no 2nd object specified */
      search = Base(thing)->bInside;
      found = CharThingFind(thing, srcKey, -1, search, TF_OBJ|TF_CONTINUE, &srcOffset);
    }
    if (srcNum>0) srcNum--;
  }

  /* Do the put operation */
  for(i=0; i<invIndex.iNum; i+=count) {
    found = invIndex.iThing[i];

    /* for messages gets [x] blah ect. must maintain a count for this */
    for(count=1; i+count<invIndex.iNum; count++ ) {
      compareThing = invIndex.iThing[i+count];
      if ( compareThing->tType!=TTYPE_OBJ ) break;
      if ( Obj(compareThing)->oTemplate->oType==OTYPE_MONEY ) break;
      if ( !ObjectEquivalent(found, compareThing) ) break;
    }

    if( count == 1 ) {
      strcpy( srcKey,"^bYou put ^g$N");
      strcpy( dstKey,"^b$n puts ^g$N");
    } else {
      sprintf( srcKey, "^bYou put ^g[%li] $N",count );
      sprintf( dstKey, "^b$n puts ^g[%li] $N",count );
    }
    SendAction(srcKey, thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
    SendAction(dstKey, thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    SendAction(" ^binto ^g$N\n", thing, dest, SEND_SRC|SEND_ROOM|SEND_VISIBLE);
    for (j=0; j<count; j++) ThingTo(invIndex.iThing[i+j], dest);
  }

  InvIndexClear();
}

CMDPROC(CmdGive) {     /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   buf[256];
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found;
  THING *search;
  THING *dest;
  LWORD  count;
  LWORD  i;
  LWORD  j;
  THING *compareThing;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  /* search the room for a target */
  if (*dstKey) {
    dest = CharThingFind(thing, dstKey, -1, Base(thing)->bInside, TF_PLR|TF_MOB, &dstOffset);
  } else
    dest = NULL;
  if (!dest) {
    SendAction("^bThere doesnt appear to be anyone like that to give things to\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* search ourself for an object */
  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  /* intercept here to give credits */
  if ( (!found && StrAbbrev("credits", srcKey))
    || (StrExact("credits", srcKey)) )
  {
    if (srcNum < 0) {
      SendAction("^wAnd how does that work again!?!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    } else if (srcNum > Character(thing)->cMoney) {
      SendAction("^wYou dont have that much money!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    } else {
      /* transfer the money */
      Character(thing)->cMoney -= srcNum;
      Character(dest)->cMoney += srcNum;
      SoundMisc( thing, SOUND_CREDIT_LOW, 0 );
      /* send the messages */
      sprintf(buf, "^CYou give $N %ld credits\n", srcNum);
      SendAction(buf, thing, dest, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      sprintf(buf, "^b$n gives you %ld credits\n", srcNum);
      SendAction(buf, thing, dest, SEND_DST |SEND_VISIBLE|SEND_CAPFIRST);
      sprintf(buf, "^C$n gives $N some money\n");
      SendAction(buf, thing, dest, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      return;
    }
  }
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around to give\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  InvIndexClear();
  while (found && srcNum!=0) {
    if (Base(found)->bWeight + Base(dest)->bConWeight > CharGetCarryMax(dest)) {
      SendAction("^CYou try to give ^g$N",
        thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^C$n tries to give ^g$N",
        thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction(" ^Cto ^g$N^C, but its too heavy!\n",
        thing, dest, SEND_SRC|SEND_ROOM|SEND_VISIBLE);
      SendAction(" ^bto ^gyou^C, but its too heavy!\n",
        thing, dest, SEND_DST|SEND_VISIBLE);
    } else {
      InvIndexAddItem( found );
    }
    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }

  for(i=0; i<invIndex.iNum; i+=count) {
    found = invIndex.iThing[i];

    /* for messages gets [x] blah ect. must maintain a count for this */
    for(count=1; i+count<invIndex.iNum; count++ ) {
      compareThing = invIndex.iThing[i+count];
      if ( compareThing->tType!=TTYPE_OBJ ) break;
      if ( Obj(compareThing)->oTemplate->oType==OTYPE_MONEY ) break;
      if ( !ObjectEquivalent(found, compareThing) ) break;
    }

    if( count == 1 ) {
      strcpy( srcKey,"^bYou give ^g$N");
      strcpy( dstKey,"^b$n gives ^g$N");
    } else {
      sprintf( srcKey, "^bYou give ^g[%li] $N\n",count );
      sprintf( dstKey, "^bYou give g[%li] $N\n",count );
    }
    SendAction(srcKey, thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
    SendAction(dstKey, thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    SendAction(" ^bto ^g$N\n", thing, dest, SEND_SRC|SEND_ROOM|SEND_VISIBLE);
    SendAction(" ^bto ^gyou\n", thing, dest, SEND_DST|SEND_VISIBLE);
    for (j=0; j<count; j++) ThingTo(invIndex.iThing[i+j], dest);
  }
}

/* Add direct applies of an equipped item */
void InvApply(THING *thing, THING *equip) {
  LWORD apply;
  LWORD aType;
  LWORD aValue;

  switch (Obj(equip)->oTemplate->oType) {
  case OTYPE_ARMOR:
    Effect(AFFECT_ARMOR,          TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_ARMOR));
    Effect(AFFECT_RES_PUNCTURE,   TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RPUNCTURE));
    Effect(AFFECT_RES_SLASH,      TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RSLASH));
    Effect(AFFECT_RES_CONCUSSIVE, TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RCONCUSSIVE));
    Effect(AFFECT_RES_HEAT,       TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RHEAT));
    Effect(AFFECT_RES_EMR,        TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_REMR));
    Effect(AFFECT_RES_LASER,      TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RLASER));
    Effect(AFFECT_RES_PSYCHIC,    TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RPSYCHIC));
    Effect(AFFECT_RES_ACID,       TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RACID));
    Effect(AFFECT_RES_POISON,     TAR_AFFECT, thing, NULL, OBJECTGETFIELD(equip, OF_ARMOR_RPOISON));
    break;
  }
  /* apply direct effects */
  for (apply=0; apply < OBJECT_MAX_APPLY; apply++) {
    aType  = Obj(equip)->oApply[apply].aType;
    if (applyList[aType].aTarget==TAR_AFFECT) {
      aValue = Obj(equip)->oApply[apply].aValue;
      Effect(*applyList[aType].aEffect, applyList[aType].aTarget, thing, NULL, aValue);
    }
  }
}

/* return FALSE if we cant equip this */
BYTE InvEquip(THING *thing, THING *equip, BYTE *message) {
  LWORD i;
  LWORD wearType;
  LWORD apply;
  LWORD aType;
  LWORD aValue;
  THING *ammo;
  LWORD ammoType;
  LWORD ammoUse;
  LWORD ammoLeft;
  WORD* wtemp;

  if (message) message[0]='\0';
  /* MinStr etc */
  if (thing->tType == TTYPE_PLR) {
    if (Plr(thing)->pStr < PropertyGetLWord(equip, "%MINSTR", 0)) {
      if (message) sprintf(message, "You are not strong enough\n");
      return FALSE;
    }
    if (Plr(thing)->pDex < PropertyGetLWord(equip, "%MINDEX", 0)) {
      if (message) sprintf(message, "You are not dexterous enough\n");
      return FALSE;
    }
    if (Plr(thing)->pCon < PropertyGetLWord(equip, "%MINCON", 0)) {
      if (message) sprintf(message, "You are not tough enough\n");
      return FALSE;
    }
    if (Plr(thing)->pWis < PropertyGetLWord(equip, "%MINWIS", 0)) {
      if (message) sprintf(message, "You are not wise enough\n");
      return FALSE;
    }
    if (Plr(thing)->pInt < PropertyGetLWord(equip, "%MININT", 0)) {
      if (message) sprintf(message, "You are not smart enough\n");
      return FALSE;
    }
  }

  if (Character(thing)->cLevel < PropertyGetLWord(equip, "%MINLEVEL", 0)) {
    if (message) sprintf(message, "You are not high enough in level\n");
    return FALSE;
  }

  /* Aura check */
  if (Character(thing)->cAura>400) {
    if (BIT(Obj(equip)->oAct, OA_ANTI_GOOD)) {
      if (message) sprintf(message, "You are too good\n");
      return FALSE;
    }
  } else if (Character(thing)->cAura<-400) {
    if (BIT(Obj(equip)->oAct, OA_ANTI_EVIL)) {
      if (message) sprintf(message, "You are too evil\n");
      return FALSE;
    }
  } else if (BIT(Obj(equip)->oAct, OA_ANTI_NEUTRAL)) {
    if (message) sprintf(message, "You are too neutral\n");
    return FALSE;
  }

  /* ensure they have ammo for it */
  if (thing->tType==TTYPE_PLR && Obj(equip)->oTemplate->oType!=OTYPE_WEAPON) {
    ammo = ObjectGetAmmo(equip, &ammoType, &ammoUse, &ammoLeft);
    if (!ammo && ammoType) {
      if (message) sprintf(message, "It is out of ammo\n");
      return FALSE;
    }
  }

  wearType = Obj(equip)->oTemplate->oWear;

  /* for loop searches for an available location to equip the item in */
  for (i=0; i<MAX_EQUIP; i++) {
    if (!wearList[wearType].wEquipList[i]) continue;
    if ( ( (thing->tType == TTYPE_PLR)
         &&(!BITANY(Character(thing)->cEquip,           wearList[wearType].wEquipList[i]))
         &&( BIT(raceList[Plr(thing)->pRace].rLocation, wearList[wearType].wEquipList[i]))
         &&(!wearList[wearType].wEquipNot || !BITANY(raceList[Plr(thing)->pRace].rLocation, wearList[wearType].wEquipNot))
         )
       ||( (thing->tType == TTYPE_MOB)
         &&(!BITANY(Character(thing)->cEquip,           wearList[wearType].wEquipList[i]))
         )
       )
    {
      BITSET(Obj(equip)->oEquip,       wearList[wearType].wEquipList[i]);
      BITSET(Character(thing)->cEquip, wearList[wearType].wEquipList[i]);

      switch (Obj(equip)->oTemplate->oType) {
      case OTYPE_WEAPON:
        Character(thing)->cWeapon = equip;
        break;
      case OTYPE_LIGHT:
        if (Base(thing)->bInside->tType==TTYPE_WLD)
          Wld(Base(thing)->bInside)->wLight += OBJECTGETFIELD(equip, OF_LIGHT_INTENSITY);
        break;
      }

      InvApply(thing, equip);

      /* add non-direct applies */
      for (apply=0; apply < OBJECT_MAX_APPLY; apply++) {
        aType  = Obj(equip)->oApply[apply].aType;
        if (applyList[aType].aTarget!=TAR_AFFECT) {
          aValue = Obj(equip)->oApply[apply].aValue;
          wtemp = applyList[aType].aEffect;
          Effect((*wtemp), applyList[aType].aTarget, thing, NULL, aValue);
        }
      }
      return TRUE;
    }
  }
  if (message) sprintf(message, "You have no place available to use that\n");
  return FALSE;
}

/* Temporarily remove applies of an equipped item, must reapply in same routine */
void InvUnapply(THING *thing, THING *equip) {
  LWORD apply;
  LWORD aType;
  LWORD aValue;

  switch (Obj(equip)->oTemplate->oType) {
  case OTYPE_ARMOR:
    EffectFree(AFFECT_ARMOR,          TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_ARMOR));
    EffectFree(AFFECT_RES_PUNCTURE,   TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RPUNCTURE));
    EffectFree(AFFECT_RES_SLASH,      TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RSLASH));
    EffectFree(AFFECT_RES_CONCUSSIVE, TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RCONCUSSIVE));
    EffectFree(AFFECT_RES_HEAT,       TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RHEAT));
    EffectFree(AFFECT_RES_EMR,        TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_REMR));
    EffectFree(AFFECT_RES_LASER,      TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RLASER));
    EffectFree(AFFECT_RES_PSYCHIC,    TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RPSYCHIC));
    EffectFree(AFFECT_RES_ACID,       TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RACID));
    EffectFree(AFFECT_RES_POISON,     TAR_AFFECT, thing, OBJECTGETFIELD(equip, OF_ARMOR_RPOISON));
    break;
  }
  /* remove direct effects */
  for (apply=0; apply < OBJECT_MAX_APPLY; apply++) {
    aType  = Obj(equip)->oApply[apply].aType;
    if (applyList[aType].aTarget==TAR_AFFECT) {
      aValue = Obj(equip)->oApply[apply].aValue;
      EffectFree(*applyList[aType].aEffect, applyList[aType].aTarget, thing, aValue);
    }
  }
}

/* return FALSE if we cant unequip this */
BYTE InvUnEquip(THING *thing, THING *equip, BYTE blockable) {
  LWORD apply;
  LWORD aType;
  LWORD aValue;

  /* check params for validity */
  if (!thing || !equip) return TRUE;
  if (thing->tType < TTYPE_CHARACTER) return TRUE;

  /* Check to see if its allready unequipped */
  if (equip->tType!=TTYPE_OBJ || Obj(equip)->oEquip==0)
    return TRUE;

  /* If we can, unequip it */
  if (blockable) {
    if (BIT(Obj(equip)->oAct, OA_NODROP))
      return FALSE;
  }
  BITCLR(Character(thing)->cEquip, Obj(equip)->oEquip);

  Obj(equip)->oEquip = 0;
  switch (Obj(equip)->oTemplate->oType) {
  case OTYPE_WEAPON:
    if (Character(thing)->cWeapon == equip)
      Character(thing)->cWeapon = NULL;
    break;
  case OTYPE_LIGHT:
    if (Base(thing)->bInside && Base(thing)->bInside->tType==TTYPE_WLD)
      Wld(Base(thing)->bInside)->wLight -= OBJECTGETFIELD(equip, OF_LIGHT_INTENSITY);
    break;
  }

  InvUnapply(thing, equip);

  /* remove non-direct apply's */
  for (apply=0; apply < OBJECT_MAX_APPLY; apply++) {
    aType  = Obj(equip)->oApply[apply].aType;
    if (applyList[aType].aTarget!=TAR_AFFECT) {
      aValue = Obj(equip)->oApply[apply].aValue;
      EffectFree(*applyList[aType].aEffect, applyList[aType].aTarget, thing, aValue);
    }
  }
  return TRUE;
}

void InvEquipShow(THING *thing) {
  BYTE  buf[256];
  LWORD shotLeft;
  THING *ammo;
  LWORD ammoType;
  LWORD ammoUse;
  LWORD ammoLeft;
  LWORD fireRate;

  SendAction("\n^pYou are currently Equipped with:\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  SendAction("^P-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n", thing, NULL, SEND_SRC|SEND_VISIBLE );

  CharShowEquip(thing, thing ); /* show them their own equip list */

  /* Show Weapon */
  if (Character(thing)->cWeapon) {
    SendAction("\n^GYour Currently Selected Weapon is:\n^g", thing, NULL, SEND_SRC|SEND_VISIBLE );
    SendAction(Character(thing)->cWeapon->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
    ammo = ObjectGetAmmo(Character(thing)->cWeapon, &ammoType, &ammoUse, &ammoLeft);
    if (ammoUse>0) {
      if (!ammo)  {
        SendAction(" ^r<UNLOADED>", thing, NULL, SEND_SRC|SEND_VISIBLE );
      } else if (ammoLeft<0) {
        SendAction(" ^r<UNLIMITED AMMO>", thing, NULL, SEND_SRC|SEND_VISIBLE );
      } else {
        fireRate = OBJECTGETFIELD(Character(thing)->cWeapon, OF_WEAPON_FIRERATE);
        shotLeft = ammoLeft/ammoUse;
        if (!shotLeft) {
          SendAction(" ^w<OUT OF AMMO>", thing, NULL, SEND_SRC|SEND_VISIBLE );
        } else if ( fireRate*3 > shotLeft) {
          sprintf(buf, " ^r<Only %ld shots left!>", shotLeft);
          SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
        } else {
          sprintf(buf, " ^y<%ld shots left>", shotLeft);
          SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
        }
      }
    } else
      SendAction(" ^r<NO AMMO NEEDED>", thing, NULL, SEND_SRC|SEND_VISIBLE );
    SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  }
}

/* Eat primitive, called by CmdEat as well as PlayerTick */
void InvEat(THING *thing, THING *found) {
  LWORD poison;

  SendAction("^bYou eat ^g$N\n",
       thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
  SendAction("^b$n eats ^g$N\n",
       thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  if (thing->tType == TTYPE_PLR) {
    SoundEat( thing, 0, 0 );
    Plr(thing)->pHunger -= OBJECTGETFIELD(found, OF_FOOD_HOWFILLING);
    /* ADD: Check OF_FOOD_POISON here too */
    poison = OBJECTGETFIELD(found, OF_FOOD_POISON);
    if (poison > 0)
      Effect(EFFECT_POISON, TAR_SELF_DEF, thing, "", poison);

    /* check min hunger against race */
    BOUNDSET((-1*raceList[Plr(thing)->pRace].rMaxThirst), Plr(thing)->pThirst, raceList[Plr(thing)->pRace].rMaxThirst);
    BOUNDSET((-1*raceList[Plr(thing)->pRace].rMaxHunger), Plr(thing)->pHunger, raceList[Plr(thing)->pRace].rMaxHunger);
    BOUNDSET((-1*raceList[Plr(thing)->pRace].rMaxIntox) , Plr(thing)->pIntox,  raceList[Plr(thing)->pRace].rMaxIntox );
  }

  CodeParseAfterEating( thing, found );
  THINGFREE(found);
}

/* Eat primitive, called by CmdEat as well as PlayerTick */
void InvDrink(THING *thing, THING *found) {
  LWORD liquid;
  LWORD liquidLeft;
  LWORD poison;

  if (Base(found)->bInside != thing) {
    SendAction("^bYou take a drink from $A ^g$N\n",
         thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
    SendAction("^b$n takes a drink from $A ^g$N\n",
         thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  } else {
    SendAction("^bYou take a drink from your ^g$N\n",
         thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
    SendAction("^b$n takes a drink from $s ^g$N\n",
         thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  }
  if (thing->tType == TTYPE_PLR) {
    SoundDrink( thing, 0, 0 );
    liquid = OBJECTGETFIELD(found, OF_DRINKCON_LIQUID);
    TYPECHECK(liquid, oLiquidList);
    /* thirst */
    if (Plr(thing)->pThirst>=-1*raceList[Plr(thing)->pRace].rMaxThirst) {
      Plr(thing)->pThirst -= oLiquidList[liquid].oLThirst;
      BOUNDSET((-1*raceList[Plr(thing)->pRace].rMaxThirst), Plr(thing)->pThirst, raceList[Plr(thing)->pRace].rMaxThirst);
    }
    /* Hunger */
    if (Plr(thing)->pHunger>=-1*raceList[Plr(thing)->pRace].rMaxHunger) {
      Plr(thing)->pHunger += oLiquidList[liquid].oLHunger;
      BOUNDSET((-1*raceList[Plr(thing)->pRace].rMaxHunger), Plr(thing)->pHunger, raceList[Plr(thing)->pRace].rMaxHunger);
    }
    /* Intox */
    if (Plr(thing)->pIntox>=-1*raceList[Plr(thing)->pRace].rMaxIntox) {
      Plr(thing)->pIntox  += oLiquidList[liquid].oLIntox;
      BOUNDSET((-1*raceList[Plr(thing)->pRace].rMaxIntox) , Plr(thing)->pIntox,  raceList[Plr(thing)->pRace].rMaxIntox );
    }
    if (oLiquidList[liquid].oLPoison > 0)
      Effect(EFFECT_POISON, TAR_SELF_DEF, thing, "", oLiquidList[liquid].oLPoison);
  }

  /* ADD: Check OF_DRINKCON_POISON here too */
  poison = OBJECTGETFIELD(found, OF_DRINKCON_POISON);
  if (poison > 0)
    Effect(EFFECT_POISON, TAR_SELF_DEF, thing, "", poison);

  liquidLeft = OBJECTGETFIELD(found, OF_DRINKCON_CONTAIN);
  if (liquidLeft>0) {
    liquidLeft--;
    MINSET(liquidLeft, 0);
  }
  OBJECTSETFIELD(found, OF_DRINKCON_CONTAIN, liquidLeft);
}

CMDPROC(CmdEquip) {   /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  BYTE   message[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  if (!*cmd) { /* give the player the list of things they have equipped */
    InvEquipShow(thing);
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJINV, &srcOffset);
  if (!found) {
    SendAction("^bEquip what?!?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  while(found && srcNum!=0) {
    if (Obj(found)->oEquip==0) {
      if (InvEquip(thing, found, message)) {
        SendAction("^bYou equip yourself with ^g$N\n",
          thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n equips $mself with ^g$N\n",
          thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      } else {
        /* was unable to equip the item */
        if (STRICMP(srcKey, "all")) {
          SendAction("^bYou dont seem to be able to equip ^g", thing, NULL, SEND_SRC|SEND_VISIBLE );
          SendAction(found->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
          SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
          SendAction(message, thing, NULL, SEND_SRC|SEND_VISIBLE );
        }
      }
    }

    found = CharThingFind(thing, srcKey, -1, thing, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}


CMDPROC(CmdUnEquip) { /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING  *found;
  AFFECT *affect;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  if (!*cmd) { /* give the player the list of things they have equipped */
    InvEquipShow(thing);
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJEQUIP, &srcOffset);
  if (!found) {
    SendAction("^bUnEquip what?!?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  while(found && srcNum!=0) {
    if (Obj(found)->oEquip!=0) {
      /* Expire the psi-power object */
      if (Obj(found)->oTemplate == phantomPocketTemplate) {
        affect = AffectFind(thing, EFFECT_PHANTOMPOCKET);
        if (affect) AffectRemove(thing, affect);

      /* Expire the psi-power object */
      } else if (Obj(found)->oTemplate == fireBladeTemplate) {
        affect = AffectFind(thing, EFFECT_FIREBLADE);
        if (affect) AffectRemove(thing, affect);

      /* Expire the psi-power object */
      } else if (Obj(found)->oTemplate == fireShieldTemplate) {
        affect = AffectFind(thing, EFFECT_FIRESHIELD);
        if (affect) AffectRemove(thing, affect);

      /* Expire the psi-power object */
      } else if (Obj(found)->oTemplate == fireArmorTemplate) {
        affect = AffectFind(thing, EFFECT_FIREARMOR);
        if (affect) AffectRemove(thing, affect);

      /* Try to unequip it */
      } else if (InvUnEquip(thing, found, IUE_BLOCKABLE)) {
        SendAction("^bYou unequip ^g$N\n",
          thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n unequips ^g$N\n",
          thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);

      /* was unable to unequip the item */
      } else {
        SendAction("^bYou dont seem to be able to unequip ^g",thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(found->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n",thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
    }

    found = CharThingFind(thing, srcKey, -1, thing, TF_OBJEQUIP|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}

CMDPROC(CmdInventory) { /* void CmdProc(THING *thing, BYTE* cmd) */
  THING *show;
  BYTE  buf[256];
  SOCK *sock;
  BYTE  expert = FALSE;

  sock = BaseControlFind(thing);
  if (sock && BIT(Plr(sock->sHomeThing)->pAuto, PA_EXPERT)) expert = TRUE;

  if (Base(thing)->bConWeight <= CharGetCarryMax(thing)/4)
    sprintf(buf,
            "^pYou are currently carrying: ^w%hd/%ld pounds ^y(Move cost = Normal)\n",
            Base(thing)->bConWeight,
            CharGetCarryMax(thing));
  else if (Base(thing)->bConWeight <= CharGetCarryMax(thing)/2)
    sprintf(buf,
            "^pYou are currently carrying: ^w%hd/%ld pounds ^y(Move cost = 1.5X)\n",
            Base(thing)->bConWeight,
            CharGetCarryMax(thing));
  else if (Base(thing)->bConWeight <= 75*CharGetCarryMax(thing)/100)
    sprintf(buf,
            "^pYou are currently carrying: ^w%hd/%ld pounds ^y(Move cost = 2X)\n",
            Base(thing)->bConWeight,
            CharGetCarryMax(thing));
  else
    sprintf(buf,
            "^pYou are currently carrying: ^w%hd/%ld pounds ^y(Move cost = 2.5X)\n",
            Base(thing)->bConWeight,
            CharGetCarryMax(thing));
  SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
  SendAction("^P-=-=-=-=-=-=-=-=-=-=-=-=-=-\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  show=thing->tContain;
  while (show) {
    if (show->tType!=TTYPE_OBJ || Obj(show)->oEquip==0) {
      show = ThingShow(show, thing );
    } else
      show = show->tNext;
  }
  /* Hint for total newbies */
  if (Character(thing)->cEquip && Character(thing)->cLevel<=5 && !expert)
    SendHint("^;HINT: To see what you are equipped with, type ^<EQUIP\n", thing );

}

CMDPROC(CmdEat) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
  THING *search;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found) {
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
    if (!found) {
      found = ThingRecurseGetObject( thing, OTYPE_FOOD, -1, 0 );
      if( !found ) {
        SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
    }
  }

  while (found && srcNum!=0) {
    if (InvUnEquip(thing, found, IUE_BLOCKABLE)) {
      if (Obj(found)->oTemplate->oType == OTYPE_FOOD) {
        InvEat(thing, found);
      } else {
        SendAction("^bYou try to eat the ^g$N ^bthen realize it isnt food\n",
          thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n tries to eat $A ^g$N ^bthen realizes it isnt food\n",
          thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      }
    } else {
      SendAction("^bYou dont seem to be able to unequip ^g$N ^blong enough to eat it!\n",
        thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^b$n  doesnt seem to be able to unequip ^g$N ^blong enough to eat it!\n",
        thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }

    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}


CMDPROC(CmdDrink) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
  THING *search;
  LWORD  liquidLeft;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found) {
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
    if (!found) {
      found = ThingRecurseGetObject( thing, OTYPE_DRINKCON, -1, 0 );
      if( !found ) {
        SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
    }
  }

  while (found && srcNum!=0) {
    if (Obj(found)->oTemplate->oType == OTYPE_DRINKCON) {
      liquidLeft = OBJECTGETFIELD(found, OF_DRINKCON_CONTAIN);
      if (liquidLeft!=0) {
        InvDrink(thing, found);
      } else {
        SendAction("^bYou try to drink from ^g$N ^bthen realize its empty\n",
          thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n tries to drink from ^g$N ^bthen realizes its empty\n",
          thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      }
    } else {
      SendAction("^bYou try to drink from ^g$N ^bthen realizes it isnt possible\n",
        thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^b$n tries to eat $A ^g$N ^bthen realizes it isnt possible\n",
        thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }

    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}


CMDPROC(CmdFill) {     /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  THING *found = NULL;
  THING *search = NULL;
  THING *dest = NULL;
  LWORD  maxFound;
  LWORD  containFound;
  LWORD  liquidFound;
  LWORD  poisonFound;
  LWORD  containDest;
  LWORD  liquidDest;
  LWORD  poisonDest;
  LWORD  transfer;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  if (*dstKey) { /* search a target */
    dest = CharThingFind(thing, dstKey, -1, thing, TF_OBJINV, &dstOffset);
    if (!dest) /* search the room */
      dest = CharThingFind(thing, dstKey, -1, Base(thing)->bInside, TF_OBJ, &dstOffset);
  } else
    dest = NULL;
  if (!dest) {
    SendAction("^bTry something along the lines of ^wfill canteen fountain\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found)
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );

  while (found && srcNum!=0) {
    if (  (Obj(found)->oTemplate->oType == OTYPE_DRINKCON)
  &&(Obj(dest )->oTemplate->oType == OTYPE_DRINKCON)
  ) {
      /* Read data from found */
      maxFound     = OBJECTGETFIELD(found, OF_DRINKCON_MAX);
      containFound = OBJECTGETFIELD(found, OF_DRINKCON_CONTAIN);
      liquidFound  = OBJECTGETFIELD(found, OF_DRINKCON_LIQUID);
      poisonFound  = OBJECTGETFIELD(found, OF_DRINKCON_POISON);
      /* read data from dest */
      containDest  = OBJECTGETFIELD(dest,  OF_DRINKCON_CONTAIN);
      liquidDest   = OBJECTGETFIELD(dest,  OF_DRINKCON_LIQUID);
      poisonDest   = OBJECTGETFIELD(dest,  OF_DRINKCON_POISON);

      if (containFound!=0 && liquidFound != liquidDest) {
        /* source allready has a different liquid in it */
        SendAction("^bYou start to fill your ^g$N^b, then realize that you allready have something in it\n",
          thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n starts to fill $s ^g$N^b, then realizes that $h allready has something in it\n",
          thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      } else if (containFound<0 || containFound>=maxFound) {
        /* trying to fill to an infinite source or full drinkcon */
        SendAction("^bYou start to fill your ^g$N^b, then realize its allready full\n",
          thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n starts to fill $s ^g$N^b, then realizes that its allready full\n",
          thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      } else if (containDest == 0) {
        /* trying to fill from a empty drinkcon */
        SendAction("^bYou start to fill your ^g$N",
          thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n starts to fill $s ^g$N",
          thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b, but the ^g$N^b is empty\n",
          thing, dest, SEND_SRC|SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE);
      } else if (dest==found) {
        /* trying to fill a drinkcon from itself */
        SendAction("^bYou fill your ^g$N ^bfrom itself (real productive).",
          thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction(
          "^b$n fills ^g$N ^bfrom itself (bright as a bubble, that one).",
          thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      } else {
        /* perform the fill */
        transfer = maxFound - containFound;
        if (containDest>=0) {
          MAXSET(transfer, containDest);
          containDest -= transfer;
          OBJECTSETFIELD(dest, OF_DRINKCON_CONTAIN, containDest);
        }
        containFound+=transfer;
        OBJECTSETFIELD(found, OF_DRINKCON_CONTAIN, containFound);
        /* Poison accordingly */
        if (poisonDest > poisonFound) {
          poisonFound = poisonDest;
          OBJECTSETFIELD(found, OF_DRINKCON_POISON, poisonFound);
        }

        SendAction("^bYou fill your ^g$N",
          thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n fills $s ^g$N",
          thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction(" ^bfrom the ^g$N\n",
          thing, dest, SEND_SRC|SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE);
      }
    } else {
      SendAction("^bYou cant fill your $N",
        thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^bfrom the $N!\n",
        thing, dest, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
    }

    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}


CMDPROC(CmdEmpty) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  BYTE   dstKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  LWORD  dstOffset;
  LWORD  contain;
  THING *found;
  THING *search;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, dstKey, &dstOffset);

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!found)
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );

  while (found && srcNum!=0) {
    if (Obj(found)->oTemplate->oType == OTYPE_DRINKCON) {
      contain = OBJECTGETFIELD(found, OF_DRINKCON_CONTAIN);
      if (contain>0) {
        /* full container */
        SendAction("^bYou empty your ^g$N ^bon the floor\n",
             thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n empties $s ^g$N ^bon the floor\n",
             thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        OBJECTSETFIELD(found, OF_DRINKCON_CONTAIN, 0);
        OBJECTSETFIELD(found, OF_DRINKCON_POISON,  0);
      } else if (contain==0) {
        /* empty container */
        SendAction("^bYou hold your ^g$N ^bupside down and shake, but nothing comes out\n",
             thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n holds $s ^g$N ^bupside down and shakes it, but nothing comes out\n",
             thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      } else {
        /* infinite liquid source */
        SendAction("^bYou hold your ^g$N ^bupside down and spray liquid all over the floor\n",
             thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^b$n holds $s ^g$N ^bupside down and sprays liquid all over the floor\n",
             thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      }
    } else {
      SendAction("^bYou empty your ^g$N\n",
                thing, found, SEND_SRC |SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^b$n empties $s ^g$N\n",
                thing, found, SEND_ROOM|SEND_AUDIBLE|SEND_VISIBLE|SEND_CAPFIRST);
    }
    while(found->tContain)
      ThingTo(found->tContain, Base(found)->bInside);


    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJINV|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}

CMDPROC(CmdRead) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE      srcKey[256];
  LWORD     srcNum;
  LWORD     srcOffset;
  THING    *found;
  LWORD     i;
  LWORD     board;
  BYTE     *origCmd;

  origCmd = cmd;
  cmd = StrOneWord(cmd, NULL);
  if (!*cmd) {
    SendAction("^rTry Read <Object>\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJ, &srcOffset);
  if (!found)
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
  if (!found) {
    /* SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE ); */
    CmdLook(thing, origCmd);
    return;
  }
  if (Obj(found)->oTemplate->oType!=OTYPE_BOARD) {
    /* SendAction("^bThat sort of thing cant have anything written on it.\n", thing, NULL, SEND_SRC|SEND_VISIBLE ); */
    CmdLook(thing, origCmd);
    return;
  }
  board = BoardOf(OBJECTGETFIELD(found, OF_BOARD_BVIRTUAL));
  if (board == -1) {
    SendAction("^wTell whoever created this thing that the virtual # is invalid\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* List of message titles */
  if (!*cmd) {
    BoardShow(board, thing );
    SendHint("^;HINT: Try ^<Read ^;<object> <msg#> to read a message\n", thing );
    return;
  }

  i = -1;
  sscanf(cmd, " %ld", &i);
  i -= 1;
  if (i<0 || i>= boardList[board].bIndex.iNum) {
    SendAction("^wThere is no message with that number\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  BoardShowMessage(board, i, thing );
}

CMDPROC(CmdEdit) {
  LWORD     board;
  BYTE      strName[256];
  BOARDMSG *boardMsg;
  LWORD     i;
  BYTE      srcKey[256];
  LWORD     srcNum;
  LWORD     srcOffset;
  THING    *found;

  if (thing->tType!=TTYPE_PLR) {
    SendAction("^rMobs cant edit messages\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  cmd = StrOneWord(cmd, NULL);
  if (!*cmd) {
    SendAction("^rTry Edit <Object> <msg #>\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJ, &srcOffset);
  if (!found)
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Obj(found)->oTemplate->oType!=OTYPE_BOARD) {
    SendAction("^bThat sort of thing can have anything written on it.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  board = BoardOf(OBJECTGETFIELD(found, OF_BOARD_BVIRTUAL));
  if (board == -1) {
    SendAction("^wTell whoever created this thing that the virtual # is invalid\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* check virtual # */
  i = -1;
  sscanf(cmd, " %ld", &i);
  i -= 1;
  if (i<0 || i>= boardList[board].bIndex.iNum) {
    SendAction("^wThere is no message with that number\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  boardMsg = BoardMsg(boardList[board].bIndex.iThing[i]);

  if (  (Character(thing)->cLevel<LEVEL_SECURITY)
      ||(!StrExact(thing->tSDesc->sText, boardMsg->bAuthor->sText)) ) {
    SendAction("^rYou can only edit your own messages\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  sprintf(strName, "Board [%s]- Edit", boardList[board].bFileName);
  EDITSTR(thing, boardMsg->bText, 2048, strName, EP_ENDLF|EP_IMMNEW);
  EDITFLAG(thing, &boardList[board].bFlag, B_UNSAVED);
}


CMDPROC(CmdWrite) {
  LWORD     board;
  BYTE      strName[256];
  BOARDMSG *boardMsg;
  BYTE      srcKey[256];
  LWORD     srcNum;
  LWORD     srcOffset;
  THING    *found;

  cmd = StrOneWord(cmd, NULL);
  if (!*cmd) {
    SendAction("^rTry Write <Object> <Message Title>\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJ, &srcOffset);
  if (!found)
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Obj(found)->oTemplate->oType!=OTYPE_BOARD) {
    SendAction("^bThat sort of thing can have anything written on it.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  board = BoardOf(OBJECTGETFIELD(found, OF_BOARD_BVIRTUAL));
  if (board == -1) {
    SendAction("^wTell whoever created this thing that the virtual # is invalid\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* chuck virtual # */
  if (!*cmd)
    boardMsg = BoardMsgCreate(board, thing, "^Y<No Title>", BOARD_NOTREPLY);
  else
    boardMsg = BoardMsgCreate(board, thing, cmd, BOARD_NOTREPLY);

  if (!boardMsg) {
    SendAction("^rNot good, critical error aborting message\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  sprintf(strName, "Board [%s] - Write", boardList[board].bFileName);
  EDITSTR(thing, boardMsg->bText, 2048, strName, EP_ENDLF|EP_IMMNEW);
  EDITFLAG(thing, &boardList[board].bFlag, B_UNSAVED);
}

CMDPROC(CmdReply) {
  LWORD     board;
  BYTE      titleBuf[50];
  BYTE      strName[256];
  BOARDMSG *boardMsg;
  LWORD     i;
  BYTE      srcKey[256];
  LWORD     srcNum;
  LWORD     srcOffset;
  THING    *found;

  cmd = StrOneWord(cmd, NULL);
  if (!*cmd) {
    SendAction("^rTry Reply <Object> <msg #> [<msg title>]\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJ, &srcOffset);
  if (!found)
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Obj(found)->oTemplate->oType!=OTYPE_BOARD) {
    SendAction("^bThat sort of thing cant have anything written on it.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  board = BoardOf(OBJECTGETFIELD(found, OF_BOARD_BVIRTUAL));
  if (board == -1) {
    SendAction("^wTell whoever created this thing that the virtual # is invalid\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* check msg # */
  i = -1;
  sscanf(cmd, " %ld", &i);
  i -= 1;
  if (i<0 || i>= boardList[board].bIndex.iNum) {
    SendAction("^wThere is no message with that number\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  boardMsg = BoardMsg(boardList[board].bIndex.iThing[i]);

  cmd = StrOneWord(cmd, NULL);
  if (!*cmd) {
    strcpy(titleBuf, "Re:");
    strncpy(titleBuf+3, boardMsg->bTitle->sText, sizeof(titleBuf)-3);
    titleBuf[sizeof(titleBuf)-1] = 0;
    boardMsg = BoardMsgCreate(board, thing, titleBuf, i);
  } else
    boardMsg = BoardMsgCreate(board, thing, cmd, i);

  if (!boardMsg) {
    SendAction("^rNot good, critical error aborting message\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  sprintf(strName, "Board [%s] - Reply", boardList[board].bFileName);
  EDITSTR(thing, boardMsg->bText, 2048, strName, EP_ENDLF|EP_IMMNEW);
  EDITFLAG(thing, &boardList[board].bFlag, B_UNSAVED);
}

CMDPROC(CmdErase) {
  LWORD     board;
  BYTE      buf[256];
  BOARDMSG *boardMsg;
  LWORD     i;
  BYTE      srcKey[256];
  LWORD     srcNum;
  LWORD     srcOffset;
  THING    *found;

  cmd = StrOneWord(cmd, NULL);
  if (!*cmd) {
    SendAction("^rTry Erase <object> <msg #>\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, thing, TF_OBJ, &srcOffset);
  if (!found)
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Obj(found)->oTemplate->oType!=OTYPE_BOARD) {
    SendAction("^bThat sort of thing can have anything written on it.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  board = BoardOf(OBJECTGETFIELD(found, OF_BOARD_BVIRTUAL));
  if (board == -1) {
    SendAction("^wTell whoever created this thing that the virtual # is invalid\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* check msg # */
  i = -1;
  sscanf(cmd, " %ld", &i);
  i -= 1;
  if (i<0 || i>= boardList[board].bIndex.iNum) {
    SendAction("^wThere is no message with that number\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  boardMsg = BoardMsg(boardList[board].bIndex.iThing[i]);

  SendAction("^wErasing:\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  sprintf(buf, "^gMessage Number: ^G[^b%ld/%ld^G]\n", i+1, boardList[board].bIndex.iNum);
  SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
  sprintf(buf, "^gAuthor:         ^G[^b%s^G]\n", boardMsg->bAuthor->sText);
  SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
  SendAction(   "^gMessage Title:  ^G[^g", thing, NULL, SEND_SRC|SEND_VISIBLE );
  SendAction(boardMsg->bTitle->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
  SendAction("^G]\n", thing, NULL, SEND_SRC|SEND_VISIBLE );

  BoardMsgDelete(board, i);
}

int InvScan(THING *thing, THING *scanner, BYTE *cmd) {
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
  THING *search;
  FLAG   cFlag;
  LWORD  cValue;
  FLAG   sFlag;
  LWORD  sBio;
  LWORD  sMax;
  LWORD  sChip;
  LWORD  sLeft;
  BYTE   buf[256];
  THING *world;

  if (!*cmd) {
    SendAction("^wPerhaps you should specify something to scan\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return TRUE;
  }

  /* Scan, yes fine but What? */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  search = Base(thing)->bInside;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return TRUE;
  }

  while (found && srcNum!=0) {
    /* Scan Found */
    while(1) {
      if (found->tType != TTYPE_OBJ
        ||Obj(found)->oTemplate->oType != OTYPE_CONTAINER) {
        SendAction("^pYou detect nothing of any particular interest\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        break;
      }
      cFlag  = OBJECTGETFIELD(found, OF_CONTAINER_CFLAG);
      cValue = OBJECTGETFIELD(found, OF_CONTAINER_SVALUE);
      if (!BIT(cFlag, OCF_CORPSE)) {
        SendAction("^CYou detect nothing of any particular interest\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        break;
      }
      /* Check out their scanner */
      sFlag = OBJECTGETFIELD(scanner, OF_SCANNER_SFLAG);
      sMax  = OBJECTGETFIELD(scanner, OF_SCANNER_MAX);
      sBio  = OBJECTGETFIELD(scanner, OF_SCANNER_BIO);
      sChip = OBJECTGETFIELD(scanner, OF_SCANNER_CHIP);
      sLeft = sMax - sBio - sChip;

      /* Check if they can do that */
      if (BIT(cFlag, OCF_ELECTRONIC)) {
        if (!BIT(sFlag, OSF_SCANCHIP)) {
          SendAction("^wI'm afraid your scanner isnt capable of scanning that\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
          break;
        }
      } else {
        if (!BIT(sFlag, OSF_SCANBIO)) {
          SendAction("^wI'm afraid your scanner isnt capable of scanning that\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
          break;
        }
      }

      /* Check if they have room */
      if (sLeft < cValue) {
        SendAction("^wI'm afraid your scanner doesnt have enough memory free to scan that\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        break;
      }

      SendAction("^wYou scan ^g$N\n",
        thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^C$n scans ^g$N\n",
        thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);

      /* guard against no gain zones etc here */
      world = Base(thing)->bInside;

      OBJECTSETFIELD(found, OF_CONTAINER_SVALUE, 0);
      if (BIT(cFlag, OCF_ELECTRONIC)) {
        sChip += cValue;
        if (world->tType==TTYPE_WLD
         && BIT(areaList[Wld(world)->wArea].aResetFlag, RF_NOMONEY)) {
          sprintf(buf, "^YYou ^rwould ^Yhave obtained ^y%ld ^Ycredits worth of chip technology\n", cValue);
        } else {
          OBJECTSETFIELD(scanner, OF_SCANNER_CHIP, sChip);
          sprintf(buf, "^YYou obtain ^y%ld ^Ycredits worth of chip technology\n", cValue);
        }
      } else {
        sBio += cValue;
        if (world->tType==TTYPE_WLD
         && BIT(areaList[Wld(world)->wArea].aResetFlag, RF_NOMONEY)) {
          sprintf(buf, "^YYou ^rwould ^Yhave obtained ^y%ld ^Ycredits worth of biological data\n", cValue);
        } else {
          OBJECTSETFIELD(scanner, OF_SCANNER_BIO, sBio);
          sprintf(buf, "^YYou obtain ^y%ld ^Ycredits worth of biological data\n", cValue);
        }
      }
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );

      if (!cValue) {
        SendAction("^wArgh! Nothing left worth scanning, someone beat you to it\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
      break;
    }

    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJ|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
  return TRUE;
}

/* for now we'll just make it so ya gotta hold it ta use it... */
CMDPROC(CmdUse) {    /* void CmdProc(THING *thing, BYTE* cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
  THING *search;
  LWORD  apply;
  LWORD  aType;
  LWORD  aValue;
  THING *ammo = NULL;
  LWORD  ammoType;
  LWORD  ammoUse;
  LWORD  ammoLeft;
  LWORD  success;

  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  search = thing;
  found = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (!found) {
    search = Base(thing)->bInside;
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  }
  if (!found) {
    SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  while (found && srcNum!=0) {
    ammo = ObjectGetAmmo(found, &ammoType, &ammoUse, &ammoLeft);

    /* Block if this is carry2use and we're not carrying it */
    if (BIT(Obj(found)->oAct, OA_CARRY2USE) && search != thing) {
      SendAction("^wI'm afraid you must be carrying that to use it.\n",
        thing, found, SEND_SRC|SEND_CAPFIRST);
    }

    /* Block if we are out of ammo */
    else if (ammoType && !ammo) {
      SendAction("^wI'm afraid ^g$N is unloaded\n",
        thing, found, SEND_SRC|SEND_CAPFIRST);
    }

    else if (ammoUse > ammoLeft) {
      SendAction("^wI'm afraid ^g$N just ran outta steam\n",
        thing, found, SEND_SRC|SEND_CAPFIRST);
    }

    /* Give code Parser a crack at this */
    else if (!CodeParseUse(thing, found)) {

      /* Send they use the item message */
      SendAction("^bYou use ^g$N\n",
        thing, found, SEND_SRC |SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^b$n uses ^g$N\n",
        thing, found, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);

      success = FALSE;
      /* Perform the objects actions */
      for (apply=0; apply < OBJECT_MAX_APPLY; apply++) {
        aType  = Obj(found)->oApply[apply].aType;
        aValue = Obj(found)->oApply[apply].aValue;
        success |= Effect(*applyList[aType].aEffect, applyList[aType].aTarget, thing, cmd, aValue);
      }
      /* Special actions due to the object type */
      switch (Obj(found)->oTemplate->oType) {
      case OTYPE_SCANNER:
        /* Find the target etc, should I do this with an effect? */
        success |= InvScan(thing, found, cmd);
        break;
      case OTYPE_DRUG:
        if (thing->tType == TTYPE_PLR) {
          Plr(thing)->pIntox += OBJECTGETFIELD(found, OF_DRUG_INTOX);
          MAXSET(Plr(thing)->pIntox, raceList[Plr(thing)->pRace].rMaxIntox);
          success = TRUE;
        }
        break;
      }
      /* after the fact script code */
      success |= CodeParseAfterUse(thing, found);

      /* affect align if good or evil */
      if (success && thing->tType==TTYPE_PLR) {
        if (BIT(Obj(found)->oAct, OA_EVIL)) {
          if (Character(thing)->cAura > 400) {
            Character(thing)->cAura -=1;
          }
        }
        if (BIT(Obj(found)->oAct, OA_GOOD)) {
          if (Character(thing)->cAura < -400) {
            Character(thing)->cAura +=1;
          }
        }
      }

      /* tweak ammo / destroy object */
      if (!success) {
        SendAction("^wBut nothing much seems to happen\n",
          thing, found, SEND_SRC|SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      } else if (!ammo) {
      /* Destroy object or decrement ammo */
        if (ammoUse>=0) THINGFREE(found);
      } else {
        ObjectUseAmmo(found);
      }
    }

    /* see if there is more (will not find successive matches unless offset is TF_ALLMATCH) */
    found = CharThingFind(thing, srcKey, -1, search, TF_OBJ|TF_CONTINUE, &srcOffset);
    if (srcNum>0) srcNum--;
  }
}







