/* Crimson2 Mud Server
 * All source written/copyright Ryan Haksi 1995 *
 * This source code is proprietary. Use in whole or in part without
 * explicity permission by the author is strictly prohibited
 *
 * Current email address(es): cryogen@infoserve.net
 * Phone number: (604) 591-5295
 *
 * C4 Script Language written/copyright Cam Lesiuk 1995
 * Email: clesiuk@engr.uvic.ca
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "crimson2.h"
#include "macro.h"
#include "queue.h"
#include "log.h"
#include "str.h"
#include "ini.h"
#include "extra.h"
#include "property.h"
#include "file.h"
#include "thing.h"
#include "exit.h"
#include "index.h"
#include "area.h"
#include "world.h"
#include "edit.h"
#include "history.h"
#include "socket.h"
#include "send.h"
#include "base.h"
#include "object.h"
#include "char.h"
#include "affect.h"
#include "fight.h"
#include "mobile.h"
#include "player.h"
#include "skill.h"
#include "parse.h"
#include "code.h"
#include "cmd_move.h"
#include "sound.h"

/* Move a single thing through an exit, code the effects of
   deathtraps and move point loss here  */
BYTE MoveExitPrimitive(THING *thing, EXIT *exit) {
  BYTE   buf[256];
  THING *i;
  THING *from;
  THING *to;
  LWORD  cost;
  SOCK  *sock;
  LWORD  damage; 
  BYTE   eName[256];

  if( !ThingCanMove( thing ) ) {
    SendAction("^wYou are unable to move at this time!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return FALSE;
  }

  if (!exit || !exit->eWorld) {
    SendAction("^wThere doesnt appear to be anything in that direction.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return FALSE;
  }
  
  sock = BaseControlFind(thing);

  if (BIT(exit->eFlag, EF_CLOSED) && !ParseCommandCheck(PARSE_COMMAND_WEXIT, sock, "")) {
    if (BIT(exit->eFlag, EF_NOPHASE) || !BIT(Character(thing)->cAffectFlag, AF_PHASEDOOR)) {
      SendAction("^wThat way is closed to you.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return FALSE;
    }
  }
  
  /* see if we have enough move points left */
  if (!ParseCommandCheck(PARSE_COMMAND_WGOTO, sock , "") && Character(thing)->cLevel>=LEVEL_MOVETIRING) {
    /* determine terrain cost */
    if (BIT(Character(thing)->cAffectFlag, AF_PHASEWALK))
      cost = 1;
    else {
      THING*startWld = Base(thing)->bInside;
      LWORD startType = Wld(startWld)->wType;
      LWORD endType = Wld(exit->eWorld)->wType;
      cost = (wTypeList[startType].wTMoveCost + wTypeList[endType].wTMoveCost)/2;
    }

    if (BIT(Character(thing)->cAffectFlag, AF_ENDURANCE))
      cost /= 2;
    if (thing->tType == TTYPE_PLR) {
      /* Adjust Move costs for encumbrance  */
      cost = CharMoveCostAdjust(thing, cost);
    }    

    /* Check if we can do it */
    if( cost > ( CharGetMovePMax( thing ) ) ) {
      SendAction("^wYou're carrying too much weight and cant move!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      SendHint("^;HINT: Try ^<Dropping or sell^;'ing some of the things your carrying!\n", thing);
      return FALSE;
    } else if (cost > Character(thing)->cMoveP) {
      SendAction("^wYou're too exhausted to move!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      SendHint("^;HINT: Try ^<SLEEP^;'ing for a little while!\n", thing);
      return FALSE;
    } else {
      Character(thing)->cMoveP -= cost;
    }
  }
  
  if (Character(thing)->cFight)
    FightMove(thing, exit);
 
  if (BIT(Character(thing)->cAffectFlag, AF_HIDDEN)) {
    BITCLR(Character(thing)->cAffectFlag, AF_HIDDEN);
    SendAction("^YYou step out of the shadows\n", 
      thing, NULL, SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);
    SendAction("^Y$n steps out of the shadows\n", 
      thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  }
 
  /* Send the move messages and move the thing to the next room */
  ExitGetName(exit, eName);
  from = Base(thing)->bInside;
  to = exit->eWorld;

  /* Sneak or normal move? */
  if (!BIT(Character(thing)->cAffectFlag, AF_SNEAK)) {
    if (exit->eDir != EDIR_UNDEFINED) {
      sprintf(buf, "^CYou leave %s.\n", eName);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^c$n leaves %s.\n", eName);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    } else {
      sprintf(buf, "^CYou leave via the %s.\n", eName);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^c$n leaves via the %s.\n", eName);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }

    ThingTo(thing, exit->eWorld);                       
    sprintf(buf, "^c$n saunters in.\n");
    SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    CmdLook(thing, "");
  
  } else {
    THING *room;
    LWORD perception;
    LWORD check;
    LWORD sneak;

    if (exit->eDir != EDIR_UNDEFINED) {
      sprintf(buf, "^CYou sneak %s.\n", eName);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
    } else {
      sprintf(buf, "^CYou sneak out via the %s.\n", eName);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
    }
    sneak = CharGetSneak(thing);
    for (room = from->tContain; room; room = room->tNext) {
      if (room->tType < TTYPE_CHARACTER) continue;
      perception = CharGetPerception(room);
      check = DiceOpenEnded(1, 100, 100);

      if (perception + 50 > sneak + check) {
        if (exit->eDir != EDIR_UNDEFINED) {
          sprintf(buf, "^c$n sneaks %s.\n", eName);
          SendAction(buf, thing, room, SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        } else {
          sprintf(buf, "^c$n sneaks out via the %s.\n", eName);
          SendAction(buf, thing, room, SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        }
      }
    }

    ThingTo(thing, to);                       
    for (room = to->tContain; room; room = room->tNext) {
      if (room->tType < TTYPE_CHARACTER) continue;
      perception = CharGetPerception(room);
      check = DiceOpenEnded(1, 100, 100);

      if (perception + 50 > sneak + check) {
        if (exit->eDir != EDIR_UNDEFINED) {
          sprintf(buf, "^c$n sneaks %s.\n", eName);
          SendAction(buf, thing, room, SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        } else {
          sprintf(buf, "^c$n sneaks out via the %s.\n", eName);
          SendAction(buf, thing, room, SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        }
      }
    }
    CmdLook(thing, "");
  
  }

  /* Check for VACUUM here */
  if (Base(thing)->bInside->tType == TTYPE_WLD) {
    if ( (BIT(Wld(Base(thing)->bInside)->wFlag, WF_VACUUM)
        ||Wld(Base(thing)->bInside)->wType==WT_VACUUM)
    && !BIT(Character(thing)->cAffectFlag, AF_VACUUMWALK)){
      damage = Number(1, CharGetHitPMax(thing)/3);
      MINSET(damage, 1);
      /* if its killed by poison whoever its fighting gets the xp */
      SendAction("^rYou're suffering explosive decompression!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      if (FightDamagePrimitive(Character(thing)->cFight, thing, damage, "Explosive Decompression")) 
        return FALSE;
    }
  }
  
  /* Check for UNDERWATER here */
  if (Base(thing)->bInside->tType == TTYPE_WLD) {
    if ( Wld(Base(thing)->bInside)->wType==WT_UNDERWATER
     && !BIT(Character(thing)->cAffectFlag, AF_BREATHWATER) 
     && !BIT(Character(thing)->cAffectFlag, AF_VACUUMWALK) ){      
      damage = Number(1, CharGetHitPMax(thing)/5);
      MINSET(damage, 1);
      /* if its killed by poison whoever its fighting gets the xp */
      SendAction("^rYou're drowning!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      if (FightDamagePrimitive(Character(thing)->cFight, thing, damage, "Drowning")) 
        return FALSE;
    }
  }
  
  /* Be a bummer if we're dead now.... */
  if (BIT(Wld(to)->wFlag, WF_DEATHTRAP)) {
    PlayerKillDeathTrap( thing , to, from );
    return FALSE;
  }
  
  /* Scan for hostiles - autoattack other aggressives */
  if (thing->tType==TTYPE_PLR && BIT(Plr(thing)->pAuto, PA_AUTOAGGR)) {
    for (i=Base(thing)->bInside->tContain; i; i=i->tNext) {
      if (i->tType==TTYPE_MOB ) {
        if( BIT(Mob(i)->mTemplate->mAct, MACT_AGGRESSIVE) ||
            BIT(Mob(i)->mTemplate->mAct, MACT_HYPERAGGR) ) {
          FightStart(thing, i, 1, NULL);
        }
      }
    }
  }
  
  thing->tWait++; /* make it so they cant initiate a command for a "round" */
  
  return TRUE;
}

/* Needed another layer for script event blocking */
BYTE MoveExitEnterStub(THING *thing, EXIT *exit) {
  THING  *i;
  BYTE    retValue = FALSE;
  BYTE    block = FALSE;
  THING  *from;
  THING  *to;
  
  if (!thing || !exit) return FALSE;
 
  /* Worlds are static, no worries about storing these pointers */
  from = Base(thing)->bInside;
  to = exit->eWorld;

  ThingSetEvent(thing);
  ThingSetEvent(exit);

  /**************
   * @ENTRY event 
   **************/
  /* check the room for a script */
  block = CodeParseEntry(thing, exit->eWorld, exit);
  /* check other things within the room for scripts */
  for (i=to->tContain; i && !block; i=i->tNext) 
    BITSET(i->tFlag, TF_CODETHING);
  while ( (i = ThingFindFlag(to->tContain, TF_CODETHING)) ) {
    BITCLR(i->tFlag, TF_CODETHING);
    if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
      block = TRUE; /* Nothing left to do */
    if (!block) block = CodeParseEntry(thing, i, exit);
  }
  /* see if area wide script intercepts */
  if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
    block = TRUE; /* Nothing left to do */
  if (!block)
    block = CodeParseEntry(thing, &areaList[Wld(to)->wArea].aResetThing, exit);

  /***********************
   * Perform the Movement 
   ***********************/
  if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
    block = TRUE; /* Nothing left to do */
  if (!block) retValue = MoveExitPrimitive(thing, exit);


  /**************
   * @AFTERENTRY event 
   **************/
  if (retValue && !block) {
    /* Check the room they are in */
    if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
      block = TRUE; /* Nothing left to do */
    if (!block) CodeParseAfterEntry(thing, to, exit);
    
    /* check for people in the room */
    for (i=to->tContain; i && !block; i=i->tNext) 
      BITSET(i->tFlag, TF_CODETHING);
    while ( (i = ThingFindFlag(to->tContain, TF_CODETHING)) ) {
      BITCLR(i->tFlag, TF_CODETHING);
      if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
        block = TRUE; /* Nothing left to do */
      if (!block) CodeParseAfterEntry(thing, i, exit);
    }
  
    /* see if area wide script intercepts */
    if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
      block = TRUE; /* Nothing left to do */
    if (!block)
      CodeParseAfterEntry(thing, &areaList[Wld(to)->wArea].aResetThing, exit);
  } 

  if (ThingIsEvent(thing)) {
    if (block) retValue = (Base(thing)->bInside == to);
    ThingDeleteEvent(thing);
  }
  ThingDeleteEvent(exit);
  return retValue;
}


/* ultimately calls MoveExitPrimitive 
   Entry point to move a single person, notably
   used by the flee command */ 
BYTE MoveExit(THING *thing, EXIT *exit) {
  THING  *i;
  BYTE    retValue = FALSE;
  BYTE    block = FALSE;
  THING  *from;
  THING  *to;
  
  if (!thing || !exit) return FALSE;
  if (thing->tType < TTYPE_BASE) return FALSE;
  if (!Base(thing)->bInside) return FALSE;
  if (Base(thing)->bInside->tType!=TTYPE_WLD) return FALSE;
  if (!exit->eWorld) return FALSE;
 
  /* Worlds are static, no worries about storing these pointers */
  from = Base(thing)->bInside;
  to = exit->eWorld;

  /* Compile on demand */
  if (to->tType == TTYPE_WLD) {
    PROPERTY *p;
    if (BIT(to->tFlag, TF_COMPILE)) {
      BITCLR(to->tFlag, TF_COMPILE);
      for (p=to->tProperty; p; p=p->pNext) {
        if (!CodeCompileProperty(p, NULL))
          CodeSetFlag(to, p);
      }
    }
  }

  ThingSetEvent(thing);
  ThingSetEvent(exit);
    
  /**************
   * @EXIT event 
   **************/
  /* check the room for a script */
  block = CodeParseExit(thing, Base(thing)->bInside, exit);
  /* check other things within the room for scripts */
  for (i=from->tContain; i && !block; i=i->tNext) 
    BITSET(i->tFlag, TF_CODETHING);
  while ( (i = ThingFindFlag(from->tContain, TF_CODETHING)) ) {
    BITCLR(i->tFlag, TF_CODETHING);
    if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
      block = TRUE; /* Nothing left to do */
    if (!block)
      block = CodeParseExit(thing, i, exit);
  }
  /* see if area wide script intercepts */
  if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
    block = TRUE; /* Nothing left to do */
  if (!block)
    block = CodeParseExit(thing, &areaList[Wld(from)->wArea].aResetThing, exit);


  /********************************
   *  Check out the Entry events 
   ********************************/
  if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
    block = TRUE; /* Nothing left to do */
  if (!block) retValue = MoveExitEnterStub(thing, exit);

  
  /**************
   * @AFTEREXIT event 
   **************/
  if (retValue && !block) {
    /* Check the room they are in */
    if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
      block = TRUE; /* Nothing left to do */
    if (!block) CodeParseAfterExit(thing, from, exit);
    
    /* check for people in the room */
    for (i=from->tContain; i&&!block; i=i->tNext) 
      BITSET(i->tFlag, TF_CODETHING);
    while ( (i = ThingFindFlag(from->tContain, TF_CODETHING)) ) {
      BITCLR(i->tFlag, TF_CODETHING);
      if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
        block = TRUE; /* Nothing left to do */
      if (!block) CodeParseAfterExit(thing, i, exit);
    }
  
    /* see if area wide script intercepts */
    if (!block && (!ThingIsEvent(thing) || !ThingIsEvent(exit)))
      block = TRUE; /* Nothing left to do */
    if (!block)
      CodeParseAfterExit(thing, &areaList[Wld(from)->wArea].aResetThing, exit);
  } 

  if (ThingIsEvent(thing)) {
    if (block) retValue = (Base(thing)->bInside == to);
    ThingDeleteEvent(thing);
  }
  ThingDeleteEvent(exit);
  return retValue;
}

/* ****************************************************
 * If you want to move somebody through an exit, this is what
 * you should be calling.

 * - stops if somebody cant move through the exit
 * - Entry point for moving entire groups, blocks
 * if they are busy fighting someone
 * - scripts are checked in ROOM, OBJECTS IN ROOM, 
 * then AREAWIDE order
 * - parms are verified just before each check and 
 * if thing or exit has ceased to exist the whole 
 * movement process is aborted
 
 * The moving process....
 * MoveExitParse {       (<-- Call this function to move something)
 *   Check @EXIT
 *   MoveExitEnterStub {
 *     Check @ENTRY
 *     MovePrimitive     ( does the move, uses mv pts up etc )
 *     Check @AFTERENTRY
 *   }
 *   Check @AFTEREXIT
 * }
 
 */
void MoveExitParse(THING *thing, EXIT *exit) {
  THING  *i;
  THING  *wasIn;
  BYTE    possibleSuccess;
  BYTE    eName[256];
  
  if (!thing || !exit) return;
  if (thing->tType < TTYPE_BASE) return;
  if (!Base(thing)->bInside) return;
  if (!exit->eWorld) return;

  if (Character(thing)->cPos==POS_FIGHTING) {
    if (!Character(thing)->cFightExit) {
      SendAction("^cYou're right in the middle of a fight here! If you want to leave ^wFLEE!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    } else {
      if (exit != Character(thing)->cFightExit) {
        ExitGetName(exit, eName);
        SendAction("^cYou're in the middle of a fight here! Either go ^p", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(eName, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(" or ^wFLEE!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
    }
  }
  
  /* Move everybody else in the group if they can move */
  wasIn = Base(thing)->bInside;
  possibleSuccess = MoveExit(thing, exit);
  i = Character(thing)->cFollow;
  while (possibleSuccess && i) {
    if (Base(i)->bInside == wasIn) {
      /* Block out fighting chars from moving */
      if (Character(i)->cPos<POS_STANDING) {
        if (Character(i)->cPos==POS_FIGHTING
         && Character(i)->cFightExit == exit) {
          possibleSuccess = MoveExit(i, exit);
        } else {
          possibleSuccess = FALSE;
        }
      } else {
        possibleSuccess = MoveExit(i, exit);
      }
    }
    i = Character(i)->cFollow;
  }
}

void MoveDir(THING *thing, BYTE dir) {
  EXIT *exit;

  
  if (Base(thing)->bInside->tType != TTYPE_WLD) {
    SendAction("There doesnt appear to be anything in that direction.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  exit = ExitDir(Wld(Base(thing)->bInside)->wExit, dir);
  if (!exit) {
    SendAction("There doesnt appear to be anything in that direction.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  MoveExitParse(thing, exit);
}

CMDPROC(CmdGo) { /* void CmdProc(THING *thing, BYTE *cmd) */
  EXIT *exit;
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
   
  cmd = StrOneWord(cmd, NULL); /* drop command */
  
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);
  if (Base(thing)->bInside->tType != TTYPE_WLD) {
    SendAction("There doesnt appear to be anything in that direction.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  exit = ExitFind(Wld(Base(thing)->bInside)->wExit, srcKey);
  if (exit && srcOffset>1) {
    exit = NULL;
    srcOffset--;
  }
  if (!exit) {
    /* Look for an exit object */
    found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_OBJ, &srcOffset);
    if (found) {
      if (Obj(found)->oTemplate->oType == OTYPE_EXIT
        ||Obj(found)->oTemplate->oType == OTYPE_VEHICLE
      ) {
        LWORD  wVirtual;
        THING *wld;
        wVirtual = OBJECTGETFIELD(found, OF_EXIT_WVIRTUAL);
        wld = WorldOf(wVirtual);
        if (!wld) {
          SendAction("That doesnt seem to go anywhere.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
          return;
        }
        /* make an exit, move thru it than turf it */
        exit = ExitAlloc(NULL, EDIR_UNDEFINED, StrAlloc(Base(found)->bKey), StrAlloc(found->tDesc), 0, -1, wld);
        MoveExitParse(thing, exit);
        ExitFree(NULL, exit);

#ifdef UGLY_VEHICLE
      } else if (Obj(found)->oTemplate->oType == OTYPE_VEHICLE) {
        THING *wld;
        wld = (THING*) Obj(found)->oDetail.lValue[3];
        if (!wld) {
          SendAction("That doesnt seem to go anywhere.\n", thing);
          return;
        }
        /* make an exit, move thru it than turf it */
        exit = ExitAlloc(NULL, EDIR_UNDEFINED, StrAlloc(Base(found)->bKey), StrAlloc(found->tDesc), 0, -1, wld);        
        MoveExitParse(thing, exit);
        ExitFree(NULL, exit);
#endif

      } else {
        SendAction("How exactly would that work now?.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
    } else {
      SendAction("There doesnt appear to be anything in that direction.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    }
    return;
  }
  MoveExitParse(thing, exit);
}

CMDPROC(CmdNorth) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_NORTH);
}

CMDPROC(CmdEast) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_EAST);
}

CMDPROC(CmdSouth) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_SOUTH);
}

CMDPROC(CmdWest) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_WEST);
}

CMDPROC(CmdUp) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_UP);
}

CMDPROC(CmdDown) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_DOWN);
}

CMDPROC(CmdOut) { /* void CmdProc(THING thing, BYTE* cmd) */
  MoveDir(thing, EDIR_OUT);
}

CMDPROC(CmdSneak) { /* void CmdProc(THING thing, BYTE* cmd) */
  BYTE *next;
  LWORD sneak;

  if (CharGetSneak(thing) <= 0) {
    SendAction("^wI'm afraid you're not too good at that sort of thing\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  next = StrOneWord(cmd, NULL);
  if (!*next) {
    /* toggle sneak */
    if (BIT(Character(thing)->cAffectFlag, AF_SNEAK)) {
      SendAction("^wYou resume walking around normally\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    } else {
      SendAction("^wYou start sneaking around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    }
    BITFLIP(Character(thing)->cAffectFlag, AF_SNEAK);
    return;
  }

  sneak = FALSE;
  if (BIT(Character(thing)->cAffectFlag, AF_SNEAK))
    sneak = TRUE;
  BITSET(Character(thing)->cAffectFlag, AF_SNEAK);
  CmdGo(thing, cmd);
  if (!sneak) BITCLR(Character(thing)->cAffectFlag, AF_SNEAK);
}

CMDPROC(CmdLook) { /* CmdLook(thing,cmd) */
  SOCK  *sock;
  BYTE   buf[256];
  BYTE   srcKey[256];
  BYTE   eName[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *show;
  EXTRA *extra;
  EXIT  *exit;
  BYTE   showDesc = TRUE;
  BYTE   canSee;
  LWORD  eOrderNum;

  sock = BaseControlFind(thing);
  if (!sock) return;
  search = Base(thing)->bInside;
  cmd = StrOneWord(cmd, buf); /* lose the command at the start */
  if (!*cmd) {
    canSee = ThingCanSee(thing, NULL);
    /* Show the room */
    SendAction("^0",thing, NULL, SEND_SRC|SEND_VISIBLE );
    SendAction(search->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
    if (*buf == '\0' && !BIT(Plr(sock->sHomeThing)->pAuto, PA_AUTOLOOK)) {
      SendAction("\n",thing, NULL, SEND_SRC|SEND_VISIBLE );
    } else if (canSee==TCS_SEENORMAL) {
      SendAction("\n^1",thing, NULL, SEND_SRC|SEND_VISIBLE );
      SendAction(search->tDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
    } else
      SendAction("\n",thing, NULL, SEND_SRC|SEND_VISIBLE );

    /* See if its too dark */
    if (canSee == TCS_CANTSEE)
      SendAction("^yIts pitch black\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    else if (canSee == TCS_SEEPARTIAL)
      SendAction("^yIts so dark you can only dimly make things out\n", thing, NULL, SEND_SRC|SEND_VISIBLE );

    /* show the Exits */
    if (BIT(Plr(sock->sHomeThing)->pAuto, PA_AUTOEXIT)) {
      sprintf(buf, "^cVisible Exits: [");
      /* ok, now we could just go through the exit list, but it 
       * would be nice to have a SORTED list. */
      for (eOrderNum=0;eOrderList[eOrderNum]<EDIR_MAX;eOrderNum++) {
        for (exit = Wld(search)->wExit; exit; exit=exit->eNext) {
          if (exit->eWorld && !BIT(exit->eFlag, EF_HIDDEN)&&
        (exit->eDir==eOrderList[eOrderNum])) {
            ExitGetName(exit, eName);
            if (BIT(exit->eFlag, EF_CLOSED))
              sprintf(buf+strlen(buf), "^r%s ", eName);
            else
              sprintf(buf+strlen(buf), "^b%s ", eName);
          }
        }
      }
      if (buf[strlen(buf)-1] == ' ')
        buf[strlen(buf)-1] = '\0';
      strcat(buf, "^c]\n");
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
    }
    
    /* show the things in the room */
    show = search->tContain;
    while(show) {
      if (show != thing) {
        show = ThingShow(show, thing);
      } else
        show = show->tNext;
    }
    return;
  }
  
  StrOneWord(cmd, buf); /* see if they tried to look in or inside something */
  if (!STRICMP(buf, "in") || !STRICMP(buf, "inside")) {
    cmd = StrOneWord(cmd, NULL);
    showDesc = FALSE;
  }
  
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);
  show = CharThingFind(thing, srcKey, -1, thing, TF_PLR|TF_OBJ|TF_MOB, &srcOffset);
  if (!show)
    show = CharThingFind(thing, srcKey, -1, search, TF_PLR|TF_OBJ|TF_MOB, &srcOffset);
  if (show) {
    ThingShowDetail(show, thing, showDesc);
    /* if we arent looking at stuff we're holding let people see the action has occured */
    if (Base(show)->bInside == Base(thing)->bInside) {
      SendAction("^b$n examines you closely\n", 
        thing, show, SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^C$n examines $N closely\n", 
        thing, show, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }
    return;
  }
  /* Look extras */
  extra = ExtraFindWithin(search, srcKey, &srcOffset);
  if (extra && srcOffset<=0) {
    SendAction("^b", thing, NULL, SEND_SRC|SEND_VISIBLE );
    SendAction(extra->eDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  
  /* Look Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    SendAction("^b", thing, NULL, SEND_SRC|SEND_VISIBLE );
    if (*exit->eDesc->sText)
      SendAction(exit->eDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
    else
      SendAction("You see nothing special\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  
  /* well guess theres nothing else to look for */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}

/* scan adjacent rooms */
BYTE  *scanAdjective[] = {
  "",
  "Far to the ",
  "Way Far "
};

CMDPROC(CmdScan) { /* CmdProc(thing,cmd) */
  BYTE   buf[256];
  BYTE   word[256];
  BYTE   eName[256];
  EXIT  *exit;
  THING *scan;
  LWORD  i;
  
  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */
  if (!*cmd) {
    /* show the Exits */
    SendAction("^wScanning all around..\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    for (exit = Wld(Base(thing)->bInside)->wExit; exit; exit=exit->eNext) {
      if (exit->eWorld && !BIT(exit->eFlag, EF_HIDDEN)) {
        ExitGetName(exit, eName);
        sprintf(buf, "^b[%s]\n", eName);
        SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
        for (scan = exit->eWorld->tContain; scan; ) {
          scan = ThingShow(scan, thing);
        }
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
    }
    return;
  }
  
  /* Scan in a particular direction here */
  cmd = StrOneWord(cmd, word); /* Get the keyword */
  sprintf(buf, "^wScanning %s...\n", word);
  SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
  scan = Base(thing)->bInside;
  for (i=0; i<3; i++) { /* scan this many rooms away */
    exit = ExitFind(Wld(scan)->wExit, word);
    if (!exit) return;
    ExitGetName(exit, eName);
    sprintf(buf, "^b[%s%s]\n", scanAdjective[i], eName);
    SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
    for (scan = exit->eWorld->tContain; scan; ) {
      scan = ThingShow(scan, thing);
    }
    SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    scan = exit->eWorld;
  }
}

CMDPROC(CmdExit) { /* CmdProc(thing,cmd) */
  EXIT  *exit;
  SOCK  *sock;
  BYTE   buf[256];
  BYTE   eName[256];

  sock = BaseControlFind(thing);

  /* Show the obvious exits */
  SendAction("^bAmong the obvious exits, include such diverse elements as:\n",thing, NULL, SEND_SRC|SEND_VISIBLE );

  /* show the Exits */
  for (exit = Wld(Base(thing)->bInside)->wExit; exit; exit=exit->eNext) {
    if (!BIT(exit->eFlag, EF_HIDDEN)) {
      ExitGetName(exit, eName);
      sprintf(buf, "^g%-7s- ^c", eName);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      if (exit->eDesc->sLen>1)
        SendAction(exit->eDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
      else if (exit->eWorld) {
        SendAction(exit->eWorld->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      }
    }
  }

}

CMDPROC(CmdOpen) { /* CmdProc(thing,cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *open;
  EXIT  *exit;
  FLAG   oFlag;
  BYTE   word[256];
  BYTE   buf[256];

  cmd = StrOneWord(cmd, NULL); /* lose command at start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  /* search ourself first */
  search = thing;
  open = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!open) { /* also search the room */
    search = Base(thing)->bInside;
    open = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  }
  
  if (open) {
    if (Obj(open)->oTemplate->oType == OTYPE_CONTAINER) {
      oFlag = OBJECTGETFIELD(open, OF_CONTAINER_CFLAG);
      if (!BIT(oFlag, OCF_CLOSED)) {
        SendAction("^wBoy, how difficult that's going to be, since its allready open....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      if (!BIT(oFlag, OCF_LOCKED)) {
        BITCLR(oFlag, OCF_CLOSED);
        ObjectSetField(OTYPE_CONTAINER, &Obj(open)->oDetail, OF_CONTAINER_CFLAG, oFlag);
        SendAction("^gYou open ", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(open->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("$n opens $N\n", thing, open, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
        SoundDoorOpen( thing, 0, 0 );
        return;
      } else {
        SendAction("^gYou cant open that, its LOCKED!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SoundDoorLocked( thing, SOUND_DOOR_LOCKED_LOW, 0 );
        return;
      }
    } else {
      SendAction("^wAnd how on earth are you supposed to do that, exactly?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* search for Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    if (!BIT(exit->eFlag, EF_ISDOOR)) {
      SendAction("^wYou cant open ^rTHAT\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    if (!BIT(exit->eFlag, EF_CLOSED)) {
      SendAction("^wBoy, how difficult that's going to be, since its allready open....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    if (BIT(exit->eFlag, EF_LOCKED)) {
      SendAction("^wIt's locked.\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      SoundDoorLocked( thing, SOUND_DOOR_LOCKED_LOW, 0 );
      return;
    }
    if (*exit->eKey->sText)
      StrOneWord(exit->eKey->sText, word);
    else
      strcpy(word, "door");
    sprintf(buf, "^gYou open the %s\n", word);
    SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
    sprintf(buf, "^g$n opens the %s\n", word);
    SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    SoundDoorOpen( thing, 0, 0 );
  
    BITCLR(exit->eFlag, EF_CLOSED);
    exit = ExitReverse(search, exit);
    if (exit)
      BITCLR(exit->eFlag, EF_CLOSED);
    return;
  }
  
  /* well guess theres nothing else to open */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}

CMDPROC(CmdClose) { /* CmdProc(thing,cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *close;
  EXIT  *exit;
  FLAG   oFlag;
  BYTE   word[256];
  BYTE   buf[256];

  cmd = StrOneWord(cmd, NULL); /* lose command at start */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  /* search ourself first */
  search = thing;
  close = CharThingFind(thing, srcKey, -1, search, TF_OBJINV, &srcOffset);
  if (!close) { /* also search the room */
    search = Base(thing)->bInside;
    close = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  }
  
  if (close) {
    if (Obj(close)->oTemplate->oType == OTYPE_CONTAINER) {
      oFlag = OBJECTGETFIELD(close, OF_CONTAINER_CFLAG);
      if (!BIT(oFlag, OCF_CLOSABLE)) {
        SendAction("^wYou cant close ^rTHAT\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      if (BIT(oFlag, OCF_CLOSED)) {
        SendAction("^wBoy, how difficult that's going to be, since its allready closed...\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      } else {
        BITSET(oFlag, OCF_CLOSED);
        ObjectSetField(OTYPE_CONTAINER, &Obj(close)->oDetail, OF_CONTAINER_CFLAG, oFlag);
        SendAction("^gYou close ", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(close->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("$n closes $N\n", thing, close, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
        SoundDoorClose( thing, 0, 0 );
        return;
      }
    } else {
      SendAction("^wAnd how on earth are you supposed to do that, exactly?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }

  /* search for Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    if (!BIT(exit->eFlag, EF_ISDOOR)) {
      SendAction("^wYou cant close ^rTHAT\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    if (BIT(exit->eFlag, EF_CLOSED)) {
      SendAction("^wBoy, how difficult that's going to be, since its allready closed....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    } else {
      if (*exit->eKey->sText)
        StrOneWord(exit->eKey->sText, word);
      else
        strcpy(word, "door");
      sprintf(buf, "^gYou close the %s\n", word);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^g$n closes the %s\n", word);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      SoundDoorClose( thing, 0, 0 );

      BITSET(exit->eFlag, EF_CLOSED);
      exit = ExitReverse(search, exit);
      if (exit)
        BITSET(exit->eFlag, EF_CLOSED);
      return;
    }
  }

  /* well guess theres nothing else to close */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}

CMDPROC(CmdUnlock) { /* CmdProc(thing,cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *unlock;
  THING *key;
  LWORD  unlockNumber;
  EXIT  *exit;
  FLAG   oFlag;
  BYTE   word[256];
  BYTE   buf[256];
  
  cmd = StrOneWord(cmd, NULL); /* lose command at start */
  search = Base(thing)->bInside;
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);
  unlock = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (unlock) {
    if (Obj(unlock)->oTemplate->oType == OTYPE_CONTAINER) {
      oFlag = OBJECTGETFIELD(unlock, OF_CONTAINER_CFLAG);
      if (!BIT(oFlag, OCF_LOCKED)) {
        SendAction("^wBoy, how difficult that's going to be, since its allready unlocked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      
      /* find if we have the key */
      unlockNumber = OBJECTGETFIELD(unlock, OF_CONTAINER_KEY);
      for(key = thing->tContain; key; key = key->tNext) {
        if (key->tType == TTYPE_OBJ && Obj(key)->oTemplate->oType == OTYPE_KEY) {
          if (Obj(key)->oTemplate->oVirtual == unlockNumber)
            break;
          if (OBJECTGETFIELD(key, OF_KEY_NUMBER) == unlockNumber)
            break;
        }
      }
      
      if (key) {
        BITCLR(oFlag, OCF_LOCKED);
        ObjectSetField(OTYPE_CONTAINER, &Obj(unlock)->oDetail, OF_CONTAINER_CFLAG, oFlag);
        SendAction("^gYou unlock ", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(unlock->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("$n unlocks $N\n", thing, unlock, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
        SoundDoorLocked( thing, SOUND_DOOR_UNLOCK_LOW, 0 );        
        return;
      } else {
        SendAction("^wI'm afraid you dont have anything that you can use to unlock it\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
    } else {
      SendAction("^wAnd how on earth are you supposed to do that, exactly?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* search for Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    if (!BIT(exit->eFlag, EF_ISDOOR)) {
      SendAction("^wYou cant unlock ^rTHAT!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    if (!BIT(exit->eFlag, EF_LOCKED)) {
      SendAction("^wBoy, how difficult that's going to be, since its allready unlocked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    /* Do we have the key */
    unlockNumber = exit->eKeyObj;
    for(key = thing->tContain; key; key = key->tNext) {
      if (key->tType == TTYPE_OBJ && Obj(key)->oTemplate->oType == OTYPE_KEY) {
        if (Obj(key)->oTemplate->oVirtual == unlockNumber)
          break;
        if (OBJECTGETFIELD(key, OF_KEY_NUMBER) == unlockNumber)
          break;
      }
    }
    
    if (key) {
      if (*exit->eKey->sText)
        StrOneWord(exit->eKey->sText, word);
      else
        strcpy(word, "door");
      sprintf(buf, "^gYou unlock %s\n", word);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^g$n unlocks %s\n", word);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      SoundDoorLocked( thing, SOUND_DOOR_UNLOCK_LOW, 0 );
      BITCLR(exit->eFlag, EF_LOCKED);
      exit = ExitReverse(search, exit);
      if (exit)
        BITCLR(exit->eFlag, EF_LOCKED);
      return;
    } else {
      SendAction("^wI'm afraid you dont have anything that you can use to unlock it\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* well guess theres nothing else to unlock */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}


CMDPROC(CmdLock) { /* CmdProc(thing,cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *lock;
  THING *key;
  LWORD  lockNumber;
  EXIT  *exit;
  FLAG   oFlag;
  BYTE   word[256];
  BYTE   buf[256];
  
  cmd = StrOneWord(cmd, NULL); /* lose command at start */
  search = Base(thing)->bInside;
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);
  lock = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (lock) {
    if (Obj(lock)->oTemplate->oType == OTYPE_CONTAINER) {
      oFlag = OBJECTGETFIELD(lock, OF_CONTAINER_CFLAG);
      if (BIT(oFlag, OCF_LOCKED)) {
        SendAction("^wBoy, how difficult that's going to be, since its allready locked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      if (!BIT(oFlag, OCF_CLOSED)) {
        SendAction("^wIt has to be closed before you can lock it\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      
      /* find if we have the key */
      lockNumber = OBJECTGETFIELD(lock, OF_CONTAINER_KEY);
      for(key = thing->tContain; key; key = key->tNext) {
        if (key->tType == TTYPE_OBJ && Obj(key)->oTemplate->oType == OTYPE_KEY) {
          if (Obj(key)->oTemplate->oVirtual == lockNumber)
            break;
          if (OBJECTGETFIELD(key, OF_KEY_NUMBER) == lockNumber)
            break;
        }
      }
      
      if (key) {
        BITSET(oFlag, OCF_LOCKED);
        ObjectSetField(OTYPE_CONTAINER, &Obj(lock)->oDetail, OF_CONTAINER_CFLAG, oFlag);
        SendAction("^gYou lock ", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(lock->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("$n locks $N\n", thing, lock, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
        SoundDoorLocked( thing, SOUND_DOOR_UNLOCK_LOW, 0 );      
        return;
      } else {
        SendAction("^wI'm afraid you dont have anything that you can use to lock it\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
    } else {
      SendAction("^wAnd how on earth are you supposed to do that, exactly?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* search for Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    if (!BIT(exit->eFlag, EF_ISDOOR)) {
      SendAction("^wYou cant lock ^rTHAT!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    if (BIT(exit->eFlag, EF_LOCKED)) {
      SendAction("^wBoy, how difficult that's going to be, since its allready locked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    if (!BIT(exit->eFlag, EF_CLOSED)) {
      SendAction("^wIt has to be closed before it can be locked\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }

    /* Do we have the key */
    lockNumber = exit->eKeyObj;
    for(key = thing->tContain; key; key = key->tNext) {
      if (key->tType == TTYPE_OBJ && Obj(key)->oTemplate->oType == OTYPE_KEY) {
        if (Obj(key)->oTemplate->oVirtual == lockNumber)
          break;
        if (OBJECTGETFIELD(key, OF_KEY_NUMBER) == lockNumber)
          break;
      }
    }
    
    if (key) {
      if (*exit->eKey->sText)
        StrOneWord(exit->eKey->sText, word);
      else
        strcpy(word, "door");
      sprintf(buf, "^gYou lock %s\n", word);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^g$n locks %s\n", word);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      SoundDoorLocked( thing, SOUND_DOOR_UNLOCK_LOW, 0 );
      BITSET(exit->eFlag, EF_LOCKED);
      exit = ExitReverse(search, exit);
      if (exit)
        BITSET(exit->eFlag, EF_LOCKED);
      return;
    } else {
      SendAction("^wI'm afraid you dont have anything that you can use to lock it\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* well guess theres nothing else to lock */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}


CMDPROC(CmdPicklock) { /* CmdProc(thing,cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *unlock;
  BYTE   success;
  LWORD  unlockNumber;
  EXIT  *exit;
  FLAG   oFlag;
  BYTE   word[256];
  BYTE   buf[256];

  thing->tWait++;
  cmd = StrOneWord(cmd, NULL); /* lose command at start */
  search = Base(thing)->bInside;
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);
  unlock = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (unlock) {
    if (Obj(unlock)->oTemplate->oType == OTYPE_CONTAINER) {
      oFlag = OBJECTGETFIELD(unlock, OF_CONTAINER_CFLAG);
      if (!BIT(oFlag, OCF_LOCKED)) {
        SendAction("^wBoy, how difficult that's going to be, since its allready unlocked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      
      if (BIT(oFlag, OCF_ELECTRONIC)) {
        SendAction("^wYou cant pick an electronic lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      SoundDoorLocked( thing, SOUND_DOOR_PICK_LOW, 0 );
      
      /* find if we picked the lock */
      success = FALSE;
      if (thing->tType == TTYPE_PLR && Plr(thing)->pSkill[SKILL_PICKLOCK]>0) {
        unlockNumber = DiceOpenEnded(1, 100, 100);
        unlockNumber += PropertyGetLWord(unlock, "%security", 0);
        if (unlockNumber <= Plr(thing)->pSkill[SKILL_PICKLOCK])
          success = TRUE;
      }
      
      if (success) {
        BITCLR(oFlag, OCF_LOCKED);
        ObjectSetField(OTYPE_CONTAINER, &Obj(unlock)->oDetail, OF_CONTAINER_CFLAG, oFlag);
        SendAction("^gYou pick the lock of the ", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(unlock->tSDesc->sText, thing , NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("$n picks the lock on the $N\n", thing, unlock, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
        SoundDoorLocked( thing, SOUND_DOOR_UNLOCK_LOW, 0 );        
        return;
      } else {
        SendAction("^wI'm afraid you fail to pick the lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
    } else {
      SendAction("^wPicklock what exactly?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* search for Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    if (!BIT(exit->eFlag, EF_ISDOOR)) {
      SendAction("^wYou cant unlock ^rTHAT!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    if (!BIT(exit->eFlag, EF_LOCKED)) {
      SendAction("^wBoy, how difficult that's going to be, since its allready unlocked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    if (BIT(exit->eFlag, EF_ELECTRONIC)) {
      SendAction("^wYou cant pick an electronic lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    SoundDoorLocked( thing, SOUND_DOOR_PICK_LOW, 0 );
    
    /* find if we picked the lock */
    success = FALSE;
    if (thing->tType == TTYPE_PLR && Plr(thing)->pSkill[SKILL_PICKLOCK]>0) {
      unlockNumber = DiceOpenEnded(1, 100, 100);
      unlockNumber += PropertyGetLWord(search, "%security", 0);
      if (unlockNumber <= Plr(thing)->pSkill[SKILL_PICKLOCK])
        success = TRUE;
    }
    
    
    if (success) {
      if (*exit->eKey->sText)
        StrOneWord(exit->eKey->sText, word);
      else
        strcpy(word, "door");
      sprintf(buf, "^gYou pick the lock of the %s\n", word);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^g$n picks the lock of the %s\n", word);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      SoundDoorLocked( thing, SOUND_DOOR_UNLOCK_LOW, 0 );
      
      BITCLR(exit->eFlag, EF_LOCKED);
      exit = ExitReverse(search, exit);
      if (exit)
        BITCLR(exit->eFlag, EF_LOCKED);
      return;
    } else {
      SendAction("^wI'm afraid you fail to pick the lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* well guess theres nothing else to unlock */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}

CMDPROC(CmdHack) { /* CmdProc(thing,cmd) */
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *search;
  THING *unlock;
  BYTE   success;
  LWORD  unlockNumber;
  EXIT  *exit;
  FLAG   oFlag;
  BYTE   word[256];
  BYTE   buf[256];

  thing->tWait++;
  cmd = StrOneWord(cmd, NULL); /* lose command at start */
  search = Base(thing)->bInside;
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);
  unlock = CharThingFind(thing, srcKey, -1, search, TF_OBJ, &srcOffset);
  if (unlock) {
    if (Obj(unlock)->oTemplate->oType == OTYPE_CONTAINER) {
      oFlag = OBJECTGETFIELD(unlock, OF_CONTAINER_CFLAG);
      if (!BIT(oFlag, OCF_LOCKED)) {
        SendAction("^wBoy, how difficult that's going to be, since its allready unlocked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      
      if (!BIT(oFlag, OCF_ELECTRONIC)) {
        SendAction("^wYou cant hack a mechanical lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
      
      /* find if we picked the lock */
      success = FALSE;
      if (thing->tType == TTYPE_PLR && Plr(thing)->pSkill[SKILL_HACKING]>0) {
        unlockNumber = DiceOpenEnded(1, 100, 100);
        unlockNumber += PropertyGetLWord(unlock, "%security", 0);
        if (unlockNumber <= Plr(thing)->pSkill[SKILL_HACKING])
          success = TRUE;
      }
      
      if (success) {
        BITCLR(oFlag, OCF_LOCKED);
        ObjectSetField(OTYPE_CONTAINER, &Obj(unlock)->oDetail, OF_CONTAINER_CFLAG, oFlag);
        SendAction("^gYou hack the lock of the ", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction(unlock->tSDesc->sText, thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        SendAction("$n hacks the lock on the $N\n", thing, unlock, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
        return;
      } else {
        SendAction("^wI'm afraid you fail to hack the lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
        return;
      }
    } else {
      SendAction("^wHack what exactly?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* search for Exits */
  do {
    exit = ExitFind(Wld(search)->wExit, srcKey);
    if (exit) srcOffset--;
  } while (exit && srcOffset>0);
  if (exit && srcOffset<=0) {
    if (!BIT(exit->eFlag, EF_ISDOOR)) {
      SendAction("^wYou cant unlock ^rTHAT!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    if (!BIT(exit->eFlag, EF_LOCKED)) {
      SendAction("^wBoy, how difficult that's going to be, since its allready unlocked....\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    if (!BIT(exit->eFlag, EF_ELECTRONIC)) {
      SendAction("^wYou cant hack a mechanical lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
    
    /* find if we picked the lock */
    success = FALSE;
    if (thing->tType == TTYPE_PLR && Plr(thing)->pSkill[SKILL_PICKLOCK]>0) {
      unlockNumber = DiceOpenEnded(1, 100, 100);
      unlockNumber += PropertyGetLWord(search, "%security", 0);
      if (unlockNumber <= Plr(thing)->pSkill[SKILL_PICKLOCK])
        success = TRUE;
    }
    
    
    if (success) {
      if (*exit->eKey->sText)
        StrOneWord(exit->eKey->sText, word);
      else
        strcpy(word, "door");
      sprintf(buf, "^gYou hack the lock of the %s\n", word);
      SendAction(buf, thing, NULL, SEND_SRC|SEND_VISIBLE );
      sprintf(buf, "^g$n hacks the lock of the %s\n", word);
      SendAction(buf, thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      
      BITCLR(exit->eFlag, EF_LOCKED);
      exit = ExitReverse(search, exit);
      if (exit)
        BITCLR(exit->eFlag, EF_LOCKED);
      return;
    } else {
      SendAction("^wI'm afraid you fail to hack the lock\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
      return;
    }
  }
  
  /* well guess theres nothing else to unlock */
  SendAction("^bThere doesnt appear to be anything like that around\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
}

CMDPROC(CmdStand) { /* CmdProc(thing,cmd) */
  /* stand up first */
  if (Character(thing)->cPos == POS_FIGHTING) {
    SendAction("^wYou're in the middle of a fight here!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_SLEEPING) {
    CmdWake(thing, cmd);
  }
  if (Character(thing)->cPos < POS_RESTING) {
    SendAction("^wNice Try...\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_STANDING) {
    SendAction("^gBut you are allready!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  
  SendAction("^YYou stand up\n", thing, NULL, SEND_SRC |SEND_VISIBLE);
  SendAction("^Y$n stands up\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
  Character(thing)->cPos = POS_STANDING; 
}

CMDPROC(CmdSit) { /* CmdProc(thing,cmd) */
  if (Character(thing)->cPos < POS_SLEEPING) {
    SendAction("^wNice Try...\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_SITTING) {
    SendAction("^gBut you are allready!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_FIGHTING) {
    SendAction("^wYou're in the middle of a fight here!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* wake up first */
  if (Character(thing)->cPos == POS_SLEEPING) {
    CmdWake(thing, cmd);
    /* if it didnt work for some reason return */
    if (Character(thing)->cPos == POS_SLEEPING) return;
  }
  if (Character(thing)->cPos < POS_SITTING) {
    SendAction("^YYou sit up\n", thing, NULL, SEND_SRC |SEND_VISIBLE);
    SendAction("^Y$n sits up\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
  } else {
    SendAction("^YYou sit down\n", thing, NULL, SEND_SRC |SEND_VISIBLE);
    SendAction("^Y$n sits down\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
  }
  Character(thing)->cPos = POS_SITTING; 
}

CMDPROC(CmdRest) { /* CmdProc(thing,cmd) */
  if (Character(thing)->cPos < POS_RESTING) {
    SendAction("^wNice Try...\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_RESTING) {
    SendAction("^gBut you are allready!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_FIGHTING) {
    SendAction("^wYou're in the middle of a fight here!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  
  SendAction("^YYou lie down, and rest awhile\n", thing, NULL, SEND_SRC |SEND_VISIBLE);
  SendAction("^Y$n lies down and starts resting\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
  Character(thing)->cPos = POS_RESTING; 
}

CMDPROC(CmdSleep) { /* CmdProc(thing,cmd) */
  if (Character(thing)->cPos < POS_RESTING) {
    SendAction("^wNice Try...\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_SLEEPING) {
    SendAction("^gBut you are allready!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos == POS_FIGHTING) {
    SendAction("^wYou're in the middle of a fight here!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* lie down first */
  if (Character(thing)->cPos > POS_RESTING) {
    CmdRest(thing, cmd);
    /* if they couldnt return */
    if (Character(thing)->cPos > POS_RESTING) return;
  } 
  SendAction("^YYou go to sleep\n", thing, NULL, SEND_SRC |SEND_VISIBLE);
  SendAction("^Y$n goes to sleep\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
  Character(thing)->cPos = POS_SLEEPING; 
}

CMDPROC(CmdWake) { /* CmdProc(thing,cmd) */
  if ( BIT(Character(thing)->cAffectFlag, AF_SLEEP ) || 
       Character(thing)->cPos < POS_SLEEPING) {
    SendAction("^wYou cant seem to wake up!!...\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  if (Character(thing)->cPos > POS_SLEEPING) {
    SendAction("^wYou are allready awake!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
  
  Character(thing)->cPos = POS_RESTING; 
  SendAction("^YYou wake up\n", thing, NULL, SEND_SRC |SEND_VISIBLE);
  SendAction("^Y$n wakes up\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
}

CMDPROC(CmdTrack) { /* Cmd(THING *thing, BYTE *cmd) */
  BYTE   srcKey[256];
  BYTE   eName[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
  EXIT  *exit;
  LWORD  trackRange;
 
  cmd = StrOneWord(cmd, NULL); /* lose the command at the start */

  if (!Base(thing)->bInside || Base(thing)->bInside->tType != TTYPE_WLD)
    return;

  /* Show where all the players are */
  if (!*cmd) {
    SendAction("^wWho was it you wanted to track again?\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }

  /* show matching mobs and plrs in the same area */
  cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

  found = CharThingFind(thing, srcKey, -1, Base(thing)->bInside, TF_MOB|TF_PLR|TF_MOB_WLD|TF_PLR_WLD, &srcOffset);
  if (!found) {
    SendAction("^wTrack who?, ^gNever heard of 'em\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  } 
  
  trackRange = 1;
  if (thing->tType == TTYPE_PLR)
    trackRange = Plr(thing)->pSkill[SKILL_TRACK];
  else if (thing->tType == TTYPE_MOB)
    trackRange = PropertyGetLWord(thing, "%Track", Mob(thing)->mTemplate->mLevel+10)*3;
 
  /* be suitably snide if they are in the same room */
  if (Base(thing)->bInside == Base(found)->bInside) {
    SendAction("^yTheir tracks are all around you!\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
    return;
  }
 
  /* Can only track one thing, no while loop */
  exit = ThingTrack(Base(thing)->bInside, found, trackRange);
  if (exit) {
    SendAction("^ySimon says: Go ", thing, NULL, SEND_SRC|SEND_VISIBLE );
    ExitGetName(exit, eName);
    SendAction(eName, thing, NULL, SEND_SRC|SEND_VISIBLE );
    SendAction("\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  } else {
    SendAction("^yYou study the ground but you cant find any tracks\n", thing, NULL, SEND_SRC|SEND_VISIBLE );
  } 
}


