/* Crimson2 Mud Server
 * All source written/copyright Ryan Haksi 1995 *
 * This source code is proprietary. Use in whole or in part without
 * explicity permission by the author is strictly prohibited
 *
 * Current email address(es): cryogen@infoserve.net
 * Phone number: (604) 591-5295
 *
 * C4 Script Language written/copyright Cam Lesiuk 1995
 * Email: clesiuk@engr.uvic.ca
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h> /* both for stat */
#include <time.h>

#include "crimson2.h"
#include "macro.h"
#include "mem.h"
#include "log.h"
#include "str.h"
#include "index.h"
#include "edit.h"
#include "queue.h"
#include "history.h"
#include "socket.h"
#include "send.h"
#include "thing.h"
#include "parse.h"
#include "world.h"
#include "base.h"
#include "object.h"
#include "char.h"
#include "group.h"
#include "mobile.h"
#include "player.h"
#include "skill.h"
#include "fight.h"
#include "affect.h"
#include "cmd_inv.h"
#include "cmd_move.h"
#include "effect.h"
#include "area.h"
#include "file.h"
#include "reset.h"

WORD EFFECT_NONE=0;          /* silly option really */
WORD EFFECT_UNCODED=0;       /* silly option really */
WORD EFFECT_CELL_REPAIR=0;   /* Body */
WORD EFFECT_REFRESH=0;       /* Body */
WORD EFFECT_ENDURANCE=0;     /* Body */
WORD EFFECT_BREATHWATER=0;   /* Body */
WORD EFFECT_STRENGTH=0;      /* Body */
WORD EFFECT_DARKVISION=0;    /* Body */
WORD EFFECT_SLOW_POISON=0;   /* Body */
WORD EFFECT_CURE_POISON=0;   /* Body */
WORD EFFECT_POISON=0;        /* Body */
WORD EFFECT_HEAL_MINOR=0;    /* Body */
WORD EFFECT_REGENERATION=0;  /* Body */
WORD EFFECT_HEAL_MAJOR=0;    /* Body */
WORD EFFECT_DEXTERITY=0;     /* Body */
WORD EFFECT_CONSTITUTION=0;  /* Body */
WORD EFFECT_HASTE=0;         /* Body */
WORD EFFECT_QUENCH=0;        /* Body */
WORD EFFECT_SUSTENANCE=0;    /* Body */
WORD EFFECT_ACIDTOUCH=0;     /* Body */
WORD EFFECT_POISONTOUCH=0;   /* Body */
WORD EFFECT_CRUSH=0;         /* Telekinetic */
WORD EFFECT_FORCESTORM=0;    /* Telekinetic */
WORD EFFECT_GHOSTFIST=0;     /* Telekinetic */
WORD EFFECT_KINETIC_SHIELD=0;/* Telekinetic */
WORD EFFECT_IMMOBILIZE=0;    /* Telekinetic */
WORD EFFECT_HEARTSTOP=0;     /* Telekinetic */
WORD EFFECT_ASPHYXIATE=0;    /* Telekinetic */
WORD EFFECT_INVISIBILITY=0;  /* Telekinetic */
WORD EFFECT_SLOW=0;          /* Telekinetic */
WORD EFFECT_IMPROVEDINVIS=0; /* Telekinetic */
WORD EFFECT_VACUUMWALK=0;    /* Telekinetic */
WORD EFFECT_PHANTOMEAR=0;    /* Telepathy */
WORD EFFECT_PHANTOMEYE=0;    /* Telepathy */
WORD EFFECT_MINDLINK=0;      /* Telepathy */
WORD EFFECT_DOMINATION=0;    /* Telepathy */
WORD EFFECT_THOUGHTBLADE=0;  /* Telepathy */
WORD EFFECT_MINDCRUSH=0;     /* Telepathy */
WORD EFFECT_DEATHDREAM=0;    /* Telepathy */
WORD EFFECT_MINDSHIELD=0;    /* Telepathy */
WORD EFFECT_SLEEP=0;         /* Telepathy */
WORD EFFECT_BERSERK=0;       /* Telepathy */
WORD EFFECT_MINDCLEAR=0;     /* Telepathy */
WORD EFFECT_IRONWILL=0;      /* Telepathy */
WORD EFFECT_TELEPORT=0;      /* Apportation */
WORD EFFECT_SUMMON=0;        /* Apportation */
WORD EFFECT_SUCCOR=0;        /* Apportation */
WORD EFFECT_BANISH=0;        /* Apportation */
WORD EFFECT_DISRUPTDOOR=0;   /* Apportation */
WORD EFFECT_PHASEDOOR=0;     /* Apportation */
WORD EFFECT_PHASEWALK=0;     /* Apportation */
WORD EFFECT_PHANTOMPOCKET=0; /* Apportation */
WORD EFFECT_WATERWALK=0;     /* Apportation */
WORD EFFECT_TELETRACK=0;     /* Apportation */
WORD EFFECT_RECALL=0;        /* Apportation */
WORD EFFECT_MARK=0;          /* Apportation */
WORD EFFECT_TRANSLOCATE=0;   /* Apportation */
WORD EFFECT_SPIRITWALK=0;    /* Spirit */
WORD EFFECT_SENSELIFE=0;     /* Spirit */
WORD EFFECT_SEEINVISIBLE=0;  /* Spirit */
WORD EFFECT_LUCKSHIELD=0;    /* Spirit */
WORD EFFECT_IDENTIFY=0;      /* Spirit */
WORD EFFECT_STAT=0;          /* Spirit */
WORD EFFECT_LUCKYHITS=0;     /* Spirit */
WORD EFFECT_LUCKYDAMAGE=0;   /* Spirit */
WORD EFFECT_PSI_IMMUNE=0;    /* Spirit */
WORD EFFECT_PSI_RESIST=0;    /* Spirit */
WORD EFFECT_FLICKER=0;       /* Spirit */
WORD EFFECT_BURNINGFIST=0;   /* Pyrokinetic */
WORD EFFECT_FLAMESTRIKE=0;   /* Pyrokinetic */
WORD EFFECT_INCINERATE=0;    /* Pyrokinetic */
WORD EFFECT_IGNITE=0;        /* Pyrokinetic */
WORD EFFECT_HEATSHIELD=0;    /* Pyrokinetic */
WORD EFFECT_FIREBLADE=0;     /* Pyrokinetic */
WORD EFFECT_FIRESHIELD=0;    /* Pyrokinetic */
WORD EFFECT_FIREARMOR=0;     /* Pyrokinetic */
WORD EFFECT_REVITALIZE=0;    /* Misc. */
WORD EFFECT_LUCKYDODGE=0;    /* spirit */
WORD EFFECT_PHANTOMDOOR=0;   /* apportation */

#define EFFECT_MAX_TARGET 16 /* pick a power of two here */


struct EffectListType effectList[] = {
/*  Name              EffectProc                            Associated Constant */
  { "NONE",           NULL,                                 &EFFECT_NONE },
  { "UNCODED",        EffectUncoded,                        &EFFECT_UNCODED },
  { "ACIDTOUCH",      EffectAcidtouch,                      &EFFECT_ACIDTOUCH },/* Body */
  { "ASPHYXIATE",     EffectAsphyxiate,                     &EFFECT_ASPHYXIATE },/* Telekinetic */
  { "BANISH",         EffectUncoded,                        &EFFECT_BANISH },/* Apportation */
  { "BERSERK",        EffectBerserk,                        &EFFECT_BERSERK },/* Telepathy */
  { "BREATHWATER",    EffectBreathwater,                    &EFFECT_BREATHWATER },/* Body */
  { "BURNINGFIST",    EffectBurningfist,                    &EFFECT_BURNINGFIST },/* Pyrokinetic */
  { "CELL-REPAIR",    EffectCellRepair,                     &EFFECT_CELL_REPAIR },/* Body */
  { "CONSTITUTION",   EffectConstitution,                   &EFFECT_CONSTITUTION },  /* Body */
  { "CRUSH",          EffectCrush,                          &EFFECT_CRUSH },/* Telekinetic */
  { "CURE-POISON",    EffectCurePoison,                     &EFFECT_CURE_POISON },/* Body */
  { "DARKVISION",     EffectDarkvision,                     &EFFECT_DARKVISION },/* Body */
  { "DEATHDREAM",     EffectDeathdream,                     &EFFECT_DEATHDREAM },/* Telepathy */
  { "DEXTERITY",      EffectDexterity,                      &EFFECT_DEXTERITY },/* Body */
  { "DISRUPTDOOR",    EffectUncoded,                        &EFFECT_DISRUPTDOOR },/* Apportation */
  { "DOMINATION",     EffectDomination,                     &EFFECT_DOMINATION },/* Telepathy */
  { "ENDURANCE",      EffectEndurance,                      &EFFECT_ENDURANCE },/* Body */
  { "FIREARMOR",      EffectFirearmor,                      &EFFECT_FIREARMOR },/* Pyrokinetic */
  { "FIREBLADE",      EffectFireblade,                      &EFFECT_FIREBLADE },/* Pyrokinetic */
  { "FIRESHIELD",     EffectFireshield,                     &EFFECT_FIRESHIELD },/* Pyrokinetic */
  { "FLAMESTRIKE",    EffectFlamestrike,                    &EFFECT_FLAMESTRIKE },/* Pyrokinetic */
  { "FLICKER",        EffectFlicker,                        &EFFECT_FLICKER },/* spirit */
  { "FORCESTORM",     EffectForcestorm,                     &EFFECT_FORCESTORM },/* Telekinetic */
  { "GHOSTFIST",      EffectGhostfist,                      &EFFECT_GHOSTFIST },/* Telekinetic */
  { "HASTE",          EffectHaste,                          &EFFECT_HASTE },  /* Body */
  { "HEAL-MAJOR",     EffectHealMajor,                      &EFFECT_HEAL_MAJOR },/* Body */
  { "HEAL-MINOR",     EffectHealMinor,                      &EFFECT_HEAL_MINOR },/* Body */
  { "HEARTSTOP",      EffectHeartStop,                      &EFFECT_HEARTSTOP },/* Telekinetic */
  { "HEATSHIELD",     EffectHeatshield,                     &EFFECT_HEATSHIELD },/* Pyrokinetic */
  { "IDENTIFY",       EffectIdentify,                       &EFFECT_IDENTIFY },/* Apportation */
  { "IGNITE",         EffectUncoded,                        &EFFECT_IGNITE },/* Pyrokinetic */
  { "IMMOBILIZE",     EffectImmobilize,                     &EFFECT_IMMOBILIZE },/* Telekinetic */
  { "IMPROVEDINVIS",  EffectImprovedInvis,                  &EFFECT_IMPROVEDINVIS },/* Telekinetic */
  { "INCINERATE",     EffectIncinerate,                     &EFFECT_INCINERATE },/* Pyrokinetic */
  { "INVISIBILITY",   EffectInvisibility,                   &EFFECT_INVISIBILITY },/* Telekinetic */
  { "IRONWILL",       EffectIronWill,                       &EFFECT_IRONWILL }, /* telepathic */
  { "KINETIC-SHIELD", EffectKineticShield,                  &EFFECT_KINETIC_SHIELD },/* Telekinetic */
  { "LUCKSHIELD",     EffectLuckshield,                     &EFFECT_LUCKSHIELD },/* Apportation */
  { "LUCKYDAMAGE",    EffectLuckydamage,                    &EFFECT_LUCKYDAMAGE },/* Apportation */
  { "LUCKYDODGE",     EffectLuckyDodge,                     &EFFECT_LUCKYDODGE }, /* spirit */
  { "LUCKYHITS",      EffectLuckyhits,                      &EFFECT_LUCKYHITS },/* Apportation */
  { "MARK",           EffectMark,                           &EFFECT_MARK },/* Apportation */
  { "MINDCLEAR",      EffectMindclear,                      &EFFECT_MINDCLEAR },/* Telepathy */
  { "MINDCRUSH",      EffectMindcrush,                      &EFFECT_MINDCRUSH },/* Telepathy */
  { "MINDLINK",       EffectSnoop,                          &EFFECT_MINDLINK },/* Telepathy */
  { "MINDSHIELD",     EffectMindshield,                     &EFFECT_MINDSHIELD },/* Telepathy */
  { "PHANTOMDOOR",    EffectPhantomDoor,                    &EFFECT_PHANTOMDOOR },/* Apportation */
  { "PHANTOMEAR",     EffectSnoop,                          &EFFECT_PHANTOMEAR },/* Telepathy */
  { "PHANTOMEYE",     EffectSnoop,                          &EFFECT_PHANTOMEYE },/* Telepathy */
  { "PHANTOMPOCKET",  EffectPhantompocket,                  &EFFECT_PHANTOMPOCKET },/* Apportation */
  { "PHASEDOOR",      EffectPhasedoor,                      &EFFECT_PHASEDOOR },/* Apportation */
  { "PHASEWALK",      EffectPhasewalk,                      &EFFECT_PHASEWALK },/* Apportation */
  { "POISON",         EffectPoison,                         &EFFECT_POISON },/* Body */
  { "POISONTOUCH",    EffectPoisontouch,                    &EFFECT_POISONTOUCH },/* Body */
  { "PSI-IMMUNE",     EffectPsiImmune,                      &EFFECT_PSI_IMMUNE }, /* spirit */
  { "PSI-RESIST",     EffectPsiResist,                      &EFFECT_PSI_RESIST }, /* spirit */
  { "QUENCH",         EffectQuench,                         &EFFECT_QUENCH },/* Body */
  { "RECALL",         EffectRecall,                         &EFFECT_RECALL },/* Apportation */
  { "REFRESH",        EffectRefresh,                        &EFFECT_REFRESH },/* Body */
  { "REGENERATION",   EffectRegeneration,                   &EFFECT_REGENERATION },/* Body */
  { "REVITALIZE",     EffectRevitalize,                     &EFFECT_REVITALIZE },/* Misc. shouldnt this go under body ?*/
  { "SEEINVISIBLE",   EffectSeeinvisible,                   &EFFECT_SEEINVISIBLE },/* Spirit */
  { "SENSELIFE",      EffectSenseLife,                      &EFFECT_SENSELIFE },/* Spirit */
  { "SLEEP",          EffectSleep,                          &EFFECT_SLEEP },/* Telepathy */
  { "SLOW",           EffectSlow,                           &EFFECT_SLOW },/* Telekinetic */
  { "SLOW-POISON",    EffectSlowPoison,                     &EFFECT_SLOW_POISON },/* Body */
  { "SPIRITWALK",     EffectSpiritwalk,                     &EFFECT_SPIRITWALK },/* Spirit */
  { "STAT",           EffectStat,                           &EFFECT_STAT },/* Apportation */
  { "STRENGTH",       EffectStrength,                       &EFFECT_STRENGTH },/* Body */
  { "SUCCOR",         EffectSuccor,                         &EFFECT_SUCCOR },/* Apportation */
  { "SUMMON",         EffectSummon,                         &EFFECT_SUMMON },/* Apportation */
  { "SUSTENANCE",     EffectSustenance,                     &EFFECT_SUSTENANCE },/* Body */
  { "TELEPORT",       EffectTeleport,                       &EFFECT_TELEPORT },/* Apportation */
  { "TELETRACK",      EffectTeletrack,                      &EFFECT_TELETRACK },/* Apportation */
  { "THOUGHTBLADE",   EffectThoughtblade,                   &EFFECT_THOUGHTBLADE },/* Telepathy */
  { "TRANSLOCATE",    EffectTranslocate,                    &EFFECT_TRANSLOCATE },/* Apportation */
  { "VACUUMWALK",     EffectVacuumwalk,                     &EFFECT_VACUUMWALK },/* Telekinetic */
  { "WATERWALK",      EffectWaterwalk,                      &EFFECT_WATERWALK },/* Apportation */
  { "", NULL, 0 }
};

INDEX tarIndex; /* passed to effect procs */


/* function is well commented see below */
LWORD EffectComputeChance( const LWORD lwMin, /* min chance to return usually 0 */
                           const LWORD lwMax, /* max chance to return usually 100 */
                           const LWORD lwFailure, /* chance of failure lwMax - this */
                           const LWORD Skill, /* skill */
                           const LWORD Resist, /* ignore if -1 */
                           THING* thing, /* one casting */
                           THING* target /* target of the spell */
                           ) {

  LWORD resist;
  LWORD chance;
  SOCK* sock;
  BYTE buf[256];

  sock = BaseControlFind(target);
  if( ParseCommandCheck(PARSE_COMMAND_WGOTO, sock , "") ) {
    return 0; /* cant affect gods with these kinds of spells */
  }
  /* compute odds
    add in target resistance
    we negate because we are adding it
    if it was a positive res then it will be subtracted from their chances
    if its a negatice resistance then it will add to the chances
  */
  if( Resist == -1 ) {
    resist = 0;
  } else {
    resist = ~(CharGetResist( target, Resist ) * 2 );
  }
  chance = ( resist + Skill );
  if( chance > ( lwMax - lwFailure ))
    chance = ( lwMax - lwFailure );
  if( chance < lwMin )
    chance = lwMin;
  else if( chance > lwMax )
    chance = lwMax;

  sock = BaseControlFind(thing);
  if (sock && BIT(Plr(sock->sHomeThing)->pSystem, PS_TESTER) ) {
    sprintf( buf,"Mods: skill  = %li\n"
      "      resist = %li\n"
      "      Chance = %li\n",
      Skill, resist, chance );
    SendAction( buf , thing, NULL, SEND_SRC );
  }
  return chance;
}

void EffectInit(void) {
  WORD i;
  for(i=0;*effectList[i].eName;i++) {
    *effectList[i].eCheck = i;
  }
  IndexInit(&tarIndex, EFFECT_MAX_TARGET*4, "tarIndex", 0);
}

/* data varies by effect, but its the value if Effect was called by apply */
/* called by CmdUse, InvEquip, CmdCast , if and only if called by InvEquip
   should cmd=NULL */
/* be carefull of what target flags you pass the entries that directly
   cause an affect without creating an AFFECT structure as you can lose
   track of things easily */
LWORD Effect(WORD effect, FLAG tarFlag, THING *thing, BYTE *cmd, LWORD data) {
  BYTE   srcKey[256];
  LWORD  srcNum;
  LWORD  srcOffset;
  THING *found;
  THING *search;
  FLAG   searchFlag;
  LWORD  t = 0;

  /*
   * do direct affects - no AFFECT structure - only for equipped objs,
   * ie, TAR_AFFECT is implicitly TAR_ON_WEAR
   */
  if (tarFlag==TAR_AFFECT && !cmd) {
    AffectThing(AFFECT_CREATE, thing, effect, data);
    return TRUE;
  }

  /* if we are equipping a tar_on_war item target ourself */
  if (!cmd || BIT(tarFlag, TAR_ON_WEAR)) BITSET(tarFlag, TAR_SELF_DEF);

  IndexClear(&tarIndex);

  /* first off, check for defaults */
  if (!cmd || !*cmd) {
    if (!cmd && !BIT(tarFlag, TAR_ON_WEAR)) return FALSE;
    if (BIT(tarFlag, TAR_NODEFAULT)) return FALSE;

    if (BIT(tarFlag, TAR_SELF_DEF))
      IndexAppend(&tarIndex, thing);

    if (BIT(tarFlag, TAR_FIGHT_DEF) && Character(thing)->cFight)
      IndexAppend(&tarIndex, Character(thing)->cFight);

    if (BIT(tarFlag, TAR_GROUP_DEF)
        /*&& BIT(Character(thing)->cAffectFlag, AF_GROUP)*/
        && Character(thing)->cLead) {
      for (found = Base(thing)->bInside->tContain; found; found=found->tNext) {
        if ((found != thing)
        /*&& Character(found)->cLead == Character(thing)->cLead
        && BIT(Character(found)->cAffectFlag, AF_GROUP))*/
        && GroupIsGroupedMember(found,thing))
          IndexAppend(&tarIndex, found);
      }
    }

    if (BIT(tarFlag, TAR_GROUPWLD_DEF)
        /*&& BIT(Character(thing)->cAffectFlag, AF_GROUP)*/
        && Character(thing)->cLead) {
      for (found = GroupGetHighestLeader(thing); found; found=Character(found)->cFollow) {
        if ((found != thing)
          && GroupIsGroupedMember(found,thing))
          IndexAppend(&tarIndex, found);
      }
    }

    if (BIT(tarFlag, TAR_NOTGROUP_DEF)) {
      if (/*!BIT(Character(thing)->cAffectFlag, AF_GROUP) ||*/ !Character(thing)->cLead) {
        BITSET(tarFlag, TAR_NOTSELF_DEF);
      } else {
        for (found = Base(thing)->bInside->tContain; found; found=found->tNext) {
          /*if (found != thing && found !=Character(thing)->cLead)*/
          if (!GroupIsGroupedMember(found,thing))
            IndexAppend(&tarIndex, found);
        }
      }
    }

    if (BIT(tarFlag, TAR_NOTSELF_DEF)) {
        for (found = Base(thing)->bInside->tContain; found; found=found->tNext) {
          if (found != thing)
            IndexAppend(&tarIndex, found);
        }
    }

  } else if (tarFlag) { /* lets see who they want to target */
    cmd = ParseFind(cmd, srcKey, &srcOffset, &srcNum, NULL, NULL);

    found = NULL;
    searchFlag = 0;
    search = thing;
    if (BIT(tarFlag, TAR_OBJ_INV))  BITSET(searchFlag, TF_OBJINV);
    if (BIT(tarFlag, TAR_PLR_ROOM)) BITSET(searchFlag, TF_PLR);
    if (BIT(tarFlag, TAR_MOB_ROOM)) BITSET(searchFlag, TF_MOB);
    if (searchFlag)
      found = ThingFind(srcKey, -1, search, searchFlag, &srcOffset);

    if (!found) {
      search = Base(thing)->bInside;
      searchFlag = 0;
      if (BIT(tarFlag, TAR_PLR_ROOM)) BITSET(searchFlag, TF_PLR);
      if (BIT(tarFlag, TAR_PLR_WLD))  BITSET(searchFlag, TF_PLR_WLD);
      if (BIT(tarFlag, TAR_MOB_ROOM)) BITSET(searchFlag, TF_MOB);
      if (BIT(tarFlag, TAR_MOB_WLD))  BITSET(searchFlag, TF_MOB_WLD);
      if (BIT(tarFlag, TAR_OBJ_ROOM)) BITSET(searchFlag, TF_OBJ);
      if (BIT(tarFlag, TAR_OBJ_WLD))  BITSET(searchFlag, TF_OBJ_WLD);
      if (BIT(tarFlag, TAR_WLD))      BITSET(searchFlag, TF_WLD );
      if (searchFlag)
        found = ThingFind(srcKey, -1, search, searchFlag, &srcOffset);
    }
    if (!found) return FALSE;

    while(found && srcNum!=0 && tarIndex.iNum<EFFECT_MAX_TARGET) {
      if( ThingCanSee( thing, found ) != TCS_CANTSEE ) {
        IndexAppend(&tarIndex, found);
        if (!BIT(tarFlag, TAR_MULTIPLE)) {
          break; /* only allow multiple sels where valid */
        }
      }
      found = ThingFind(srcKey, -1, search, searchFlag, &srcOffset);
      if (srcNum>0) {
        srcNum--;
      }
    }
  }
  if (tarIndex.iNum == 0) {
    return FALSE;
  }
  /*
     handle psi_immune here, cant do in concentrate since we dont have the target
     list there unfortunately.
  */
  for( t = 0; t < tarIndex.iNum; t++ ) {
    if( tarIndex.iThing[t] &&
        /*thing != tarIndex.iThing[t] &&*/
        tarIndex.iThing[t]->tType >= TTYPE_CHARACTER &&
        BIT(Character(tarIndex.iThing[t])->cAffectFlag, AF_PSI_IMMUNE ) ) {
      SendAction("^c$N laughs at your attempt!\n",thing, tarIndex.iThing[t], SEND_SRC|SEND_VISIBLE );
      SendAction("^cYou laugh at $n attempt!\n",thing, tarIndex.iThing[t], SEND_DST|SEND_VISIBLE );
      IndexDelete(&tarIndex,tarIndex.iThing[t], NULL );
      t--; /* decrement counter since we removed one */
    }
    if (tarIndex.iNum == 0) {
      return TRUE; /* want to lose the points still */
    }
  }
  if (effectList[effect].eProc) {
    (*effectList[effect].eProc)(EVENT_EFFECT, thing, cmd, data, &tarIndex, effect, NULL, NULL);
    return TRUE;
  } else {
    sprintf( srcKey, "Effect: eProc NULL on called effect# %i", effect );
    Log(LOG_ERROR,srcKey );
  }
  return FALSE;
}

/* Called by the unequip routine */
/* only TAR_SELF_DEF && TAR_ON_WEAR stuff should apply direct mods to abilities
   and unlimited durations */
LWORD EffectFree(WORD effect, FLAG tarFlag, THING *thing, LWORD data) {
  AFFECT *affect;

  if (tarFlag==TAR_AFFECT) {
    AffectThing(AFFECT_FREE, thing, effect, data);
    return TRUE;
  }

  else if (BIT(tarFlag,TAR_ON_WEAR)) {
    affect = AffectFind(thing, effect);
    if (affect) AffectFree(thing, affect);
    return TRUE;
  }

  return FALSE;
}

/* Effect Procedures follow, in general they should call AffectCreate to
   apply flags and modifiers. When the duration expires, the affects will
   automaticly be removed by AffectTick. (which is called by the appropriate
   Tick() procedure - MobileTick, PlayerTick etc). The Effect proc is also
   given a chance to handle things as it is passed an event of UNEFFECT.
   Note that tarIndex is null in this case!
 */

/* Uncoded as of yet */
EFFECTPROC(EffectUncoded) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    /* Let people know what happened */
    SendAction("^cSadly, the EffectProc for that is still uncoded.\n",
      thing, NULL, SEND_SRC);
  } break;

  }
  return FALSE;
}

/* EFFECT_PHANTOMDOOR
   Create a door to another virtual

   BYTE event, THING *thing, BYTE *cmd, LWORD data, INDEX *tarIndex,
   WORD effectNum, AFFECT *affect, FILE *file
*/
EFFECTPROC(EffectPhantomDoor) {
  LWORD  i;
  THING  *from = NULL;
  THING  *dest = NULL;
  THING  *to   = NULL;
  THING  *source = Base(thing)->bInside;
  LWORD  duration = data/10+20;
  BYTE   done;
  LWORD  chance;
  LWORD  dice;
  LWORD  rnum;
  WORD   area;
  LWORD  max = areaListMax-1;

  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      /* Check if they have a door allready */
      affect = AffectFind(thing, effectNum);
      if (affect) {
        AffectFree( thing, affect );
      }
      if ( Base(tarIndex->iThing[i])->bInside == Base(thing)->bInside) {
        SendAction("\n$n tries to create a dimensional doorway but is already there!", thing, NULL, SEND_ROOM|SEND_VISIBLE);
        SendAction("\nYou try to create a dimensional doorway but your already there!", thing, NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      done = FALSE;
      chance = data/2;
      if( chance < 25  ) /* give em at least a 25% chance of success */
        chance = 25;
      chance = 100 - chance;
      if ( chance < 1 )
        chance = 1; /* 0 is always failure so chance is always at least 1 to fail on 0*/
      dice = Number( 0, 100 );
      done = FALSE;
      if( dice < chance ) { /* chance of failure */
        SendAction("\nThat didnt seem right somehow.\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
        area = Number( 0, max ); /* only time this will get called */
        dest = NULL;
        while( dest == NULL ) {
          done = FALSE;
          while( !done ) {
            if( !BIT(areaList[area].aResetFlag, RF_NOENTER      ) &&
                !BIT(areaList[area].aResetFlag, RF_NOTAKE       ) &&
                !BIT(areaList[area].aResetFlag, RF_NOGAIN       ) &&
                !BIT(areaList[area].aResetFlag, RF_NOMONEY      ) &&
                !BIT(areaList[area].aResetFlag, RF_AREAISCLOSED ) ) {
              done = TRUE;
            } else {
              area++;
              if( area > max ) {
                area = 0;
              }
            }
          }/* ok got the area lets get a room */
          done = FALSE;
          rnum = Number( 0, areaList[area].aWldIndex.iNum );/* new area so reseed room */
          while( !done ) {
            dest = areaList[area].aWldIndex.iThing[rnum];
            if( dest && dest->tType == TTYPE_WLD &&
                !BIT(Wld(dest)->wFlag, WF_SMALL        ) &&
                !BIT(Wld(dest)->wFlag, WF_PRIVATE      ) &&
                !BIT(Wld(dest)->wFlag, WF_NOTELEPORTIN ) ) {
              done = TRUE;
            } else {
              dest = NULL;
              rnum++;
              if( rnum > areaList[area].aWldIndex.iNum ) {
                done = TRUE; /* go get another area since we are avoiding recalling Number*/
              }
            }
          }
        }
      } else {
        dest = tarIndex->iThing[i];
      }
      if( !dest || dest->tType != TTYPE_WLD ) {/* being paranoid again I guess */
        Log(LOG_ERROR,"EffectPhantomDoor : Null wld pointer or invalid !" );
        SendAction("^yFor some inexplicable reason you failed to create the phantomdoor\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( BIT(areaList[Wld(dest)->wArea].aResetFlag, RF_AREAISCLOSED ) ) {
        SendAction("^ySorry that Area isnt open yet!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( BIT(Wld(dest)->wFlag, WF_PRIVATE ) || BIT(Wld(dest)->wFlag, WF_NOTELEPORTIN ) ) {
        SendAction("^yThe doorway flickers into life then dies.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }

      /* Create object */
      to = ObjectCreate(phantomDoorTemplate, dest );
      Obj(to)->oLink = thing;
      OBJECTSETFIELD(to, OF_EXIT_WVIRTUAL, Obj(to)->oTemplate->oVirtual );

      from = ObjectCreate(phantomDoorTemplate, source );
      Obj(from)->oLink = thing;
      OBJECTSETFIELD(from, OF_EXIT_WVIRTUAL, Wld(dest)->wVirtual );

      affect = AffectCreate(thing, AFFECT_NONE, 0, duration, effectNum);
      affect->target = from; /* reuse target for this */
      affect->caster = to; /* reuse this one too */
      ThingSetEvent( from );
      ThingSetEvent( to );

      /* Let people know what happened */
      SendAction("^cYou create a doorway-sized rip in space-time.\n",
        thing, NULL, SEND_SRC|SEND_AUDIBLE);
      SendAction("^cSuddently a doorway-sized rip in space-time opens nearby\n",
        tarIndex->iThing[i], NULL, SEND_SRC|SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n creates a doorway-sized rip in space-time\n",
        thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
    }
    break;
  case EVENT_UNEFFECT:
    SendAction("^cYour portal hole in space-time suddenly winks out of existence.\n", thing, NULL, SEND_SRC|SEND_AUDIBLE);
    if( affect && affect->target && ThingIsEvent( affect->target ) && affect->target->tType==TTYPE_OBJ && Obj(affect->target)->oTemplate == phantomDoorTemplate) {
      SendAction("^c$N's portal hole in space-time suddenly winks out of existence.\n",
        affect->target,thing, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }
    if( affect && affect->caster && ThingIsEvent( affect->caster ) && affect->caster->tType==TTYPE_OBJ && Obj(affect->caster)->oTemplate == phantomDoorTemplate) {
      SendAction("^c$N's portal hole in space-time suddenly winks out of existence.\n",
        affect->caster, thing, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }
    /* then fall thru to free it */
  case EVENT_FREE:
    if( affect && affect->target && ThingIsEvent( affect->target ) && affect->target->tType==TTYPE_OBJ && Obj(affect->target)->oTemplate == phantomDoorTemplate) {
      /* door is history */
      THINGFREE(affect->target);
    }
    if( affect && affect->caster && ThingIsEvent( affect->caster ) && affect->caster->tType==TTYPE_OBJ && Obj(affect->caster)->oTemplate == phantomDoorTemplate) {
      /* door is history */
      THINGFREE(affect->caster);
    }
    break;
  case EVENT_READ:
  case EVENT_WRITE:
    break;
  }
  return TRUE;
}

/* flicker
   reduce damage by 25%
*/
EFFECTPROC(EffectFlicker) {
/* BYTE event, THING *thing, BYTE *cmd, LWORD data, INDEX *tarIndex,
  WORD effectNum, AFFECT *affect, FILE *file */
  LWORD i;
  LWORD duration = (data!=-1) ? data/10+20 : -1;
  switch (event) {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        AffectFree( tarIndex->iThing[i] , affect );
      }
      /* AffectCreate */
      affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      /* Let people know what happened */
      SendAction("^c$n makes you apear to flicker in and out of space\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE );
      SendAction("^c$n makes $N apear to flicker in and out of space\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE );
      SendAction("^cYou make $N apear to flicker in and out of space.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE );
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_FLICKER );
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cyou stop weaving in and out of space.\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
    SendAction("^c$n stops weaving in and out of space.\n",thing, NULL, SEND_ROOM|SEND_VISIBLE );
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_FLICKER );
    break;
  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_FLICKER );
    break;
  default:
    break;
  }
  return FALSE;
}


/* Psi-Immune */
EFFECTPROC(EffectPsiImmune) {
/* BYTE event, THING *thing, BYTE *cmd, LWORD data, INDEX *tarIndex,
  WORD effectNum, AFFECT *affect, FILE *file */
  LWORD i;
  LWORD duration = (data!=-1) ? data/10+20 : -1;
  switch (event) {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        AffectFree( tarIndex->iThing[i] , affect );
      }
      /* AffectCreate */
      affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      /* Let people know what happened */
      SendAction("^c$n makes you feel Immune from mind affecting spells.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE );
      SendAction("^c$n makes $N feel Immune from mind affecting spells.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE );
      SendAction("^cYou make $N feel Immune from mind affecting spells.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE );
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_PSI_IMMUNE );
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour mind no longer feels Impervious.\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
    SendAction("^c$n mind no longer feels Impervious.\n",thing, NULL, SEND_ROOM|SEND_VISIBLE );
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_PSI_IMMUNE );
    break;
  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_PSI_IMMUNE );
    break;
  default:
    break;
  }
  return FALSE;
}



/* Psi-Resist */
EFFECTPROC(EffectPsiResist) {
/* BYTE event, THING *thing, BYTE *cmd, LWORD data, INDEX *tarIndex,
  WORD effectNum, AFFECT *affect, FILE *file */
  LWORD i;
  LWORD duration = (data>=0) ? data/10+20 : -1;
  switch (event) {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        AffectFree( tarIndex->iThing[i] , affect );
      }
      /* AffectCreate */
      affect = AffectCreate(tarIndex->iThing[i], AFFECT_RES_PSYCHIC, (( data/5 ) + 20 ), duration, effectNum);
      /* Let people know what happened */
      SendAction("^c$n makes you feel a little bit more secure from mind affecting spells.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE );
      SendAction("^c$n makes $N feel a little bit more secure from mind affecting spells.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE );
      SendAction("^cYou make $N feel a little bit more secure from mind affecting spells.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE );
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour mind no longer feels so secure.\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
    SendAction("^c$n mind no longer feels so secure.\n",thing, NULL, SEND_ROOM|SEND_VISIBLE );
    break;
  }
  return FALSE;
}

/* Iron Will */
EFFECTPROC(EffectIronWill) {
/* BYTE event, THING *thing, BYTE *cmd, LWORD data, INDEX *tarIndex,
  WORD effectNum, AFFECT *affect, FILE *file */
  LWORD i;
  LWORD duration = (data>=0) ? data/10+20 : -1;
  switch (event) {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;
      } else { /* AffectCreate */
        affect = AffectCreate(tarIndex->iThing[i], AFFECT_IRONWILL, ( ( data/10 ) + 1 ), duration, effectNum);        
      }
      /* Let people know what happened */
      SendAction("^c$n makes your mind feel more secure.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE );
      SendAction("^c$n makes $N mind feel more secure.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE );
      SendAction("^cYou make $N mind feel more secure.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE );
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^c$n mind no longer feels so secure.\n",thing, NULL, SEND_ROOM|SEND_VISIBLE );
    SendAction("^cYour mind no longer feels so secure.\n", thing, NULL, SEND_SRC|SEND_VISIBLE);    
    break;
  }
  return FALSE;
}

/* LUCKYDODGE */
EFFECTPROC(EffectLuckyDodge) {
/* BYTE event, THING *thing, BYTE *cmd, LWORD data, INDEX *tarIndex,
  WORD effectNum, AFFECT *affect, FILE *file */
  LWORD i;
  LWORD duration = (data>=0) ? data/10+20 : -1;
  switch (event) {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;
      } else { /* AffectCreate */
        affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
        if (tarIndex->iThing[i]->tType == TTYPE_PLR) {
          /* add new amount */
          affect->aValue = ( ( data/10 ) + 1 );
          Plr(tarIndex->iThing[i])->pSkill[SKILL_DODGE] += affect->aValue;
        }
      }
      /* Let people know what happened */
      SendAction("^c$n makes you feel like you could dodge bullets almost.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE );
      SendAction("^c$n makes $N feel like they could dodge bullets almost.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE );
      SendAction("^cYou make $N feel like they could dodge bullets almost.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE );
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^c$n no longer feels like they could dodge bullets.\n",
      thing, NULL, SEND_ROOM|SEND_VISIBLE );
    SendAction("^cYou no longer feel like you could dodge bullets.\n",
        thing, NULL, SEND_SRC|SEND_VISIBLE );
    if ( thing->tType == TTYPE_PLR) {
      Plr(thing)->pSkill[SKILL_DODGE] -= affect->aValue;
    }
    break;
  }
  return FALSE;
}

/* Body */
/* EFFECT_CELL_REPAIR - more hit points */
EFFECTPROC(EffectCellRepair) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD hitMax;
    LWORD bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou cell repair $N.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n cell repairs you.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n cell repairs $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      hitMax = CharGetHitPMax(tarIndex->iThing[i]);
      bonus = Dice(data/10+1,4) + data/10 + 1;
      Character(tarIndex->iThing[i])->cHitP += bonus;
      MAXSET(Character(tarIndex->iThing[i])->cHitP, hitMax);
    }
  } break;

  }
  return FALSE;
}

/* EFFECT_REFRESH - more move points */
EFFECTPROC(EffectRefresh) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD moveMax;
    LWORD bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou refresh $N.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n refreshes you.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n refreshes $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      moveMax = CharGetMovePMax(tarIndex->iThing[i]);
      bonus = Dice(data/10+1,4) + data/10 + 1;
      Character(tarIndex->iThing[i])->cMoveP += bonus;
      MAXSET(Character(tarIndex->iThing[i])->cMoveP, moveMax);
    }
  } break;

  }
  return FALSE;
}

/* EFFECT_ENDURANCE - set endurance flag - half move pt expenditure*/
EFFECTPROC(EffectEndurance) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD duration = (data>=0) ? data/10+20 : -1;
  switch (event) {
  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou feel like you could run all day without tiring.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE );
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_ENDURANCE);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cSuddenly you feel a little winded.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_ENDURANCE);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_ENDURANCE);
    break;

  }
  return FALSE;
}

/* Typical sort of general affect spell */
EFFECTPROC(EffectBreathwater) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/20+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou start to breath a little easier.\n",
        thing, tarIndex->iThing[i], SEND_DST);
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_BREATHWATER);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou suddenly find it a little harder to breath.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_BREATHWATER);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_BREATHWATER);
    break;

  }
  return FALSE;
}

/* more strength for a while */
EFFECTPROC(EffectStrength) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(2,10) + Number(0,data/30);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration += (( data/10 ) + 20 );
      } else {  /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_STR, bonus, data/10+20, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel a little stronger.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);

    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou feel a little weaker.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
    break;
  default:
    break;
  }
  return FALSE;
}

/* see in the dark */
EFFECTPROC(EffectDarkvision) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  LWORD duration = (data>=0) ? data/10+20 : -1;
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;

      } else { /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cThe light suddenly seems a little brighter.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);

      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_DARKVISION);
    }
    break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cEverything suddenly gets a little darker.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_DARKVISION);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_DARKVISION);
    break;

  }
  return FALSE;
}

/* higher resistance towards poison for a while */
EFFECTPROC(EffectSlowPoison) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   bonus;
    AFFECT *affect;
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(2,10) + Number(0,data/30);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration += (( data/10 ) + 20 );
       } else { /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_RES_POISON, bonus, data/10+20, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel a little tougher.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou feel a little weaker.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
  } break;

  }
  return FALSE;
}

/* EFFECT_CURE_POISON    */
/* Remove a single poison effect */
/* less intox */
EFFECTPROC(EffectCurePoison) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], EFFECT_POISON);
      if (affect) {
        affect->aValue -= (data/10+1);
        if (affect->aValue < 0) AffectFree(tarIndex->iThing[i], affect);
        /* Let people know what happened */
        SendAction("^cYou help reduce the poisons effect on $n.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^c$n reduces the effect of your poison.\n",
          thing, tarIndex->iThing[i], SEND_DST);
      } else {
        SendAction("^cNothing much seems to happen.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
      }
    }
    break;
  default:
    break;
  }
  return FALSE;
}

/* Generic poison them proc */
EFFECTPROC(EffectPoison) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD   i;
  LWORD   damDice  = (data/10) + 1;
  LWORD   poison;
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      damDice = MAXV(1, damDice);
      poison = Dice(damDice, 3);
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2))
        poison >>= 1;
      MINSET(poison, 1);
      affect = AffectFind(tarIndex->iThing[i], EFFECT_POISON);
      if (affect) {
        affect->aValue += poison;
        MINSET(affect->aDuration, data/10+10);
      } else {
        affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, poison, data/10+10, EFFECT_POISON);
      }
      affect->caster = thing;
      affect->target = tarIndex->iThing[i];
      ThingSetEvent(affect->caster);
    }
  break;

  case EVENT_UNEFFECT:
  case EVENT_FREE:
    ThingDeleteEvent( affect->caster );
  case EVENT_UNAPPLY:
    break;
  }
  return FALSE;
}


/* EFFECT_HEAL_MINOR     */
/* better at healing than cell-repair, also cure poison */
EFFECTPROC(EffectHealMinor) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD hitMax;
    LWORD bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou heal $n.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n heals you.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n heals $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      hitMax = CharGetHitPMax(tarIndex->iThing[i]);
      bonus = Dice(data/10+1,6) + data/5 + 10;
      Character(tarIndex->iThing[i])->cHitP += bonus;
      MAXSET(Character(tarIndex->iThing[i])->cHitP, hitMax);
    }
  } break;
  case EVENT_UNEFFECT:
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    break;
  }
  return FALSE;
}


/* Get your points back faster */
EFFECTPROC(EffectRegeneration) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;
      } else { /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYour metabolism speeds up.\n", thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_REGENERATION);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour metabolism slows down to normal.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_REGENERATION);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_REGENERATION);
    break;

  }
  return FALSE;
}

/* EFFECT_HEAL_MAJOR     */
/* same as heal-minor but also heal move pts */
/* better at healing than cell-repair, also cure poison */
EFFECTPROC(EffectHealMajor) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD hitMax;
    LWORD moveMax;
    LWORD bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou heal $N.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n heals you.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n heals $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);

      /* heal them */
      hitMax  = CharGetHitPMax(tarIndex->iThing[i]);
      bonus = Dice(data/10+1,6) + data;
      Character(tarIndex->iThing[i])->cHitP += bonus;
      MAXSET(Character(tarIndex->iThing[i])->cHitP, hitMax);

      /* refresh them too */
      moveMax = CharGetMovePMax(tarIndex->iThing[i]);
      bonus = Dice(data/10+1,6) + data/5 + 25;
      Character(tarIndex->iThing[i])->cMoveP += bonus;
      MAXSET(Character(tarIndex->iThing[i])->cMoveP, moveMax);
    }
  } break;

  }
  return FALSE;
}

/* more dexterity for a while */
EFFECTPROC(EffectDexterity) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD   i;
  LWORD   bonus;
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(2,10) + Number(0,data/30);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration += (( data/10 ) + 20 );
      } else {     /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_DEX, bonus, data/10+20, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel a little nimbler.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou feel a little less nimble.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    break;
  }
  return FALSE;
}

/* more strength for a while */
EFFECTPROC(EffectConstitution) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   bonus;
    AFFECT *affect;
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(2,10) + Number(0,data/30);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration += (( data/10 ) + 20 );
      } else {
        /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_CON, bonus, data/10+20, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel a little tougher.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou feel a little wimpier.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
  } break;

  }
  return FALSE;
}

/* more speed for a while */
EFFECTPROC(EffectHaste) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD   i;
  LWORD   bonus;
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(5,10) + Number(0,data/2);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration += (( data/20 ) + 10 );
      } else { /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_SPEED, bonus, data/20+10, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel a little faster.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
    }
    break;
  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou feel a little slower.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
    break;
  }
  return FALSE;
}

/* EFFECT_QUENCH - less thirst */
EFFECTPROC(EffectQuench) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  LWORD thirstMin;
  LWORD bonus;
  switch (event)  {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou quench $N's thirst.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n quenches your thirst.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n quenches $N's thirst.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      if (tarIndex->iThing[i]->tType == TTYPE_PLR) {
        thirstMin = -1*raceList[Plr(tarIndex->iThing[i])->pRace].rMaxThirst;
        bonus = Dice(data/20+1,3) + 5;
        Plr(tarIndex->iThing[i])->pThirst -= bonus;
        MINSET(Plr(tarIndex->iThing[i])->pThirst, thirstMin);
      }
    }
    break;
  }
  return FALSE;
}

/* less hunger */
EFFECTPROC(EffectSustenance) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  LWORD hungerMin;
  LWORD bonus;
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou take away $N's hunger.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n takes away your hunger.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n takes away $N's hunger.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      if (tarIndex->iThing[i]->tType == TTYPE_PLR) {
        hungerMin = -1*raceList[Plr(tarIndex->iThing[i])->pRace].rMaxHunger;
        bonus = Dice(data/20+1,3) + 5;
        Plr(tarIndex->iThing[i])->pHunger -= bonus;
        MINSET(Plr(tarIndex->iThing[i])->pHunger, hungerMin);
      }
    }
    break;
  }
  return FALSE;
}

/* typical sort of attack spell - wimpy like */
EFFECTPROC(EffectAcidtouch) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 2;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_ACID);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 4) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_ACID, effectNum, "acidic touch");
    }
  } break;

  }
  return FALSE;
}

/* typical sort of attack spell - wimpy like */
EFFECTPROC(EffectPoisontouch) { /* (thing, cmd, data, tarIndex, effectNum, event) */
   LWORD   i;
   LWORD   damDice  = (data/10) + 1;
   LWORD   damBonus = 2;
   LWORD   damage;
   LWORD   poison;
   LWORD   resist;
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_POISON);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 4) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2)) {
        damage >>= 1;
      }
      poison = data/20;
      affect = AffectFind(tarIndex->iThing[i], EFFECT_POISON);
      MINSET(poison, 1);
      if (affect) {
        affect->aValue += poison;
        MINSET(affect->aDuration, data/10);
      } else {
        affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, poison, data/10, EFFECT_POISON);
      }
      affect->caster = thing;
      affect->target = tarIndex->iThing[i];
      ThingSetEvent( affect->caster );
      FightDamage(thing, damage, 1, FD_POISON, effectNum, "poisonous touch");
    }
    break;

  case EVENT_UNEFFECT:
  case EVENT_FREE:
    ThingDeleteEvent( affect->caster );
  case EVENT_UNAPPLY:
    break;
  }
  return FALSE;
}

/* typical sort of attack spell - 3rd grade */
EFFECTPROC(EffectCrush) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  LWORD damDice  = (data/10) + 1;
  LWORD damBonus = 30;
  LWORD damage;
  LWORD resist;
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_CONCUSSIVE);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 6) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_CONCUSSIVE, effectNum, "invisible vise");
    }
    break;
  }
  return FALSE;
}

/* typical sort of attack spell - one grade better */
EFFECTPROC(EffectForcestorm) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  LWORD damDice  = (data/10) + 1;
  LWORD damBonus = 20;
  LWORD damage;
  LWORD resist;
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_CONCUSSIVE);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 5) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_CONCUSSIVE, effectNum, "invisible blow");
    }
    break;
  }
  return FALSE;
}

/* typical sort of attack spell - wimpy like */
EFFECTPROC(EffectGhostfist) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  LWORD damDice  = (data/10) + 1;
  LWORD damBonus = 2;
  LWORD damage;
  LWORD resist;
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_CONCUSSIVE);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 4) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2+20))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_CONCUSSIVE, effectNum, "ghostly fist");
    }
    break;
  }
  return FALSE;
}

/* higher resistance towards kinetic energy type weapons */
EFFECTPROC(EffectKineticShield) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD   i;
  LWORD   bonus;
  LWORD duration = (data>=0) ? data/10+20 : -1;
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(2,10) + Number(0,data/30);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        for (affect=Character(tarIndex->iThing[i])->cAffect; affect; affect=affect->aNext) {
          if (affect->aEffect == effectNum ) {
            affect->aDuration = duration;
          }
        }
      } else { /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_RES_PUNCTURE,   bonus, duration, effectNum);
        AffectCreate(tarIndex->iThing[i], AFFECT_RES_SLASH,      bonus, duration, effectNum);
        AffectCreate(tarIndex->iThing[i], AFFECT_RES_CONCUSSIVE, bonus, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel somewhat sheltered.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou stop feeling sheltered.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
    break;
  }
  return FALSE;
}

/* EFFECT_IMMOBILIZE     */
/* paralyze somebody for a while, no flee or move */
EFFECTPROC(EffectImmobilize) {  /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  switch (event)
  {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      if ( BIT(Character(tarIndex->iThing[i])->cAffectFlag, AF_IMMOBILIZE ) ) {
        SendAction("^cYou attempt to Immobilize $N but they are already!\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        continue;        /* already affected */
      }
      if( Number( 0, 100  ) > EffectComputeChance( 0, 100, 25, data,FD_PSYCHIC, thing, tarIndex->iThing[i]) ) {
        SendAction("^cyou attempt to Immobilize $N but nothing seems to happen.\n",
                   thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^c$n trys to Immobilize you.\n",
                   thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        FightStart(thing, tarIndex->iThing[i], 1, NULL);
        continue;
      }

      /* Let people know what happened */
      SendAction("^c$N seems to be immobilized.\n",thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);
      SendAction("^csuddenly you are unable to move or attack.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n Immobilizes $N.\n",
                 thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* AffectCreate */
      affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, ( data / 10 ) + 20, EFFECT_IMMOBILIZE );
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_IMMOBILIZE );
      affect->caster = thing;
      affect->target = tarIndex->iThing[i];
      ThingSetEvent( affect->caster );
    }
    break;
  case EVENT_UNEFFECT:
    BITCLR(Character(thing)->cAffectFlag, AF_IMMOBILIZE );
    thing->tWait++; /* make it so they cant initiate a command for a "round" */
    /* Let people know what happened */
    SendAction("^cyou seem to be able to move again.\n",
                  thing,NULL, SEND_SRC|SEND_CAPFIRST);
    SendAction("^c$n appears to be able to move again\n",
                  thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  case EVENT_FREE:
    ThingDeleteEvent( affect->caster );
  case EVENT_UNAPPLY:
    break;

  case EVENT_APPLY:
    break;
  }
  return FALSE;
}

/* EFFECT_HEARTSTOP      */
/* Do massive damage to someone or maybe instakill them */
EFFECTPROC(EffectHeartStop) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD   i;
  LWORD   damDice;
  LWORD   damBonus;
  LWORD   damage;
  LWORD   resist;
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      damDice  = data/10;
      damBonus = data/10;
      resist = CharGetResist(tarIndex->iThing[i], FD_PSYCHIC);
      damDice = MAXV(1, damDice - resist );
      damage = Dice(damDice, 20) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2)) {
        damage = 0; /* all or nothing */
      }
      MINSET(damage, 1);
      affect = AffectFind(tarIndex->iThing[i], EFFECT_HEARTSTOP );
      if (affect) {
        affect->aValue += damage;
      } else {
        affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, damage, data/10 + 1, EFFECT_HEARTSTOP);
      }
      affect->caster = thing;
      ThingSetEvent( affect->caster );
      affect->target = tarIndex->iThing[i];

      /* Let people know what happened */
      if (damage > 1) {
        SendAction("^c$N seems to be having a heart attack.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^cYou're having a heart attack!\n",
          thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^c$n causes $N to have a heart attack!\n",
          thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      } else {
        SendAction("^cYou try to grab hold of $N's heart but they dodge to one side.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^cSome unseen force almost managed to grasp hold of your heart!\n",
          thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^c$n tries to heartstop $N but they evade $s grasp!\n",
          thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      }
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      FightDamage(thing, damage, 1, FD_PSYCHIC, effectNum, "Heart Stop");
    }
    break;
  case EVENT_UNEFFECT:
    /* Shouldnt we have a wear off message here */
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    if( ThingIsEvent( affect->caster ) ) {
      ThingDeleteEvent( affect->caster );
    }
  case EVENT_APPLY:
    break;
  }
  return FALSE;
}

/* EFFECT_ASPHYXIATE     */
/* do some damage every round for awhile */
EFFECTPROC(EffectAsphyxiate) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD   i;
  LWORD   damage;
  LWORD   damDice = data;
  LWORD   resist;
  LWORD   damBonus = data;
  switch (event) {
  case EVENT_EFFECT:

    for (i=0; i<tarIndex->iNum; i++) {
      damDice = data;
      damBonus = data;
      resist = CharGetResist(tarIndex->iThing[i], FD_PSYCHIC);
      damDice = MAXV(damDice/2+1, damDice - resist );
      damage = Dice(damDice, 6) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2)) {
        damage >>= 1;
      }
      MINSET(damage, 1);
      affect = AffectFind(tarIndex->iThing[i], EFFECT_ASPHYXIATE );
      if (affect) {
        affect->aValue += damage;
        /*MINSET(affect->aDuration, data/10);*/
      } else {
        affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, damage, data/10, EFFECT_ASPHYXIATE);
      }
      affect->caster = thing;
      affect->target = tarIndex->iThing[i];
      ThingSetEvent(affect->caster);
      /* Let people know what happened */
      SendAction("^c$N seems to be unable to breathe.\n", thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);
      SendAction("^csuddenly you cant seem to breathe!.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n ashyxiates $N.\n",
                 thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      FightDamage(thing, damage*2, 1, FD_PSYCHIC, effectNum, "Asphyxiate");
    }
    break;

  case EVENT_UNEFFECT:
  case EVENT_FREE:
    ThingDeleteEvent( affect->caster );
  case EVENT_UNAPPLY:
    break;

  case EVENT_APPLY:
    break;
  }
  return FALSE;
}

/* Invisible */
EFFECTPROC(EffectInvisibility) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum );
      if (affect) {
        affect->aDuration = duration;
      } else {
        /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou abruptly fade into invisibility.\n",
        thing, tarIndex->iThing[i], SEND_DST);
      SendAction("^c$N abruptly fades out of existence.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_INVIS);
      BITCLR(Character(tarIndex->iThing[i])->cAffectFlag, AF_IMPROVED);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou slowly fade into existence.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n slowly fades into existence.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_INVIS);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_INVIS);
    BITCLR(Character(thing)->cAffectFlag, AF_IMPROVED);
    break;

  }
  return FALSE;
}

/* less speed for a while */
EFFECTPROC(EffectSlow) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   bonus;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) AffectFree(tarIndex->iThing[i], affect);
      /* Let people know what happened */
      SendAction("^cYou feel slowed down.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      /* AffectCreate */
      bonus = Dice(5,10) + Number(0,data/2);
      AffectCreate(tarIndex->iThing[i], AFFECT_SPEED, -bonus, duration, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou feel a little faster.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
    break;
  }
  return FALSE;
}

/* Improved Invisibility */
EFFECTPROC(EffectImprovedInvis) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum );
      if (affect) {
        affect->aDuration = duration;
      } else {
        /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou abruptly fade into invisibility.\n",
        thing, tarIndex->iThing[i], SEND_DST);
      SendAction("^c$N abruptly fades out of existence.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_INVIS|AF_IMPROVED);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou slowly fade into existence.\n",
         thing, NULL, SEND_SRC|SEND_VISIBLE);
    SendAction("^c$n slowly fades into existence.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_INVIS|AF_IMPROVED);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_INVIS|AF_IMPROVED);
    break;

  }
  return FALSE;
}

/*  EFFECT_VACUUMWALK      */
/* Walk in vacuum without damage */
EFFECTPROC(EffectVacuumwalk) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/20+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum );
      if (affect) {
        affect->aDuration = duration;
      } else {
        /* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou surround yourself with a pocket of air.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_VACUUMWALK);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour pocket of air dissipates.\n",
         thing, NULL, SEND_SRC|SEND_VISIBLE);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_VACUUMWALK);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_VACUUMWALK);
    break;
  }
  return FALSE;
}

/* Telepathy */
/*  EFFECT_PHANTOMEAR  hear what they hear */
/*  EFFECT_PHANTOMEYE  see what they see */
/*  EFFECT_MINDLINK    get to see what they type, a god-level snoop link */
EFFECTPROC(EffectSnoop) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  BASELINK *baseLink = NULL;
  BASELINK *baseLinkDest = NULL;
  THING    *found = NULL;
  SOCK     *sockFound;
  SOCK     *sockThing;
  WORD     blSource;
  WORD     blDest;
  LWORD     i;

  if( effectNum == EFFECT_MINDLINK ) {
    blSource = BL_TELEPATHY_RCV;
    blDest = BL_TELEPATHY_SND;
  } else if( effectNum == EFFECT_PHANTOMEAR ) {
    blSource = BL_HEAR_RCV;
    blDest = BL_HEAR_SND;
  } else if( effectNum == EFFECT_PHANTOMEYE ) {
    blSource = BL_SEE_RCV;
    blDest = BL_SEE_SND;
  } else { /* shouldnt get this ever!! */
    Log(LOG_ERROR,"Invalid effect# in EffectSnoop" );
    return FALSE;
  }
  switch (event) {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      found = tarIndex->iThing[i];
      for (affect=Character(found)->cAffect; affect; affect=affect->aNext) {
        if (affect->aEffect == effectNum && affect->caster == thing ) {
          SendAction("^rYou try to spy on $N but you already ARE!\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
          break;
        }
      }
      if( affect != NULL ) {
        continue;
      }
      sockFound=BaseControlFind( found );
      sockThing=BaseControlFind( thing );

      if( sockFound && ParseCommandCheck(PARSE_COMMAND_WGOTO, sockFound , "") ) {
        SendAction("^wYou try to spy on $N, but they are too powerfull.You cant spy on a god\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^w$n just tried to spy on you.\n", thing, sockFound->sHomeThing, SEND_DST|SEND_CAPFIRST|SEND_VISIBLE);
        continue;
      }
      if( sockFound && sockFound == sockThing) {
        SendAction("^wYou try to spy on yourself cute\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        continue;
      }
      if( CharWillPowerCheck(tarIndex->iThing[i], -data/2)) {
        SendAction("^wYou try to spy on $N, but they repulsed you from their mind.\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^w$n just tried to spy on you.\n", thing, found, SEND_DST|SEND_CAPFIRST|SEND_VISIBLE);
        continue;
      }
      /* make a new link */
      if( effectNum == EFFECT_MINDLINK ) {
        SendAction("^rYou start spying on $N\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^r$N starts spying on you\n", found, thing , SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        BaseLinkCreate(found, thing, BL_TELEPATHY_SND);
        BITSET(Character(found)->cAffectFlag, AF_MINDLINK);
      } else if( effectNum == EFFECT_PHANTOMEAR ) {
        SendAction("^rYou start listening in on $N\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^r$N starts listening in on you\n", found, thing , SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        BaseLinkCreate( found, thing, BL_HEAR_SND );
        BITSET(Character(found)->cAffectFlag, AF_PHANTOMEAR);
      } else if( effectNum == EFFECT_PHANTOMEYE ) {
        SendAction("^rYou start watching $N\n", thing, found, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^r$N starts watching you\n", found, thing , SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        BaseLinkCreate(found, thing, BL_SEE_SND );
        BITSET(Character(found)->cAffectFlag, AF_PHANTOMEYE);
      } else { /* shouldnt get this ever!! */
        /* should log this maybe ? */
        continue;
      }
      affect = AffectCreate(thing, AFFECT_NONE, 0, data/20 + 1, effectNum );
      affect->caster = thing;
      affect->target = found;
      ThingSetEvent( affect->target );
      MINSET( affect->aDuration, 1 );
    }
    break;
    /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    baseLink = BaseLinkFind(Base(thing)->bLink, blSource, NULL );
    baseLinkDest = BaseLinkFind(Base(thing)->bLink, blDest, NULL );
    if( baseLink ) {
      BaseLinkFree(thing, baseLink);
    } else if( baseLinkDest ) {
      BaseLinkFree(thing, baseLinkDest);
    }
    if( effectNum == EFFECT_MINDLINK ) {
      BITCLR(Character(affect->target)->cAffectFlag, AF_MINDLINK);
      if( affect->target && ThingIsEvent( affect->caster ) ) {
        SendAction("^rYou stop spying on $N\n", thing, affect->target , SEND_SRC|SEND_VISIBLE);
        SendAction("^r$N stops spying on you.\n",affect->target, thing, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
      }
    } else if( effectNum == EFFECT_PHANTOMEAR ) {
      BITCLR(Character(affect->target)->cAffectFlag, AF_PHANTOMEAR);
      if( affect->target && ThingIsEvent( affect->target ) ) {
        SendAction("^rYou stop listening in on $N\n", thing, affect->target, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^rSomeone stops listening in on you.\n",affect->target, NULL, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
      }
    } else if( effectNum == EFFECT_PHANTOMEYE ) {
      BITCLR(Character(affect->target)->cAffectFlag, AF_PHANTOMEYE);
      if( affect->target && ThingIsEvent( affect->caster ) ) {
        SendAction("^rYou stop watching $N\n", thing, affect->target, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
        SendAction("^rSomeone stops watching you.\n", affect->target, NULL, SEND_SRC|SEND_CAPFIRST|SEND_VISIBLE);
      }
    }
  case EVENT_FREE:
    ThingDeleteEvent( affect->target );
  case EVENT_UNAPPLY:
    break;

  case EVENT_APPLY:
    break;
  case EVENT_WRITE:
  case EVENT_READ: /* no save to disk for this one */
    return TRUE;
  }
  return FALSE;
}

/*  EFFECT_DOMINATION     */
/* essentially charm person */
/* Typical sort of general affect spell */
EFFECTPROC(EffectDomination) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    for (i=0; i<tarIndex->iNum; i++) {
      if (BIT(Character(tarIndex->iThing[i])->cAffectFlag, AF_DOMINATED)) {
        SendAction("^cNothing new seems to happen.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( CharWillPowerCheck(tarIndex->iThing[i], -data/10)) {
        /* well well, we have attacked them */
        FightStart(thing, tarIndex->iThing[i], 1, NULL);
        continue;
      }
      /* Make 'em follow */
      CharAddFollow(tarIndex->iThing[i], thing);

      /* Let people know what happened */
      SendAction("^cYou are bent to the will of $n.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);

      /* AffectCreate */
      affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, data/10+5, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_DOMINATED);
      affect->caster = thing;
      affect->target = tarIndex->iThing[i];
      ThingSetEvent( affect->caster );
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: /* Let people know what happened */
    SendAction("^cYour mind is now your own.\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
    /* Mob should attack leader now */
    if( affect->caster && ThingIsEvent( affect->caster ) ) {
      CharRemoveFollow(thing);
      if( thing->tType == TTYPE_MOB ) {
        FightStart( thing, affect->caster, 1, NULL);
      }
    }
  case EVENT_FREE:
    ThingDeleteEvent( affect->target );
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_DOMINATED);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_DOMINATED);
    break;

  }
  return FALSE;
}


/* Attack their mind - wimpy */
EFFECTPROC(EffectThoughtblade) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 2;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_PSYCHIC);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 4) + damBonus;
      if (CharWillPowerCheck(tarIndex->iThing[i], -data/2+20))
        damage >>= 1;

      FightDamage(thing, damage, 1, FD_PSYCHIC, effectNum, "blade of thought");
    }
  } break;

  }
  return FALSE;
}


/* Attack their mind - a bit better */
EFFECTPROC(EffectMindcrush) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 20;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_PSYCHIC);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 5) + damBonus;
      if (CharWillPowerCheck(tarIndex->iThing[i], -data/2))
        damage >>= 1;

      FightDamage(thing, damage, 1, FD_PSYCHIC, effectNum, "brutal mindcrush");
    }
  } break;

  }
  return FALSE;
}


/* Attack their mind - 3rd grade */
EFFECTPROC(EffectDeathdream) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 40;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_PSYCHIC);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 6) + damBonus;
      if (CharWillPowerCheck(tarIndex->iThing[i], -data))
        damage >>= 1;

      FightDamage(thing, damage, 1, FD_PSYCHIC, effectNum, "dreams of death");
    }
  } break;

  }
  return FALSE;
}


/* higher resistance towards psychic skills */
EFFECTPROC(EffectMindshield) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   bonus;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      bonus = Dice(2,10) + Number(0,data/30);
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        affect->aDuration = duration;
      } else {/* AffectCreate */
        AffectCreate(tarIndex->iThing[i], AFFECT_RES_PSYCHIC,   bonus, duration, effectNum);
      }
      /* Let people know what happened */
      SendAction("^cYou feel more determined.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou feel less determined.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
  } break;

  }
  return FALSE;
}

/*  EFFECT_SLEEP          */
EFFECTPROC(EffectSleep) {  /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  switch (event)
  {
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      if ( BIT(Character(tarIndex->iThing[i])->cAffectFlag, AF_SLEEP ) ) {
        SendAction("^cYou attempt to sleep $N but they are already asleep!\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        continue;        /* already affected by sleep */
      }
      if( Number( 0, 100  ) > EffectComputeChance( 0, 100, 0, data,FD_PSYCHIC, thing, tarIndex->iThing[i]) ) {
        SendAction("^cyou attempt to sleep $N but nothing seems to happen.\n",
                   thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);
        SendAction("^c$n trys to sleep you.\n",
                   thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
        if( Number( 0, 100 ) < 21 ) /* 20% chance they will attack */
          FightStart(thing, tarIndex->iThing[i], 1, NULL);
        continue;
      }
      if ( Character(tarIndex->iThing[i])->cFight) { /* if fighting */
        FightStop( tarIndex->iThing[i] );
      }
      /* Let people know what happened */
      SendAction("^c$N starts snoring loudly.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^csuddenly you are feeling very sleepy zzzzzz.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n puts $N to sleep.\n",
                 thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* AffectCreate */
      affect = AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, data/10+10, EFFECT_SLEEP );
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_SLEEP);
      affect->caster = thing;
      ThingSetEvent( affect->caster );
      affect->target = tarIndex->iThing[i];
    }
    break;

  case EVENT_UNEFFECT:
    thing->tWait++; /* make it so they cant initiate a command for a "round" */
    BITCLR(Character(thing)->cAffectFlag, AF_SLEEP);
    if( Character(thing)->cFight ) { /* fighting */
      Character(thing)->cPos = POS_FIGHTING;
      SendAction("^cyou abrupty startle awake as someone hits you ouch that hurts.\n",
                   thing,NULL, SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n is rudely awakened\n",
                   thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    } else {
      SendAction("^cyou awaken feeling refreshed from your nap.\n",
        thing,NULL, SEND_SRC|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n wakes up and stands up\n",
        thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    }
  case EVENT_FREE:
    ThingDeleteEvent( affect->target );
  case EVENT_UNAPPLY:
    break;
  }
  return FALSE;
}

/* less intox */
EFFECTPROC(EffectMindclear) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD intoxMin;
    LWORD bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou help clear $n's mind.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n helps clear your mind.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n helps clear $N's mind.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      if (tarIndex->iThing[i]->tType == TTYPE_PLR) {
        intoxMin = -1*raceList[Plr(tarIndex->iThing[i])->pRace].rMaxIntox;
        bonus = Dice(data/20+1,3) + 5;
        Plr(tarIndex->iThing[i])->pIntox -= bonus;
        MINSET(Plr(tarIndex->iThing[i])->pIntox, intoxMin);
      }
    }
  } break;

  }
  return FALSE;
}

/* Makes you do more damage, hard to hit people */
EFFECTPROC(EffectBerserk) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD i;
    AFFECT *affect;
    LWORD duration = (data>=0) ? data/30+10 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) AffectFree(tarIndex->iThing[i], affect);
      /* Let people know what happened */
      SendAction("^cYou make $N berserk.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n makes you berserk.\n",
        thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n makes $N berserk.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      AffectCreate(tarIndex->iThing[i], AFFECT_DAMROLL, data/25+1, duration, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou stop seeing the world through a red haze.\n",
         thing, NULL, SEND_SRC|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
    SendAction("^c$n stops being berserk.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  } break;
  case EVENT_UNAPPLY:
    break;
  case EVENT_APPLY:
    break;
  }
  return FALSE;
}

/* Apportation */
/*  EFFECT_TELEPORT       */
/* go somewhere - keep in same zone - good escape spell */
/* Transport away */
EFFECTPROC(EffectTeleport) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  LWORD i;
  THING* dest;
  LWORD  dice;
  BYTE   done = FALSE;
  LWORD  max = areaListMax-1;
  LWORD  rnum;
  LWORD  area;
  LWORD  chance;

  switch (event)
  {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    for (i=0; i<tarIndex->iNum; i++) {
      if ( Base(tarIndex->iThing[i])->bInside == Base(thing)->bInside) {
        SendAction("\n$n tries to teleport but is already there!", thing, NULL, SEND_ROOM|SEND_VISIBLE);
        SendAction("\nYou try to teleport but your already there!", thing, NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      done = FALSE;
      chance = data/2;
      if( chance < 25  ) /* give em at least a 25% chance of success */
        chance = 25;
      chance = 100 - chance;
      if ( chance < 1 )
        chance = 1; /* 0 is always failure so chance is always at least 1 to fail on 0*/
      dice = Number( 0, 100 );

#define TESTINGTELEPORTMETHOD
#ifdef TESTINGTELEPORTMETHOD
      done = FALSE;
      dest = NULL;
      if( dice < chance ) { /* chance of failure */
        SendAction("\nOops you took a wrong turn somewhere!\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
        area = Number( 0, max ); /* only time this will get called */
        dest = NULL;
        while( dest == NULL ) {
          done = FALSE;
          while( !done ) {
            if( !BIT(areaList[area].aResetFlag, RF_NOENTER      ) &&
                !BIT(areaList[area].aResetFlag, RF_NOTAKE       ) &&
                !BIT(areaList[area].aResetFlag, RF_NOGAIN       ) &&
                !BIT(areaList[area].aResetFlag, RF_NOMONEY      ) &&
                !BIT(areaList[area].aResetFlag, RF_AREAISCLOSED ) ) {
              done = TRUE;
            } else {
              area++;
              if( area > max ) {
                area = 0;
              }
            }
          }/* ok got the area lets get a room */
          done = FALSE;
          rnum = Number( 0, areaList[area].aWldIndex.iNum );/* new area so reseed room */
          while( !done ) {
            dest = areaList[area].aWldIndex.iThing[rnum];
            if( dest && dest->tType == TTYPE_WLD &&
                !BIT(Wld(dest)->wFlag, WF_SMALL        ) &&
                !BIT(Wld(dest)->wFlag, WF_PRIVATE      ) &&
                !BIT(Wld(dest)->wFlag, WF_NOTELEPORTIN ) ) {
              done = TRUE;
            } else {
              dest = NULL;
              rnum++;
              if( rnum > areaList[area].aWldIndex.iNum ) {
                done = TRUE; /* go get another area since we are avoiding recalling Number*/
              }
            }
          }
        }
      } else {
        dest = Base(tarIndex->iThing[i])->bInside;
      }
#else
      dice = Number( 0, 100 );
      done = FALSE;
      if( dice < chance ) { /* chance of failure */
        SendAction("\nOops you took a wrong turn somewhere!\n", thing, NULL, SEND_SRC|SEND_VISIBLE);
        while( !done ) {
          area = Number( 0, max );
          if( !BIT(areaList[area].aResetFlag, RF_NOENTER      ) &&
              !BIT(areaList[area].aResetFlag, RF_NOTAKE       ) &&
              !BIT(areaList[area].aResetFlag, RF_NOGAIN       ) &&
              !BIT(areaList[area].aResetFlag, RF_NOMONEY      ) &&
              !BIT(areaList[area].aResetFlag, RF_AREAISCLOSED ) ) {
            done = TRUE;
          }
        }/* ok got the area lets get a room */
        done = FALSE;
        while( !done ) {
          rnum = Number( 0, areaList[area].aWldIndex.iNum );
          dest = areaList[area].aWldIndex.iThing[rnum];
          if( dest && dest->tType == TTYPE_WLD &&
              !BIT(Wld(dest)->wFlag, WF_SMALL        ) &&
              !BIT(Wld(dest)->wFlag, WF_PRIVATE      ) &&
              !BIT(Wld(dest)->wFlag, WF_NOTELEPORTIN ) ) {
            done = TRUE;
          }
        }
      } else {
        dest = Base(tarIndex->iThing[i])->bInside;
      }
#endif
      if( !dest || dest->tType != TTYPE_WLD ) {/* being paranoid again I guess */
        Log(LOG_ERROR,"EffectTeleport : Null wld pointer !" );
        continue;
      }
      if( BIT(areaList[Wld(dest)->wArea].aResetFlag, RF_AREAISCLOSED ) ) {
        SendAction("^yYou try to teleport in but that Area isnt open yet!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( BIT(Wld(dest)->wFlag, WF_SMALL ) ) { /* check contents make sure theres room */
        /* should we do a count before failing this or just fail auto ? */
        SendAction("^yYou try to teleport in but there wasnt room!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( BIT(Wld(dest)->wFlag, WF_PRIVATE ) ) { /* private cant go there */
        SendAction("^yYou try to teleport in but its a private room!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( BIT(Wld(dest)->wFlag, WF_NOTELEPORTIN ) ) { /* no teleport allowed */
        SendAction("^yYou try to teleport in but are repulsed!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }
      if( BIT(Wld(dest)->wFlag, WF_DEATHTRAP ) ) { /* OOPS! */
        /* will only happen on failure or tele to a god at dt */
        PlayerKillDeathTrap( thing , dest, Base(thing)->bInside );
        continue;
      }
      if (thing->tType == TTYPE_PLR) {
        SendAction(Plr(thing)->pExit->sText, thing, NULL, SEND_ROOM|SEND_VISIBLE);
      } else {
        SendAction("$n dissolves into a cloud of blue sparkles.\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
      }
      ThingTo( thing, dest );
      FightStop(thing);
      if (thing->tType == TTYPE_PLR) {
        SendAction(Plr(thing)->pEnter->sText, thing, NULL, SEND_ROOM|SEND_VISIBLE);
      } else  {
        SendAction("$n appears from a cloud of blue sparkles.\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
      }
      CmdLook(thing, "");
    }
    return FALSE;
  default:
    break;
  }
  return TRUE;
}

/*  EFFECT_SUMMON         */
/* bring someone to us */
EFFECTPROC(EffectSummon) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  BYTE bFail = FALSE;
  LWORD   i;
  BYTE bCheck = FALSE;
  SOCK* sock;

  switch (event)
  {
    /* Works on tarIndex */
  case EVENT_EFFECT:
    {
      for (i=0; i<tarIndex->iNum; i++) {
        bFail = FALSE;
        bCheck = TRUE;
        sock = BaseControlFind( tarIndex->iThing[i] );

        /*
          check why we cant do this at all
          first no gods
        */
        if( thing != tarIndex->iThing[i] && ParseCommandCheck(PARSE_COMMAND_WGOTO, sock , "") ) { /* cant summon gods */
          SendAction("^c$N laughs at your pitiful attempt to summon $e, you cant summon a god!\n",
            thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^c$n just tried to summon you! You laughed at $e pathetic attempt.\n",
            thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          SendAction("^c$n tries to summon a GOD!.\n",
            thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          continue;
        }
        /*
           check area/wld flags for from area
        */
        if( BIT(areaList[Wld(Base(tarIndex->iThing[i])->bInside)->wArea].aResetFlag, RF_AREAISCLOSED   ) ||
            BIT(areaList[Wld(Base(tarIndex->iThing[i])->bInside)->wArea].aResetFlag, RF_NOENTER        ) ||
            BIT(areaList[Wld(Base(tarIndex->iThing[i])->bInside)->wArea].aResetFlag, RF_NOTAKE         ) ||
            BIT(areaList[Wld(Base(tarIndex->iThing[i])->bInside)->wArea].aResetFlag, RF_NOGAIN         ) ||
            BIT(areaList[Wld(Base(tarIndex->iThing[i])->bInside)->wArea].aResetFlag, RF_NOMONEY        ) ||
            BIT(areaList[Wld(Base(tarIndex->iThing[i])->bInside)->wArea].aResetFlag, RF_NOTELEPORT_OUT ) ||
            BIT(Wld(Base(tarIndex->iThing[i])->bInside)->wFlag, WF_NOTELEPORTOUT ) ) {
          SendAction("^cA loudspeaker hisses into life \"Warning this operation is NOT ALLOWED! Forcefield Activated!\"\nI guess that wont work huh ?\n",
                     thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^cA loudspeaker hisses into life \"Warning this operation is NOT ALLOWED! Forcefield Activated!\"\nThat didnt seem to work\n",
                     thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          continue;
        }
        /*
           check area/wld flags for dest area

           dont care about other ones here, their own stupidity to summon mobs to
           closed areas
        */
        if( BIT(Wld(Base(thing)->bInside)->wFlag, WF_NOTELEPORTIN ) ) {
          SendAction("^cA loudspeaker hisses into life \"Warning this operation is NOT ALLOWED! Forcefield Activated!\"\nI guess that wont work huh ?\n",
                     thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^cA loudspeaker hisses into life \"Warning this operation is NOT ALLOWED! Forcefield Activated!\"\nThat didnt seem to work\n",
                     thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          continue;
        }
        /*
          separate area check, this one is for noaggro mobs
        */
        if( BIT(areaList[Wld(Base(thing)->bInside)->wArea].aResetFlag, RF_NOAGGRO) &&
            tarIndex->iThing[i]->tType == TTYPE_MOB &&
            ( BIT(Mob(tarIndex->iThing[i])->mTemplate->mAct, MACT_AGGRESSIVE ) ||
            BIT(Mob(tarIndex->iThing[i])->mTemplate->mAct,MACT_HYPERAGGR))) {
          SendAction("^cA loudspeaker hisses into life \"Warning this operation is NOT ALLOWED! Forcefield Activated!\"\nI guess that wont work huh ?\n",
                     thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^cA loudspeaker hisses into life \"Warning this operation is NOT ALLOWED! Forcefield Activated!\"\nThat didnt seem to work\n",
                     thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          continue;
        }
        /*
           check for nosummon bit on mobs
        */
        if( tarIndex->iThing[i]->tType == TTYPE_MOB &&
            BIT(Mob(tarIndex->iThing[i])->mTemplate->mAct,MACT_NOSUMMON) ) {
          SendAction("^c$N laughs at your pitiful attempt to summon $e\n",
                     thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^c$n tries to summon a $N but fails miserably.\n",
                     thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          continue;
        }
        if( tarIndex->iThing[i]->tType == TTYPE_PLR && !BIT(Plr(tarIndex->iThing[i])->pSystem,PS_NOSUMMON) ) {
          bCheck = FALSE;
        }
        if( thing == tarIndex->iThing[i] ) {
          bCheck = FALSE;
        }
        if( bCheck == TRUE && Number( 0, 100  ) > EffectComputeChance( 0, 100, 10, data, FD_PSYCHIC, thing, tarIndex->iThing[i]) ) {
          bFail = TRUE;
        } else {
          bFail = FALSE;
        }
        if( bFail ) {
          SendAction("^cYou try to summon $N but fail miserably.\n",
            thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^c$n just tried to summon you!\n",
            thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          SendAction("^c$n tries to summon $N.\n",
            thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          continue;
        }
        if (Base(tarIndex->iThing[i])->bInside != Base(thing)->bInside) {
          if (tarIndex->iThing[i]->tType == TTYPE_PLR)
            SendAction(Plr(tarIndex->iThing[i])->pExit->sText, tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
          else
            SendAction("$n dissolves into a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
          SendAction("^cYou summon $N.\n", thing, tarIndex->iThing[i], SEND_SRC );
          SendAction("^c$n just summoned you!\n", thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
          FightStop(tarIndex->iThing[i]);
          ThingTo(tarIndex->iThing[i], Base(thing)->bInside);
          if (tarIndex->iThing[i]->tType == TTYPE_PLR)
            SendAction(Plr(tarIndex->iThing[i])->pEnter->sText, tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
          else
            SendAction("$n appears from a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
          CmdLook(thing, "");
          CmdLook(tarIndex->iThing[i], "");
        }
      }
    } break;
  }
  return FALSE;
}

/*  EFFECT_SUCCOR         */
/* everyone go visit the group leader */
EFFECTPROC(EffectSuccor) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    if (!Character(thing)->cLead) {
      SendAction("^wBut you have no leader to succor...\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      break;
    }
    for (i=0; i<tarIndex->iNum; i++) {
      /* Transport away */
      if (Base(tarIndex->iThing[i])->bInside != Base(Character(thing)->cLead)->bInside) {
        if (tarIndex->iThing[i]->tType == TTYPE_PLR)
          SendAction(Plr(tarIndex->iThing[i])->pExit->sText, tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n dissolves into a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        ThingTo(tarIndex->iThing[i], Base(Character(thing)->cLead)->bInside);
        FightStop(tarIndex->iThing[i]);
        if (tarIndex->iThing[i]->tType == TTYPE_PLR)
          SendAction(Plr(tarIndex->iThing[i])->pEnter->sText, tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n appears from a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
      }
    }
  } break;

  }
  return FALSE;
}


/*  EFFECT_BANISH         */
/* banish some clod somewhere random (nearby) */

/*  EFFECT_DISRUPTDOOR    */
/* get rid of a phantom door prematurely */

/*  EFFECT_PHANTOMDOOR      */
/* create a timed portal */

/*  EFFECT_PHASEDOOR    */
/* walk through doors */
/* Typical sort of general affect spell */
EFFECTPROC(EffectPhasedoor) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou start to move a little out of phase with the rest of the world.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_PHASEDOOR);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou are suddenly wrenched fully into the normal universe.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_PHASEDOOR);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_PHASEDOOR);
    break;

  }
  return FALSE;
}


/*  EFFECT_PHASEWALK      */
/* Walk out of phase, ignore terrain costs */
/* Typical sort of general affect spell */
EFFECTPROC(EffectPhasewalk) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou start to move a little out of phase with the rest of the world.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_PHASEWALK);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou are suddenly wrenched fully into the normal universe.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_PHASEWALK);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_PHASEWALK);
    break;

  }
  return FALSE;
}

/* EFFECT_PHANTOMPOCKET */
/* Create a pocket to put stuff in */
EFFECTPROC(EffectPhantompocket) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD  i;
    THING  *item;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    LWORD  containerMax = data+20;

    for (i=0; i<tarIndex->iNum; i++) {
      /* Check if they have a pocket allready */
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration += duration;
        continue;
      }

      /* Create object */
      item = ObjectCreate(phantomPocketTemplate, tarIndex->iThing[i]);
      OBJECTSETFIELD(item, OF_CONTAINER_MAX, containerMax);

      /* If we can equip it */
      if (InvEquip(tarIndex->iThing[i], item, NULL)) {
        /* Let people know what happened */
        SendAction("^cYou create a pocket-sized rip in space-time for $N\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^c$n creates a pocket-sized rip in space-time for you\n",
          thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        SendAction("^c$n creates a pocket-sized rip in space-time for $N\n",
          thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, data, duration, effectNum);
      } else {
        /* If we cant equip it turf the object */
        /* Let caster know their hand was full */
        SendAction("^cYou try to create a phantom pocket for $N but have no place to put it.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        THINGFREE(item);
      }

    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour pocket-sized hole in space-time suddenly winks out of existence.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n's pocket-sized hole in space-time suddenly winks out of existence.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    /* then fall thru to free it */
  case EVENT_FREE: {
    THING *i;
    THING *next;

    /* search for pocket - turf em as we find em, should only be one */
    for (i = thing->tContain; i; i=next) {
      next = i->tNext;
      if (i->tType==TTYPE_OBJ && Obj(i)->oTemplate == phantomPocketTemplate) {
        while (i->tContain) {
          /* move contents to the room */
          if (Base(thing)->bInside) {
            SendAction("^c$n winks into existence and falls to the floor\n",
              i->tContain, NULL, SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
            ThingTo(i->tContain, Base(thing)->bInside);
          } else
            ThingFree(i->tContain);
        }
        /* Pocket is history */
        THINGFREE(i);
      }
    }
  } break;


  case EVENT_READ: {
    THING *item;
    LWORD  containerMax = affect->aValue;

    AffectReadPrimitive(thing, affect, file);
    item = ObjectCreate(phantomPocketTemplate, thing);
    OBJECTSETFIELD(item, OF_CONTAINER_MAX, containerMax);
    InvEquip(thing, item, NULL);
  } break;

  case EVENT_WRITE: {
    AffectWritePrimitive(thing, affect, file);
  } break;

  }
  return TRUE;
}


/*  EFFECT_WATERWALK      */
/* Walk on water */
EFFECTPROC(EffectWaterwalk) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        if( data < 1 )
          affect->aDuration = -1;
        else
          affect->aDuration += duration;
        continue;
      }
      if (BIT(Character(tarIndex->iThing[i])->cAffectFlag, AF_WATERWALK)) {
        SendAction("^cNothing new seems to happen.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
        return FALSE;
      }
      /* Let people know what happened */
      SendAction("^cYou start to move a little out of phase with the rest of the world.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_WATERWALK);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou are suddenly wrenched fully into the normal universe.\n",
         thing, NULL, SEND_SRC);
  case EVENT_FREE:
  case EVENT_UNAPPLY:
    BITCLR(Character(thing)->cAffectFlag, AF_WATERWALK);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_WATERWALK);
    break;

  }
  return FALSE;
}

/* everyone go to the target */
EFFECTPROC(EffectTeletrack) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   trackRange;

    for (i=0; i<tarIndex->iNum; i++) {
      /* Check if they are in range */
      trackRange = data;

      /* be suitably snide if they are in the same room */
      if (Base(thing)->bInside == Base(tarIndex->iThing[i])->bInside) {
        SendAction("^yTheir tracks are all around you!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        return FALSE;
      }

      /* Can only track one thing, no while loop */
      if (!ThingTrack(Base(thing)->bInside, tarIndex->iThing[i], trackRange)) {
        SendAction("^yThey're out of range I'm afraid\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
        return FALSE;
      }

      /* Transport away */
      if (Base(tarIndex->iThing[i])->bInside != Base(thing)->bInside) {
        if (thing->tType == TTYPE_PLR)
          SendAction(Plr(thing)->pExit->sText, thing, NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n dissolves into a cloud of blue sparkles.\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
        ThingTo(thing, Base(tarIndex->iThing[i])->bInside);
        FightStop(thing);
        if (thing->tType == TTYPE_PLR)
          SendAction(Plr(thing)->pEnter->sText, thing, NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n appears from a cloud of blue sparkles.\n", thing, NULL, SEND_ROOM|SEND_VISIBLE);
        CmdLook(thing, "");
      }
      /* use effect flag to build target for us, should do it here ourself instead so that
         I can add MassTeletrack more conveniently but this is easy */
      return FALSE;
    }
  } break;

  }
  return FALSE;
}


/* everyone go to the target */
EFFECTPROC(EffectRecall) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    THING  *to = NULL;

    /* Check where they have marked to recall */
    if (thing->tType == TTYPE_PLR) {
      to = WorldOf(Plr(thing)->pRecallRoom);
    }
    if (!to) to = WorldOf(playerStartRoom);

    /* be suitably snide if they are in the same room */
    if (!to) {
      SendAction("^yYou have no place marked in memory to recall to!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      return FALSE;
    }

    for (i=0; i<tarIndex->iNum; i++) {
      /* Transport away */
      if (Base(tarIndex->iThing[i])->bInside != to) {
        if (tarIndex->iThing[i]->tType == TTYPE_PLR)
          SendAction(Plr(tarIndex->iThing[i])->pExit->sText, tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n dissolves into a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        ThingTo(tarIndex->iThing[i], to);
        FightStop(tarIndex->iThing[i]);
        if (tarIndex->iThing[i]->tType == TTYPE_PLR)
          SendAction(Plr(tarIndex->iThing[i])->pEnter->sText, thing, NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n appears from a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        CmdLook(tarIndex->iThing[i], "");
      } else {
        SendAction("^yYou try to recall but you're allready there!\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      }
    }
  } break;

  }
  return FALSE;
}


/* everyone go to the target */
EFFECTPROC(EffectMark) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;

    for (i=0; i<tarIndex->iNum; i++) {
      /* be suitably snide if they are in the same room */
      if (Base(tarIndex->iThing[i])->bInside->tType != TTYPE_WLD) {
        SendAction("^yTheres nothing about this place you can mark in your memory!\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
        continue;
      }

      /* Mark away */
      if (tarIndex->iThing[i]->tType == TTYPE_PLR) {
        Plr(tarIndex->iThing[i])->pRecallRoom = Wld(Base(tarIndex->iThing[i])->bInside)->wVirtual;
        SendAction("^yYou mentally mark your surroundings so you can recall them later\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      } else {
        SendAction("^yOnly Players can Mark!\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      }
    }
  } break;

  }
  return FALSE;
}

/* swap current position and marked position */
EFFECTPROC(EffectTranslocate) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    THING  *to = NULL;
    LWORD   recallVirtual;

    /* be suitably snide if they are in the same room */
    if (Base(thing)->bInside->tType != TTYPE_WLD) {
      SendAction("^yTheres nothing about this place you can mark in your memory!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      return FALSE;
    }
    recallVirtual = Wld(Base(thing)->bInside)->wVirtual;

    /* Check where they have marked to recall */
    if (thing->tType == TTYPE_PLR) {
      to = WorldOf(Plr(thing)->pRecallRoom);
    }
    if (!to) to = WorldOf(playerStartRoom);
    Plr(thing)->pRecallRoom = recallVirtual;

    /* be suitably snide if they are in the same room */
    if (!to) {
      SendAction("^yYou have no place marked in memory to recall to!\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      return FALSE;
    }

    for (i=0; i<tarIndex->iNum; i++) {
      /* Transport away */
      if (Base(tarIndex->iThing[i])->bInside != to) {
        if (tarIndex->iThing[i]->tType == TTYPE_PLR)
          SendAction(Plr(tarIndex->iThing[i])->pExit->sText, tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n dissolves into a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        ThingTo(tarIndex->iThing[i], to);
        FightStop(tarIndex->iThing[i]);
        if (tarIndex->iThing[i]->tType == TTYPE_PLR)
          SendAction(Plr(tarIndex->iThing[i])->pEnter->sText, thing, NULL, SEND_ROOM|SEND_VISIBLE);
        else
          SendAction("$n appears from a cloud of blue sparkles.\n", tarIndex->iThing[i], NULL, SEND_ROOM|SEND_VISIBLE);
        CmdLook(tarIndex->iThing[i], "");
      } else {
        SendAction("^yYou try to translocate but you're allready there!\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      }
    }
  } break;

  }
  return FALSE;
}




  /* Spirit */
/*  EFFECT_SPIRITWALK     */
/* walk around without a body */
EFFECTPROC(EffectSpiritwalk) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    THING *spirit;
    SOCK  *sock;
    for (i=0; i<tarIndex->iNum; i++) {
      sock = BaseControlFind(tarIndex->iThing[i]);
      if (!sock) return FALSE;
      /* Let people know what happened */
      spirit = MobileCreate(spiritTemplate, Base(tarIndex->iThing[i])->bInside);
      BaseControlFree(tarIndex->iThing[i], sock);
      BaseControlAlloc(spirit, sock);
      SendAction("^cYou feel your spirit leave your body.\n", spirit,NULL, SEND_SRC|SEND_VISIBLE);
      SendHint("^;HINT: type ^<STOP SPIRITWALK^; to return to your body\n", spirit);
      AffectCreate(spirit, AFFECT_NONE, 0, data/10+10, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYou are abrubtly yanked back to your real body.\n",
         thing, NULL, SEND_SRC);
    THINGFREE(thing);
    break;

  case EVENT_FREE: {
    SOCK *sock;

    /* restore link back to real body */
    sock = BaseControlFind(thing);
    if (sock) {
      BaseControlFree(thing, sock);
      BaseControlAlloc(sock->sHomeThing, sock);
    }
  } break;

  }
  return FALSE;
}


/*  EFFECT_SENSELIFE      */
/* see hidden creatures */
EFFECTPROC(EffectSenseLife) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        if( duration > 0 )
          affect->aDuration += duration;
        else
          affect->aDuration = -1;
        continue;
      }
      if (BIT(Character(tarIndex->iThing[i])->cAffectFlag, AF_SENSELIFE)) {
        SendAction("^cNothing new seems to happen.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
        return FALSE;
      }
      /* Let people know what happened */
      SendAction("^cYou start to see the world from a loftier plane.\n",
        thing, tarIndex->iThing[i], SEND_DST);
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_SENSELIFE);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour vision returns to normal.\n",
         thing, NULL, SEND_SRC);
  case EVENT_UNAPPLY:
  case EVENT_FREE:
    BITCLR(Character(thing)->cAffectFlag, AF_SENSELIFE);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_SENSELIFE);
    break;

  }
  return FALSE;
}


/* See Invisible things */
EFFECTPROC(EffectSeeinvisible) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        if( duration > 0 )
          affect->aDuration += duration;
        else
          affect->aDuration = -1;
        continue;
      }
      if (BIT(Character(tarIndex->iThing[i])->cAffectFlag, AF_SEEINVIS)) {
        SendAction("^cNothing new seems to happen.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
        return FALSE;
      }
      /* Let people know what happened */
      SendAction("^cYou start to see the world from a higher plane.\n",
        thing, tarIndex->iThing[i], SEND_DST);
      /* AffectCreate */
      AffectCreate(tarIndex->iThing[i], AFFECT_NONE, 0, duration, effectNum);
      BITSET(Character(tarIndex->iThing[i])->cAffectFlag, AF_SEEINVIS);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour vision returns to normal.\n",
         thing, NULL, SEND_SRC);
  case EVENT_UNAPPLY:
  case EVENT_FREE:
    BITCLR(Character(thing)->cAffectFlag, AF_SEEINVIS);
    break;

  case EVENT_APPLY:
    BITSET(Character(thing)->cAffectFlag, AF_SEEINVIS);
    break;

  }
  return FALSE;
}


/* Lowers actual armor class - GOOD protect spell */
EFFECTPROC(EffectLuckshield) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou create a force-field around $N.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);
      SendAction("^c$n creates a force-field around you.\n",
        thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n creates a force-field around $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      AffectCreate(tarIndex->iThing[i], AFFECT_ARMOR, data/10, duration, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cThe forcefield around you abruptly disappears.\n",
         thing, NULL, SEND_SRC);
    SendAction("^cThe forcefield around $n abruptly disappears.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  } break;

  }
  return FALSE;
}

/* Identify an object */
EFFECTPROC(EffectIdentify) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD         i;
    LWORD         fieldNum;
    LWORD         apply;
    OBJTEMPLATE   *object;
    BYTE          buf[512];
    WORD          word;

    for (i=0; i<tarIndex->iNum; i++) {
      if (!tarIndex->iThing[i]
      || tarIndex->iThing[i]->tType!=TTYPE_OBJ
      || Number(0,100)>data) {
        SendAction("^cYou fail to identify $N.\n",
          thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);
        continue;
      }

      SendAction("^cYou identify $N.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);

      object = Obj(tarIndex->iThing[i])->oTemplate;

      /* first line, # Name & Type */
      SendAction("^gName:^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(object->oSDesc->sText, thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G] ^gType:^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(TYPESPRINTF(buf, object->oType, oTypeList, 512), thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n^gKeywords:^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(object->oKey->sText, thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n^gAct: ^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(FlagSprintf(buf, Obj(tarIndex->iThing[i])->oAct, oActList, ' ', 512), thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n^gWear: ^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(TYPESPRINTF(buf, object->oWear, wearList, 512), thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n", thing,NULL, SEND_SRC|SEND_VISIBLE);

      sprintf(buf, "^gWeight:^G[^c%hd^G] ", object->oWeight);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "^gValue:^G[^c%ld^G] ", object->oValue);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "^gRent:^G[^c%ld^G]\n", object->oRent);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);

      SendAction("^gType-Specific Information\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      for (fieldNum=0;
              fieldNum<OBJECT_MAX_FIELD
           && ObjectGetFieldStr(object->oType, &Obj(tarIndex->iThing[i])->oDetail, fieldNum, buf, 512);
           fieldNum++) {
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      }

      for (apply=0; apply<OBJECT_MAX_APPLY; apply++) {
        if (object->oApply[apply].aType) {
          sprintf(buf,"^gApply%ld:^G[^c", apply);
          SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
          SendAction(TYPESPRINTF(buf, Obj(tarIndex->iThing[i])->oApply[apply].aType, applyList, 512), thing,NULL, SEND_SRC|SEND_VISIBLE);
          word = Obj(tarIndex->iThing[i])->oApply[apply].aValue;
          sprintf(buf, "^G] ^gValue:^G[^c%hd^G]\n", word);
          SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
        }
      }
    }
  } break;

  }
  return FALSE;
}

/* Identify a person */
EFFECTPROC(EffectStat) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD         i;
    BYTE          buf[512];
    BYTE          buf2[256];
    THING        *statThing;

    for (i=0; i<tarIndex->iNum; i++) {
      if (!tarIndex->iThing[i]
      || tarIndex->iThing[i]->tType<TTYPE_CHARACTER
      || Number(0,100)+Character(tarIndex->iThing[i])->cLevel-20>data) {
        SendAction("^cYou fail to stat $N.\n",
          thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);
        continue;
      }

      statThing=tarIndex->iThing[i];

      SendAction("^cYou stat $N.\n",
        thing, statThing, SEND_SRC|SEND_VISIBLE);

      /* show them the stats */
      SendAction("^gName:^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(statThing->tSDesc->sText, thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      if (statThing->tType == TTYPE_PLR) {
        SendAction("^gClass:^G[", thing,NULL, SEND_SRC|SEND_VISIBLE);
        TYPESPRINTF(buf2, Plr(statThing)->pClass, classList, sizeof(buf2));
        strcat(buf2, "^G]");
        sprintf(buf, "%-12s ", buf2);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
        SendAction("         ^gRace:^G[", thing,NULL, SEND_SRC|SEND_VISIBLE);
        SendAction(TYPESPRINTF(buf, Plr(statThing)->pRace, raceList, sizeof(buf)), thing,NULL, SEND_SRC|SEND_VISIBLE);
        SendAction("^G]\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      }
      sprintf(buf, "^gLevel:^G[^c%4hd^G] ", Character(statThing)->cLevel);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "              ^gAura:^G[^c%4hd^G]", Character(statThing)->cAura);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("               ^gSex:^G[", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(TYPESPRINTF(buf, Character(statThing)->cSex, sexList, sizeof(buf)), thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n", thing,NULL, SEND_SRC|SEND_VISIBLE);

      sprintf(buf, "^gArmor:^G[^c%4hd^G] ", Character(statThing)->cArmor);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "              ^gHitBonus:^G[^c%4ld^G] ", CharGetHitBonus(statThing, Character(statThing)->cWeapon));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "          ^gDamBonus:^G[^c%4ld^G]\n", CharGetDamBonus(statThing, Character(statThing)->cWeapon));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "^gMoney:^G[^c%ld^G] ", Character(statThing)->cMoney);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf, "^gExp:^G[^c%ld^G]\n", Character(statThing)->cExp);
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      if (statThing->tType == TTYPE_PLR) {
        SendAction("\n^cAbilities:\n^g", thing,NULL, SEND_SRC|SEND_VISIBLE);
        sprintf(buf, "^gStr: ^G[^c%4hd^G]                ^gHP:^G[^c%5ld^G/^c%5ld^g]          ^gHunger:^G[^c%hd^G]\n",
                Plr(statThing)->pStr,
                Character(statThing)->cHitP,
                CharGetHitPMax(statThing),
                Plr(statThing)->pHunger);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
        sprintf(buf, "^gDex: ^G[^c%4hd^G]                ^gMP:^G[^c%5ld^G/^c%5ld^g]          ^gThirst:^G[^c%hd^G]\n",
                Plr(statThing)->pDex,
                Character(statThing)->cMoveP,
                CharGetMovePMax(statThing),
                Plr(statThing)->pThirst);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
        sprintf(buf, "^gCon: ^G[^c%4hd^G]                ^gPP:^G[^c%5ld^G/^c%5ld^g]          ^gIntox:^G[^c%hd^G]\n",
                Plr(statThing)->pCon,
                Character(statThing)->cPowerP,
                CharGetPowerPMax(statThing),
                Plr(statThing)->pIntox);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
        sprintf(buf, "^gWis: ^G[^c%4hd^G]\n", Plr(statThing)->pWis);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
        sprintf(buf, "^gInt: ^G[^c%4hd^G]                ^gPractices:^G[^c%ld^G]\n",
                Plr(statThing)->pInt,
                Plr(statThing)->pPractice);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      } else {
        sprintf(buf, "^gHP:^G[^c%5ld^G^G]  ^gMP:^G[^c%5ld^G]  ^gPP:^G[^c%5ld^G]\n",
                Character(statThing)->cHitP,
                Character(statThing)->cMoveP,
                Character(statThing)->cPowerP);
        SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      }
      SendAction("\nResistances:\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "^gPuncture  [^c%3hd^G]",
              CharGetResist(statThing, FD_PUNCTURE));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "            ^gSlash [^c%3hd^G]",
                CharGetResist(statThing, FD_SLASH));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "               ^gConcussive[^c%3hd^G]\n",
                CharGetResist(statThing, FD_CONCUSSIVE));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "^gHeat      [^c%3hd^G]",
                CharGetResist(statThing, FD_HEAT));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "            ^gEMR   [^c%3hd^G]",
                CharGetResist(statThing, FD_EMR));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "               ^gLaser     [^c%3hd^G]\n",
                CharGetResist(statThing, FD_LASER));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "^gPsychic   [^c%3hd^G]",
                CharGetResist(statThing, FD_PSYCHIC));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "            ^gAcid  [^c%3hd^G]",
                CharGetResist(statThing, FD_ACID));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      sprintf(buf,
              "               ^gPoison    [^c%3hd^G]\n",
                CharGetResist(statThing, FD_POISON));
      SendAction(buf, thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("\n^gAffected by: ^G[^c", thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction(FlagSprintf(buf, Character(statThing)->cAffectFlag, affectList, ' ', 512), thing,NULL, SEND_SRC|SEND_VISIBLE);
      SendAction("^G]\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
    }
  } break;

  }
  return FALSE;
}

/* Makes it easier for you to hit */
EFFECTPROC(EffectLuckyhits) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }
      if (affect) AffectFree(tarIndex->iThing[i], affect);
      /* Let people know what happened */
      SendAction("^cYou improve the luck of $N.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n makes you luckier.\n",
        thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n improves the luck of $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      AffectCreate(tarIndex->iThing[i], AFFECT_HITROLL, data/5, duration, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou dont feel quite so lucky anymore.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n stops feeling quite so lucky.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  } break;

  }
  return FALSE;
}

/* Makes it easier for you to hit */
EFFECTPROC(EffectLuckydamage) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD i;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou make $N seem a little more ominous somehow.\n",
        thing, tarIndex->iThing[i], SEND_SRC|SEND_VISIBLE);
      SendAction("^c$n makes you a little more ominous somehow.\n",
        thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      SendAction("^c$n makes $N seem a little more ominous somehow.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
      AffectCreate(tarIndex->iThing[i], AFFECT_DAMROLL, data/25+1, duration, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou dont feel quite so ominous anymore.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n stops feeling quite so ominous.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  } break;

  }
  return FALSE;
}

  /* Pyrokinetic */
/* typical sort of attack spell - wimpy */
EFFECTPROC(EffectBurningfist) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 2;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_HEAT);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 4) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2+20))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_HEAT, effectNum, "burning fist");
    }
  } break;

  }
  return FALSE;
}


/* typical sort of attack spell - one grade better */
EFFECTPROC(EffectFlamestrike) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 20;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_HEAT);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 5) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data/2))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_HEAT, effectNum, "pillar of flame");
    }
  } break;

  }
  return FALSE;
}


/* typical sort of attack spell - powerfull */
EFFECTPROC(EffectIncinerate) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {
  case EVENT_EFFECT: {
    LWORD i;
    LWORD damDice  = (data/10) + 1;
    LWORD damBonus = 40;
    LWORD damage;
    LWORD resist;

    for (i=0; i<tarIndex->iNum; i++) {
      FightStart(thing, tarIndex->iThing[i], 1, NULL);
      resist = CharGetResist(tarIndex->iThing[i], FD_HEAT);
      damDice = MAXV(damDice/2+1, damDice - resist/15);
      damage = Dice(damDice, 6) + damBonus;
      if (CharDodgeCheck(tarIndex->iThing[i], -data))
        damage >>= 1;
      FightDamage(thing, damage, 1, FD_HEAT, effectNum, "blast of heat");
    }
  } break;

  }
  return FALSE;
}

/* EFFECT_IGNITE         */
/* This should ignite any explosives on a character
   the chance of which is affected by RESIST_HEAT */

/* higher resistance towards kinetic energy type weapons */
EFFECTPROC(EffectHeatshield) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD   i;
    LWORD   bonus;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    for (i=0; i<tarIndex->iNum; i++) {
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }
      /* Let people know what happened */
      SendAction("^cYou feel a little cooler.\n", tarIndex->iThing[i],NULL, SEND_SRC|SEND_VISIBLE);
      /* AffectCreate */
      bonus = Dice(2,10) + Number(0,data/30);
      AffectCreate(tarIndex->iThing[i], AFFECT_RES_HEAT, bonus, duration, effectNum);
    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT: {
    /* Let people know what happened */
    SendAction("^cYou feel a little warmer.\n", thing,NULL, SEND_SRC|SEND_VISIBLE);
  } break;

  }
  return FALSE;
}


/* Create a blade */
EFFECTPROC(EffectFireblade) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD  i;
    THING  *item;
    AFFECT *affect;
    LWORD  duration = data/10+5;
    LWORD  damDieSize = data/30+6;

    for (i=0; i<tarIndex->iNum; i++) {
      /* Check if they have a pocket allready */
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration += duration;
        continue;
      }

      /* Create object */
      item = ObjectCreate(fireBladeTemplate, tarIndex->iThing[i]);
      OBJECTSETFIELD(item, OF_WEAPON_DIENUM,  1);
      OBJECTSETFIELD(item, OF_WEAPON_DIESIZE, damDieSize);
      OBJECTSETFIELD(item, OF_WEAPON_TYPE, TYPEFIND("FIREBLADE", weaponList) );

      /* If we can equip it */
      if (InvEquip(thing, item, NULL)) {
        /* Let people know what happened */
        SendAction("^cYou create a blade of fire\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^c$n creates a blade of fire for you\n",
          thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        SendAction("^c$n creates a blade of fire for $N\n",
          thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, data, duration, effectNum);
      } else {
        /* If we cant equip it turf the object */
        /* Let caster know their hand was full */
        SendAction("^cYou try to create a blade of fire for $N but have no place to put it.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        THINGFREE(item);
      }

    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour blade of fire suddenly winks out of existence.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n's blade of fire suddenly winks out of existence.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    /* fall thru to free */
  case EVENT_FREE: {
    THING *i;
    THING *next;

    /* search for it - turf em as we find em, should only be one */
    for (i = thing->tContain; i; i=next) {
      next = i->tNext;
      if (i->tType==TTYPE_OBJ && Obj(i)->oTemplate == fireBladeTemplate) {
        /* its history */
        THINGFREE(i);
      }
    }
  } break;

  case EVENT_READ: {
    THING *item;
    LWORD  damDieSize = affect->aValue/30+6;

    AffectReadPrimitive(thing, affect, file);
    item = ObjectCreate(fireBladeTemplate, thing);
    OBJECTSETFIELD(item, OF_WEAPON_DIESIZE, damDieSize);
    OBJECTSETFIELD(item, OF_WEAPON_TYPE, TYPEFIND("FIREBLADE", weaponList) );
    InvEquip(thing, item, NULL);
  } break;

  case EVENT_WRITE: {
    AffectWritePrimitive(thing, affect, file);
  } break;

  }
  return TRUE;
}



EFFECTPROC(EffectFireshield) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD  i;
    AFFECT *affect;
    THING  *item;
    LWORD duration = (data>=0) ? data/10+20 : -1;
    LWORD  armor = data/30+3;
    LWORD  rheat = data/10;

    for (i=0; i<tarIndex->iNum; i++) {
      /* Check if they have a pocket allready */
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        affect->aDuration = duration;
        continue;
      }

      /* Create object */
      item = ObjectCreate(fireShieldTemplate, tarIndex->iThing[i]);
      OBJECTSETFIELD(item, OF_ARMOR_ARMOR, armor);
      OBJECTSETFIELD(item, OF_ARMOR_RHEAT, rheat);

      /* If we can equip it */
      if (InvEquip(thing, item, NULL)) {
        /* Let people know what happened */
        SendAction("^cYou create a shield of fire\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^c$n creates a shield of fire for you\n",
          thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        SendAction("^c$n creates a shield for $N\n",
          thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, data, duration, effectNum);
      } else {
        /* If we cant equip it turf the object */
        /* Let caster know their hand was full */
        SendAction("^cYou try to create a shield of fire for $N but have no place to put it.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        THINGFREE(item);
      }

    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour shield of fire winks out of existence.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n's shield of fire suddenly winks out of existence.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
    /* then fall thru to free it */
  case EVENT_FREE: {
    THING *i;
    THING *next;

    /* search for it - turf em as we find em, should only be one */
    for (i = thing->tContain; i; i=next) {
      next = i->tNext;
      if (i->tType==TTYPE_OBJ && Obj(i)->oTemplate == fireShieldTemplate) {
        /* Pocket is history */
        THINGFREE(i);
      }
    }
  } break;

  case EVENT_READ: {
    THING *item;
    LWORD  armor = affect->aValue/30+3;
    LWORD  rheat = affect->aValue/10;

    AffectReadPrimitive(thing, affect, file);
    item = ObjectCreate(fireShieldTemplate, thing);
    OBJECTSETFIELD(item, OF_ARMOR_ARMOR, armor);
    OBJECTSETFIELD(item, OF_ARMOR_RHEAT, rheat);
    InvEquip(thing, item, NULL);
  } break;

  case EVENT_WRITE: {
    AffectWritePrimitive(thing, affect, file);
  } break;


  }
  return TRUE;
}



EFFECTPROC(EffectFirearmor) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  case EVENT_EFFECT: {
    LWORD  i;
    THING  *item;
    AFFECT *affect;
    LWORD  duration = data/10+10;
    LWORD  armor = data/25+3;
    LWORD  rheat = data/10;

    for (i=0; i<tarIndex->iNum; i++) {
      /* Check if they have a pocket allready */
      affect = AffectFind(tarIndex->iThing[i], effectNum);
      if (affect) {
        /* strengthen duration */
        if (duration > affect->aDuration)
          affect->aDuration = duration;
        continue;
      }

      /* Create object */
      item = ObjectCreate(fireArmorTemplate, tarIndex->iThing[i]);
      OBJECTSETFIELD(item, OF_ARMOR_ARMOR, armor);
      OBJECTSETFIELD(item, OF_ARMOR_RHEAT, rheat);

      /* If we can equip it */
      if (InvEquip(thing, item, NULL)) {
        /* Let people know what happened */
        SendAction("^cYou create armor of fire\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        SendAction("^c$n creates armor of fire for you\n",
          thing, tarIndex->iThing[i], SEND_DST |SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        SendAction("^c$n creates armor of fire for $N\n",
          thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_AUDIBLE|SEND_CAPFIRST);
        AffectCreate(tarIndex->iThing[i], AFFECT_NONE, data, duration, effectNum);
      } else {
        /* If we cant equip it turf the object */
        /* Let caster know their hand was full */
        SendAction("^cYou try to create armor of fire for $N but have no place to put it.\n",
          thing, tarIndex->iThing[i], SEND_SRC);
        THINGFREE(item);
      }

    }
  } break;

  /* called separately as each thing expires, tarIndex ignored */
  case EVENT_UNEFFECT:
    /* Let people know what happened */
    SendAction("^cYour armor of fire winks out of existence.\n",
         thing, NULL, SEND_SRC);
    SendAction("^c$n's armor of fire suddenly winks out of existence.\n",
         thing, NULL, SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
  case EVENT_FREE: {
    THING *i;
    THING *next;

    /* search for it - turf em as we find em, should only be one */
    for (i = thing->tContain; i; i=next) {
      next = i->tNext;
      if (i->tType==TTYPE_OBJ && Obj(i)->oTemplate == fireArmorTemplate) {
        /* its is history */
        THINGFREE(i);
      }
    }
  } break;

  case EVENT_READ: {
    THING *item;
    LWORD  armor = affect->aValue/25+3;
    LWORD  rheat = affect->aValue/10;

    AffectReadPrimitive(thing, affect, file);
    item = ObjectCreate(fireArmorTemplate, thing);
    OBJECTSETFIELD(item, OF_ARMOR_ARMOR, armor);
    OBJECTSETFIELD(item, OF_ARMOR_RHEAT, rheat);
    InvEquip(thing, item, NULL);
  } break;

  case EVENT_WRITE: {
    AffectWritePrimitive(thing, affect, file);
  } break;


  }
  return TRUE;
}




/* EFFECT_REVITALIZE - more power points */
EFFECTPROC(EffectRevitalize) { /* (thing, cmd, data, tarIndex, effectNum, event) */
  switch (event) {

  /* Works on tarIndex */
  case EVENT_EFFECT: {
    LWORD i;
    LWORD powerMax;
    LWORD bonus;
    for (i=0; i<tarIndex->iNum; i++) {
      /* Let people know what happened */
      SendAction("^cYou revitalize $n.\n",
        thing, tarIndex->iThing[i], SEND_SRC);
      SendAction("^c$n revitalizes you.\n",
        thing, tarIndex->iThing[i], SEND_DST|SEND_VISIBLE|SEND_CAPFIRST);
      SendAction("^c$n revitalizes $N.\n",
        thing, tarIndex->iThing[i], SEND_ROOM|SEND_VISIBLE|SEND_CAPFIRST);
      /* heal them */
      powerMax = CharGetPowerPMax(tarIndex->iThing[i]);
      bonus = Dice(data/20+1,4) + data/10 + 1;
      Character(tarIndex->iThing[i])->cPowerP += bonus;
      MAXSET(Character(tarIndex->iThing[i])->cPowerP, powerMax);
    }
  } break;

  }
  return FALSE;
}

