/* Crimson2 Mud Server
 * All source written/copyright Ryan Haksi 1995 *
 * This source code is proprietary. Use in whole or in part without
 * explicity permission by the author is strictly prohibited
 *
 * Current email address(es): cryogen@infoserve.net
 * Phone number: (604) 591-5295
 *
 * C4 Script Language written/copyright Cam Lesiuk 1995
 * Email: clesiuk@engr.uvic.ca
 *
 * Portions written/copyright Richard Day 2000
 * Current Email: thelvyn@home.net
 * permanent email: atragorn@netscape.net
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>

#include "crimson2.h"
#include "log.h"
#include "str.h"
#include "macro.h"
#include "ini.h"

/* more or less standard windows-type ini file, sans groups:
setting=value
  setting=value ; your comment here
; commented line
# commented line
setting one=blah ; invalid will be interpreted as setting=one

As with windows an INI file > 64K will cause a crash....
white space is mostly ignored
# and ; are both reserved for comments
dont use lines > 256 chars in length, as weird stuff occurs...
*/

/* result should be >= 256 char buffer */
void INIRead(BYTE *fileName, BYTE *setting, BYTE *defResult, BYTE *result) {
  IniGetString( NULL, setting, defResult, result, 512, fileName );
}

LWORD INILWordRead(BYTE *fileName, BYTE *setting, LWORD defResult) {  
  BYTE  buf[512];
  if( IniGetString( NULL, setting, NULL, buf, 512, fileName ) != -1 ) {
    return atol( buf );
  }
  return defResult;
}

void INIWrite(BYTE *fileName, BYTE *setting, BYTE *result) {
  #define INI_MAX_SIZE 65535

  FILE       *iniFile;
  BYTE        iniSetting[256];
  LWORD       iniSettingLen;
  BYTE        iniValue[256];
  BYTE        iniBuf[INI_MAX_SIZE];  
  struct stat iniStat;
  LWORD       i;
  LWORD       iFound;
  LWORD       iMax;


  stat(fileName, &iniStat);
  if (iniStat.st_size >= INI_MAX_SIZE) {
    sprintf(iniBuf, "Unable to open INI file %s. (file to large!)\n", fileName);
    Log(LOG_ERROR, iniBuf);
    return;
  }
  
  iniFile = fopen(fileName, "rb+");
  if (!iniFile) {
    result[0]='\0';
    sprintf(iniBuf, "Unable to open INI file %s.\n", fileName);
    Log(LOG_ERROR, iniBuf);
    return;
  } 
  fread(iniBuf, iniStat.st_size, 1, iniFile);
  iniBuf[iniStat.st_size] = '\0'; /* ensure termination */
  iniSettingLen = strlen(iniSetting);
  
  /* room for setting\t= value */
  iMax = iniStat.st_size - strlen(iniSetting) - strlen(iniValue) - 3;
  iFound = -1;
  for (i=0;i<iMax;i++){
    if (!STRNICMP(iniSetting, iniBuf+i, iniSettingLen)) {
      iFound = i;
      break;
    }
  }
  for (;iniBuf[i]!='\0' && iniBuf[i]!='#' && iniBuf[i]!=';' && iniBuf[i]!='\n'; i++);
  if (iFound == -1)
    fseek(iniFile, 0, SEEK_END);
  else 
    fseek(iniFile, iFound, SEEK_SET);
  fprintf(iniFile, "\t%-15s=%s%s", iniSetting, iniValue, iniBuf+i);
  fclose(iniFile);
}

/* will return starting position of the string or -1 for error */
LWORD IniFindString( FILE* file,      /* an open file */
                     const BYTE* find,      /* string to find */
                     const LWORD flag,      /* flags to use */
                     const BYTE* not  /* chars to end the search in failure on */                
                      ) {
  
  BYTE   buf[8192];
  WORD   ch = 0;
  WORD   i = 0;  
  LWORD  pos = 0;
  WORD   size;
  WORD   nsz = 0;
  BYTE*  str = NULL;
  BYTE   dar = BIT( flag, INI_DONT_ALLOW_RETURN );
  
  if( !file || !find || !find[0] ) {    
    return -1;
  }
  size = strlen( find ) + 1;
  if( size > 8192 ) { /* cant handle larger then this, shouldnt be this damn big anyway */    
    Log(LOG_ERROR, "IniFindString: String too large! > 8192 \n" );
    return -1;
  }
  if( not && not[0] ) {
    nsz = strlen( not );
  }
  if( ferror( file ) ) { /* clear any previous error */
    clearerr( file );
  }
  if( BIT( flag, INI_RESET_ON_SEARCH ) ) {
    fseek( file, 0, SEEK_SET ); /* seek to begining of file to start search */
    if( feof( file ) || ferror( file ) ) {
      return -1;
    }
  }
  while( 1 ) {
    pos = ftell( file );
    ch = fgetc( file );
    if( feof( file ) ) { /* return if error after clearing */
      if( ferror( file ) ) {
        clearerr( file );
      }
      return -1;
    }
    switch( ch ) {
    case '\n':
      if( dar ) /* if not allowing linefeeds on this search */
        return -1;
      break;
    case ';':
    case '#':
      if( dar ) /* if not allowing linefeeds on this search */
        return -1; /* comment so implicit line feed */
      while( ch != '\n' && !feof( file ) ) {
        ch = fgetc( file );
      }  
      continue;
    default:
      break;
    }
    if( tolower(ch) == tolower(find[0]) ) {
      if( !fseek( file, pos, SEEK_SET ) ) {
        str = fgets( buf, size, file );
        if( !str ) {
          if( ferror( file ) ) {
            clearerr( file );
          }
          fseek( file, 0, SEEK_SET );
          return -1;
        }
        buf[size] = 0;
        if( !STRICMP( find, buf ) ) { /* found it if true */          
          return pos;
        }
      }
    } else if( nsz ) {
      for( i = 0; i < nsz; i++ ) {
        if( ch == not[i] ) {          
          return -1;
        }
      }
    }
  }
  return -1; /* never get here */ 
}

LWORD IniGetString( const BYTE* fsection,  /* section name like [Config] or something */
                    const BYTE* fkey,      /* keyname usually blah = blah or something like that*/
                    const BYTE* fdefault,  /* default if not found */
                    BYTE*       fbuffer,   /* destination buffer */
                    const LWORD fsize,     /* size of destination buffer */
                    const BYTE* fileName   /* initialization file name */
                    ) {
  BYTE  section = FALSE; /* section found */
  FILE* file; /* file stream pointer */
  BYTE  buffer[8192]; /* internal buffer */ 
  BYTE  not[5]; /* chars to fail on */
  WORD  ch;     /* byte to check/add to return for fgetc */
  BYTE  skipping = TRUE; /* skip preceeding whitespace till we hit a valid char */
  LWORD i = 0;
  
  if( !fbuffer ) { /* should we allocate if needed ? have to think about it */
      return -1;
  }

  if( fdefault && fdefault[0] ) {
    strcpy( fbuffer, fdefault );
  }
  if( !fkey || !fkey[0] || !fileName || !fileName[0] ) {
    return -1;
  }
  file = fopen( fileName , "rt" );
  if( !file ) {
    return -1;
  }
  if( fsection && fsection[0] ) { /* Section is not required but get from it if they pass one in */
    sprintf( buffer,"[%s]", fsection );
    if( IniFindString( file, buffer, INI_RESET_ON_SEARCH, NULL ) != -1 ) {
      section = TRUE;
    } else {
      fclose( file );
      return -1; /* they asked for a section and not found oops return failure */
    }
  }
  if( section ) {
    not[0] = '[';
    not[1] = 0;
  } else {
    not[0] = 0;
  }
  if( IniFindString( file, fkey, 0, not ) != -1 &&
      IniFindString( file, "=", INI_DONT_ALLOW_RETURN, not ) != -1 ) {
    for( i = 0; i < fsize && !feof( file ); i++ ) {
      ch = fgetc( file );
      if( ch == ';' || ch == '#' || ch == '\n' ) {        
        break;
      }
      if( isspace( ch ) && skipping ) {
        i--;
        continue;
      }
      skipping = FALSE; /* we got a char so done skipping whitespace */
      fbuffer[i] = ch;
    }
    fclose( file );
    fbuffer[i] = 0;
    if( skipping ) {      
      return -1;
    }
    return strlen( fbuffer );
  }
  fclose( file );
  return -1;
}