/* Crimson2799 Help file to HTML document converter
 * Copyright (c) B. Cameron Lesiuk, 2000
 *
 * This software is part of the Crimson2799 distribution.
 * Consequently, distribution and use of this software 
 * product falls under the same license as the Crimson2799 
 * source.
 * 
 */
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<unistd.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<errno.h>

typedef struct COLOURtag {
  char code;
  char *html;
} COLOUR;

typedef struct LINKtag {
  char key[80];
  char link[156];
} LINK;

LINK *LinkIndex;
long LinkIndexSize;

COLOUR cBank[] = {
 { 'w',"FFFFFF" },
 { 'a',"B0B0B0" },
 { 'r',"FF0000" },
 { 'y',"FFFF00" },
 { 'g',"00FF00" },
 { 'b',"0000FF" },
 { 'c',"00FFFF" },
 { 'p',"FF00FF" },
 { 'v',NULL },

 { 'W',"404040" },
 { 'A',"808080" },
 { 'R',"800000" },
 { 'Y',"808000" },
 { 'G',"008000" },
 { 'B',"000080" },
 { 'C',"008080" },
 { 'P',"800080" },
 { 'V',NULL },
 { 0x00,NULL }
};


int stricmp(char *s1,char *s2) {
  int i;
  char c1,c2;
  i=0;
  while(s1[i]&&s2[i]) {
    c1=s1[i];
    if ((c1>='A')&&(c1<='Z'))
      c1=c1-'A'+'a';
    c2=s2[i];
    if ((c2>='A')&&(c2<='Z'))
      c2=c2-'A'+'a';
    if (c1=='_')
      c1=127;
    if (c2=='_')
      c2=127;
    if (c1<c2)
      return -1;
    if (c1>c2)
      return 1;
    i++;
  }
  if (s1[i])
    return 2;
  if (s2[i])
    return -2;
  return 0;
}

/* This proc sorts the index, starting at index #start and going
 * no further than index #end. No boundary checks are made so
 * make sure your calling function knows what it's doing */
void sort_index(LINK *Index,long start,long end) {
  LINK tmp;
  long i,j;
  if (start==end)
    return;
  if (end<start) {
    i=start;
    start=end;
    end=i;
  }
  for (i=start;i<end;i++) {
    for (j=i+1;j<=end;j++) {
      if (stricmp(Index[j].key,Index[i].key)<0) {
        /* swap these two */
        memcpy(&tmp,&(Index[i]),sizeof(LINK));
        memcpy(&(Index[i]),&(Index[j]),sizeof(LINK));
        memcpy(&(Index[j]),&tmp,sizeof(LINK));
      }
    }
  }
}

/* This proc takes a string and prints it out in nicely formatted html, 
 * complete with colours based on ^<color> codes */
void write_formatted(int outfile, char *buf) {
  int  colour=-1;
  int  i,j,k,l;
  int  len;
  char tmp[512];
  char word[512];

  if (!buf) 
    return;
  len=strlen(buf);
  for(i=0;i<len;i++) {
    if (buf[i]=='^') {
      /* colour change - first close current colour if there is one */
      if (colour>=0) {
        sprintf(tmp,"</font>");
        write(outfile,tmp,strlen(tmp));
      }
      i++;
      /* find new colour */
      for (j=0;(cBank[j].code)&&(cBank[j].code!=buf[i]);j++);
      /* now start new colour, if possible */
      if (cBank[j].html) {
        colour=j;
        sprintf(tmp,"<font color=\"%s\">",cBank[colour].html);
        write(outfile,tmp,strlen(tmp));
      } else {
        colour=-1;
      }
    } else if (buf[i]=='\t') {
      /* throw in a tab - 8 spaces */
      sprintf(tmp,"        ");
      write(outfile,tmp,strlen(tmp));
    } else if (buf[i]==0x0A) {
      /* throw in a break */
      /* Actually, don't - the <pre> will do this for us by virtue of doing a \n */
      /* sprintf(tmp,"<br>\n");*/
      sprintf(tmp,"\n");
      write(outfile,tmp,strlen(tmp));
    } else if (buf[i]<32) {
      /* simply ignore this character */
    } else if (buf[i]=='<') {
      sprintf(tmp,"&lt;");
      write(outfile,tmp,strlen(tmp));
    } else if (buf[i]=='>') {
      sprintf(tmp,"&gt;");
      write(outfile,tmp,strlen(tmp));
    } else {
      /* write this character to the output */
      /* first, lets extract our "word". We do this so we can try and pick 
       * out links -- just read down a bit. */
      for (j=0;
           (buf[i+j]>=32)&&
	    ( ((buf[i+j]>='a')&&(buf[i+j]<='z')) || 
	      ((buf[i+j]>='A')&&(buf[i+j]<='Z')) || 
	      ((buf[i+j]>='0')&&(buf[i+j]<='9')) || 
	      (buf[i+j]=='.') ||
	      (buf[i+j]=='_') ||
	      (buf[i+j]=='%') ||
	      (buf[i+j]=='-')
	    );
           j++) 
        word[j]=buf[i+j];
      word[j]=0;
      if (j) {
        /* print this item as a complete word */
	/* ah! but let's check to see if it's a reference to something else first! */
	for(k=0;LinkIndex[k].key[0];k++) {
	  if (LinkIndex[k].key[0]=='_') {
	    l= stricmp(word,&(LinkIndex[k].key[1]));
	    if (l==0) {
              break;
	    }
	  }
	  /* note even if we start with a '_', we still do a normal check */
	  l=stricmp(word,LinkIndex[k].key);
	  if (l==0) {
            break;
	  }
	}
	if (LinkIndex[k].key[0]) {
	  /* We found it! This sucker's a link!!! */
	  sprintf(tmp,"<a href=\"%s\">%s</a>",LinkIndex[k].link,word);
          write(outfile,tmp,strlen(tmp));
	} else {
	  /* nope, didn't find it. */
          write(outfile,word,strlen(word));
	}
	i+=j-1;
      } else {
        /* I duno what kind of character this is - just throw it to the file (its probably a space) */
        write(outfile,&(buf[i]),1);
      }
    }
  }
  /* finally, check if we've got to terminate our colour */
  if (colour>=0) {
    sprintf(tmp,"</font>");
    write(outfile,tmp,strlen(tmp));
  }
  /* and that's it. The data has been written! */
}

/* This proc gets the first word from "line" and copies it to "buf". 
 * It returns a pointer to the next word in the line. This proc returns
 * NULL if it's reached the end of the line. Note word may still be valid. */
char *get_word(char *word,char *line) {
  char *p;
  if (!word)
    return NULL;
  word[0]=0;
  if (!line) 
    return NULL;
  p=line;
  /* strip initial white space */
  while((*p)&&((*p==' ')||(*p<32))) {
    p++;
  }
  while((*p)&&(*p!=' ')&&(*p>=32)) {
    *word=*p;
    word++;
    p++;
  }
  *word=0;
  /* advance p past whitespace */
  while((*p)&&((*p==' ')||(*p<32))) {
    p++;
  }
  if (*p)
    return p;
  return NULL;
}

/* this proc reads in line(s) until a ~ is hit. It allocs the buffer
 * required and returns a pointer to said buffer, or NULL on error.  
 * The buffer returned is null-terminated and will include all characters
 * including CRLF's. */
char *get_data(int file) {
  char *buf;
  char *bufPtr;
  int bufSize=1024;
  char tmp[512];
  int rc;

  /* initial alloc for buffer */
  buf=malloc(bufSize);
  if (!buf)
    return NULL;
  /* let's read in this line now. I do this one char at a time.. very
   * inefficient, I know, but I don't expect this program to be run much. */
  bufPtr=buf;
  *bufPtr=0; /* terminate our string- ALWAYS leave it terminated. */
  while(1) {
    rc=read(file,tmp,1);
    if (rc<0) {
      if (bufPtr==buf) {
        /* no data read yet! */
        free(buf);
        buf=NULL;
      } 
      return buf;
    }
    if (!rc) {
      /* reached eof */
      if (buf[0])
        return buf;
      free(buf);
      return NULL;
    }
    if (tmp[0]=='~') {
      /* we've found our end of line. */
      /* read next CRLF and exit */
      /* first, we need to get past everything to get to our CRLF */
      do {
        rc=read(file,tmp,1);
      } while((tmp[0]!=0x0A)&&(tmp[0]!=0x0D)&&(rc>0));
      /* now finish reading CRLF */
      do {
        rc=read(file,tmp,1);
      } while(((tmp[0]==0x0A)||(tmp[0]==0x0D))&&(rc>0));
      lseek(file,-1,SEEK_CUR);
      /* now exit */
      return buf;
    }
    *bufPtr=tmp[0];
    bufPtr++;
    *bufPtr=0;
    if ((bufPtr-buf)>=(bufSize-4)) {
      /* we need to realloc this sucker */
      bufSize<<=1;
      buf=realloc(buf,bufSize);
    }
  }
}


int main(int argc,char **argv) {
  int infile,outfile;
  long i;
  char *buf,*p;
  char tmp[512];
  char word[512];
  char ref[512];
  char title[512];
  long index;
  char outfilename[40];

  if (argc<2) {
    printf("USEAGE: %s <Crimson2799 file> [<html output file>]\n",argv[0]);
    exit(1);
  }
  infile=open(argv[1],O_RDONLY);
  if (infile<0) {
    i=errno;
    printf("Could not open input file %s: %s\n",argv[1],strerror(i));
    exit(1);
  }
  if (argc<3) {
    outfile=STDOUT_FILENO;
    outfilename[0]=0;
  } else {
    outfile=open(argv[2],O_CREAT|O_RDWR);
    if (outfile<0) {
      i=errno;
      printf("Could not open output file %s: %s\n",argv[1],strerror(i));
      close(infile);
      exit(1);
    } 
    strcpy(outfilename,argv[2]);
  }

  /* ok we've got both our files open. Let's do this */

  /* initialize our link index */
  LinkIndexSize=512; /* not BYTES, the # of entries!! */
  LinkIndex=malloc(LinkIndexSize*sizeof(LINK));
  if (!LinkIndex) {
    printf("Could not allocate enough memory to perform operation. Aborted.\n");
    close(infile);
    exit(1);
  }
  /* terminate our index */
  LinkIndex[0].key[0]=0;
  LinkIndex[0].link[0]=0;
  index=0;

  /* get first line */
  buf=get_data(infile);
  strcpy(title,buf);
  /* reading body */
  /* printf("Reading body of %s (%s).\n",buf,argv[1]);*/
  sprintf(tmp,"<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<html>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<head>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<meta name=\"GENERATOR\" content=\"HELP2HTML/1.0 [en] (Linux) [Crimson2799]\">\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<meta name=\"Author\" content=\"B. Cameron Lesiuk\">\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<meta name=\"Description\" content=\"%s\">\n",buf);
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<meta name=\"KeyWords\" content=\"%s\">\n",buf);
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<title>Crimson2 Help</title>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"</head>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<body text=\"#B0B0B0\" bgcolor=\"#000000\" link=\"#6060E0\" vlink=\"#5050A0\" alink=\"#0000FF\">\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<h1><center>%s</center></h1>\n",buf);
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<hr>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<p><h2>Contents:</h2><p><p>");
  write(outfile,tmp,strlen(tmp));
  free(buf);
 
  /* We go through the entire file twice: once to pick out the key words (and generate the
   * table of contents) and the second time to fill out the body of the page. */
  /* STEP 1: Table of contents */

  while(1) {

    buf=get_data(infile); /* this should be our keywords */
    if (!buf)
      break;
    /* strip our first # */
    p=buf;
    p=get_word(word,p);
    if (word[0]=='$') {
      /* we're at the end of our file! */
      break;
    } else if (word[0]!='#') {
      printf("ERROR in help file format! Line: %s\n",buf);
      printf("Expected %c(%X) got %c(%X)\n",'#','#',word[0],word[0]);
    }
    p=get_word(word,p);
    strcpy(ref,word);
    while(word[0]) {
      strncpy(LinkIndex[index].key,word,sizeof(LinkIndex[index].key));
      LinkIndex[index].key[sizeof(LinkIndex[index].key)-1]=0; /* just in case */
      /* sprintf(LinkIndex[index].link,"%s#%s",outfilename,ref);*/
      sprintf(LinkIndex[index].link,"#%s",ref);
      LinkIndex[index].link[sizeof(LinkIndex[index].link)-1]=0; /* just in case */
      index++;
      LinkIndex[index].key[0]=0;
      LinkIndex[index].link[0]=0;
      if (index>=(LinkIndexSize-3)) {
        LINK *ttt;
        sprintf(tmp,"          *********** REALLOCATING INDEX ************\n");
        write(STDERR_FILENO,tmp,strlen(tmp));
        /* reallocate our index */
        LinkIndexSize<<=1;
        ttt=realloc(LinkIndex,LinkIndexSize*sizeof(LINK));
	if (ttt) {
	  LinkIndex=ttt;
	} else {
	  printf("Ran out of memory for key word & link index!!\n");
	  close(infile);
	  exit(0);
	}
	sprintf(tmp,"New size %li entries (%li bytes)\n",LinkIndexSize,LinkIndexSize*sizeof(LINK));
        write(STDERR_FILENO,tmp,strlen(tmp));
      }
      p=get_word(word,p);
    }

    free(buf);

    buf=get_data(infile); /* this should be our data */
    if (!buf)
      break;
    free(buf);
  }

  /* we've read in our keys. Let's write out our table of contents */
  sort_index(LinkIndex,0,index-1);
  sprintf(tmp,"<ul>\n");
  write(outfile,tmp,strlen(tmp));
  for (i=0;LinkIndex[i].key[0];i++) {
    sprintf(tmp,"<li>\n");
    write(outfile,tmp,strlen(tmp));
    sprintf(tmp,"<A HREF=\"%s\">%s</A> ",LinkIndex[i].link,LinkIndex[i].key);
    write(outfile,tmp,strlen(tmp));
    sprintf(tmp,"</li>\n");
    write(outfile,tmp,strlen(tmp));
    sprintf(tmp,"<br>\n");
    write(outfile,tmp,strlen(tmp));
  }

  /* sprintf(tmp,"Index size: %li entries (%li bytes) out of %li allocated\n",index,index*sizeof(LINK),LinkIndexSize);
  write(STDERR_FILENO,tmp,strlen(tmp)); */
  sprintf(tmp,"</ul>\n");
  write(outfile,tmp,strlen(tmp));


  /* STEP 2: body */
  lseek(infile,0,SEEK_SET);
  /* read (and ignore) our first line */
  buf=get_data(infile);
  free(buf);

  while(1) {

    sprintf(tmp,"<hr>\n");
    write(outfile,tmp,strlen(tmp));

    buf=get_data(infile); /* this should be our keywords */
    if (!buf)
      break;
    /* strip our first # */
    p=buf;
    p=get_word(word,p);
    if (word[0]=='$') {
      /* we're at the end of our file! */
      break;
    } else if (word[0]!='#') {
      printf("ERROR in help file format! Line: %s\n",buf);
      printf("Expected %c(%X) got %c(%X)\n",'#','#',word[0],word[0]);
    }
    p=get_word(word,p);
    strcpy(ref,word);

    sprintf(tmp,"<center><a name=\"%s\"> <font color=\"505050\">[ </font>",ref);
    write(outfile,tmp,strlen(tmp));
    sprintf(tmp,"<font color=\"FFFFFF\">%s</font>",title);
    write(outfile,tmp,strlen(tmp));
    sprintf(tmp,"<font color=\"505050\"> ]</font></a><br>\n");
    write(outfile,tmp,strlen(tmp));

    sprintf(tmp,"<font color=\"505050\">Keys: [ </font><font color=\"00FFFF\">");
    write(outfile,tmp,strlen(tmp));
    while(word[0]) {
      sprintf(tmp,"<A HREF=\"#%s\">%s</A> ",ref,word);
      write(outfile,tmp,strlen(tmp));
      p=get_word(word,p);
    }
    sprintf(tmp,"</font><font color=\"505050\"> ]</font><p></center>\n");
    write(outfile,tmp,strlen(tmp));

    free(buf);

    buf=get_data(infile); /* this should be our data */
    if (!buf)
      break;
    sprintf(tmp,"<pre>\n"); /* this is to make it equal-spaced font */
    write(outfile,tmp,strlen(tmp));
    write_formatted(outfile,buf);
    sprintf(tmp,"</pre>\n");
    write(outfile,tmp,strlen(tmp));
    free(buf);
  }

  sprintf(tmp,"This page was automatically generated by the help2html utility,\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"part of the Crimson2799 distribution.<br>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"For more info or comments, email ");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"<a href=\"mailto:B. Cameron Lesiuk <wi961@freenet.victoria.bc.ca>\">B. Cameron Lesiuk</a>\n");
  write(outfile,tmp,strlen(tmp));


  sprintf(tmp,"</body>\n");
  write(outfile,tmp,strlen(tmp));
  sprintf(tmp,"</html>\n");
  write(outfile,tmp,strlen(tmp));

  close(infile);
  if (outfile!=STDOUT_FILENO) 
    close(outfile);
  return(0);
}
